package com.xxh.cloud.datalabel.config.framework.modules.biz.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.datalabel.config.framework.modules.base.entity.DataLabelRuleBackUpInfo;
import com.xxh.cloud.datalabel.config.framework.modules.base.entity.DataLabelRuleConfig;
import com.xxh.cloud.datalabel.config.framework.modules.base.service.DataLabelRuleBackUpInfoService;
import com.xxh.cloud.datalabel.config.framework.modules.base.service.DataLabelRuleConfigService;
import com.xxh.cloud.datalabel.config.framework.modules.biz.bean.dto.DataLabelRuleBackUpListDTO;
import com.xxh.cloud.datalabel.config.framework.modules.biz.bean.dto.DataLabelRuleBackUpSaveDTO;
import com.xxh.cloud.datalabel.config.framework.modules.biz.bean.dto.DataLabelRuleBackUpUpdateDTO;
import com.xxh.cloud.datalabel.config.framework.modules.biz.bean.vos.DataLabelRuleBackUpVO;
import com.xxh.cloud.datalabel.config.framework.modules.biz.convert.DataLabelRuleBackUpConvert;
import com.xxh.cloud.datalabel.config.framework.modules.biz.service.DataLabelRuleBackUpBizService;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.log.bean.constant.ModuleTypeEnum;
import com.xxh.cloud.log.bean.constant.ObjectTypeEnum;
import com.xxh.cloud.log.bean.constant.OperationTypeEnum;
import com.xxh.cloud.log.utils.OperationLogUtils;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class DataLabelRuleBackUpBizServiceImpl implements DataLabelRuleBackUpBizService {


    @Autowired
    private DataLabelRuleBackUpInfoService dataLabelRuleBackUpInfoService;

    @Autowired
    private DataLabelRuleConfigService dataLabelRuleConfigService;


    @Override
    public List<DataLabelRuleBackUpVO> list(DataLabelRuleBackUpListDTO params) {
        ValidatorUtils.validateEntity(params);
        return null;
    }

    @Override
    public DataLabelRuleBackUpVO backup(DataLabelRuleBackUpSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        DataLabelRuleConfig dataLabelRuleConfig= dataLabelRuleConfigService.getById(params.getConfigId());
        if(dataLabelRuleConfig==null || YN.NO.getCode().equals(dataLabelRuleConfig.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        DataLabelRuleBackUpInfo dataLabelRuleBackUp= dataLabelRuleBackUpInfoService.getOne(
                new QueryWrapper<DataLabelRuleBackUpInfo>()
                        .select("max(version_no) as versionNo")
                        .eq("config_id",params.getConfigId())
        );

        Integer version;
        if(dataLabelRuleBackUp==null || dataLabelRuleBackUp.getVersionNo()==null){
            version=dataLabelRuleConfig.getVersionNo();
        }else {
            version=dataLabelRuleBackUp.getVersionNo()+1;
        }

        DataLabelRuleBackUpInfo entity=new DataLabelRuleBackUpInfo();
        entity.setConfigId(params.getConfigId());
        entity.setProjectCode(dataLabelRuleConfig.getProjectCode());
        entity.setVersionNo(version);
        entity.setName(dataLabelRuleConfig.getName());
        entity.setType(dataLabelRuleConfig.getType());
        entity.setRemark(dataLabelRuleConfig.getRemark());
        entity.setConfig(dataLabelRuleConfig.getConfig());
        entity.setOrders(dataLabelRuleConfig.getOrders());
        entity.setLabelIds(dataLabelRuleConfig.getLabelIds());
        entity.setInitDateType(dataLabelRuleConfig.getInitDateType());
        entity.setInitDateValue(dataLabelRuleConfig.getInitDateValue());
        entity.setBackUpRemark(params.getBackUpRemark());
        dataLabelRuleBackUpInfoService.save(entity);

        //日志
//        Map<String, Object> logContent=new HashMap<>();
//        logContent.put("ID",entity.getId());
//        logContent.put("名称",entity.getName());
//        OperationLogUtils.addOpLogGen(OperationTypeEnum.OP_Add, ModuleTypeEnum.MT_DataLabel, ObjectTypeEnum.OT_DataLabel_Rule,entity.getId(),logContent);


        return DataLabelRuleBackUpConvert.getDataLabelRuleBackUpVO(entity);
    }

    @Override
    public DataLabelRuleBackUpVO update(DataLabelRuleBackUpUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        DataLabelRuleBackUpInfo entity=dataLabelRuleBackUpInfoService.getById(params.getId());
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setBackUpRemark(params.getBackUpRemark());
        dataLabelRuleBackUpInfoService.save(entity);
        //日志
//        Map<String, Object> logContent=new HashMap<>();
//        logContent.put("ID",entity.getId());
//        logContent.put("名称",entity.getName());
//        OperationLogUtils.addOpLogGen(OperationTypeEnum.OP_Add, ModuleTypeEnum.MT_DataLabel, ObjectTypeEnum.OT_DataLabel_Rule,entity.getId(),logContent);

        return DataLabelRuleBackUpConvert.getDataLabelRuleBackUpVO(entity);
    }

    @Override
    public DataLabelRuleBackUpVO getById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR);
        }
        DataLabelRuleBackUpInfo entity=dataLabelRuleBackUpInfoService.getById(id);
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return DataLabelRuleBackUpConvert.getDataLabelRuleBackUpVO(entity);
    }


}
