package com.xxh.cloud.datalabel.config.framework.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.datalabel.config.framework.modules.base.entity.DataLabelRuleCategoryInfo;
import com.xxh.cloud.datalabel.config.framework.modules.base.entity.DataLabelRuleConfig;
import com.xxh.cloud.datalabel.config.framework.modules.base.service.DataLabelRuleCategoryInfoService;
import com.xxh.cloud.datalabel.config.framework.modules.base.service.DataLabelRuleConfigService;
import com.xxh.cloud.datalabel.config.framework.modules.biz.bean.dto.DataLabelRuleCategorySaveDTO;
import com.xxh.cloud.datalabel.config.framework.modules.biz.bean.dto.DataLabelRuleCategoryTreeDTO;
import com.xxh.cloud.datalabel.config.framework.modules.biz.bean.dto.DataLabelRuleCategoryUpdateDTO;
import com.xxh.cloud.datalabel.config.framework.modules.biz.bean.vos.DataLabelRuleCategoryVO;
import com.xxh.cloud.datalabel.config.framework.modules.biz.convert.DataLabelRuleCategoryConvert;
import com.xxh.cloud.datalabel.config.framework.modules.biz.service.DataLabelRuleCategoryBizService;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.tree.TreeUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class DataLabelRuleCategoryBizServiceImpl implements DataLabelRuleCategoryBizService {

    @Autowired
    private DataLabelRuleCategoryInfoService dataLabelRuleCategoryInfoService;


    @Autowired
    private DataLabelRuleConfigService dataLabelRuleConfigService;


    @Override
    public List<DataLabelRuleCategoryVO> tree(DataLabelRuleCategoryTreeDTO params) {
        List<DataLabelRuleCategoryInfo> list= dataLabelRuleCategoryInfoService.list();
        if(CollUtil.isEmpty(list)){
            return null;
        }

        List<DataLabelRuleCategoryVO> dataList=new ArrayList<>();
        list.forEach(item->{
            dataList.add(DataLabelRuleCategoryConvert.getDataLabelRuleCategoryVO(item));
        });

        List<DataLabelRuleCategoryVO> data= TreeUtils.build(dataList);
        return data;
    }

    @Override
    public DataLabelRuleCategoryVO save(DataLabelRuleCategorySaveDTO params) {
        ValidatorUtils.validateEntity(params);
        DataLabelRuleCategoryInfo entity=new DataLabelRuleCategoryInfo();
        entity.setId(IdUtil.getSnowflake(2,1).nextIdStr());
        entity.setProjectCode(params.getProjectCode());
        entity.setName(params.getName());
        entity.setRemark(params.getRemark());
        entity.setOrders(params.getOrders());
        entity.setPid(params.getPid());
        dataLabelRuleCategoryInfoService.save(entity);
        return DataLabelRuleCategoryConvert.getDataLabelRuleCategoryVO(entity);
    }

    @Override
    public DataLabelRuleCategoryVO update(DataLabelRuleCategoryUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        DataLabelRuleCategoryInfo entity=dataLabelRuleCategoryInfoService.getById(params.getId());
        if(entity==null ){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setName(params.getName());
        entity.setOrders(params.getOrders());
        entity.setRemark(params.getRemark());
        dataLabelRuleCategoryInfoService.updateById(entity);
        return DataLabelRuleCategoryConvert.getDataLabelRuleCategoryVO(entity);
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        DataLabelRuleCategoryInfo entity=dataLabelRuleCategoryInfoService.getById(id);
        if(entity==null ){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        long childNum= dataLabelRuleCategoryInfoService.count(
                new QueryWrapper<DataLabelRuleCategoryInfo>()
                        .eq("pid",entity.getId())
        );

        if(childNum>0){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_EXISTS,"存在子数据");
        }

        long num= dataLabelRuleConfigService.count(
                new QueryWrapper<DataLabelRuleConfig>()
                        .eq("category_id",entity.getId())
                        .eq("rows_status", YN.YES.getCode())
        );
        if(num>0){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_EXISTS,"规则配置已存在");
        }





        dataLabelRuleCategoryInfoService.removeById(id);
    }

    @Override
    public DataLabelRuleCategoryVO infoById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        DataLabelRuleCategoryInfo entity=dataLabelRuleCategoryInfoService.getById(id);
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return DataLabelRuleCategoryConvert.getDataLabelRuleCategoryVO(entity);
    }
}
