package com.xxh.cloud.datalabel.config.framework.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.unit.DataUnit;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.TableFieldInfo;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.config.datalabel.client.bean.dto.LabelRuleStatusUpdateDTO;
import com.xxh.cloud.config.datalabel.constant.LabelRuleInitDateType;
import com.xxh.cloud.datalabel.config.framework.modules.base.entity.DataLabelRuleConfig;
import com.xxh.cloud.datalabel.config.framework.modules.base.entity.DataLabelRuleRelInfo;
import com.xxh.cloud.datalabel.config.framework.modules.base.service.DataLabelRuleConfigService;
import com.xxh.cloud.datalabel.config.framework.modules.base.service.DataLabelRuleRelInfoService;
import com.xxh.cloud.datalabel.config.framework.modules.biz.bean.dto.*;
import com.xxh.cloud.datalabel.config.framework.modules.biz.bean.vos.DataLabelRuleConfigVO;
import com.xxh.cloud.datalabel.config.framework.modules.biz.convert.DataLabelRuleConfigConvert;
import com.xxh.cloud.datalabel.config.framework.modules.biz.mapper.LabelInfoOpMapper;
import com.xxh.cloud.datalabel.config.framework.modules.biz.service.DataLabelRuleConfigBizService;
import com.xxh.cloud.datalabel.config.framework.modules.cache.service.DataLabelRuleConfigCacheService;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.healthy.basic.label.client.api.LabelClientService;
import com.xxh.cloud.healthy.basic.label.client.api.bean.bos.LabelBO;
import com.xxh.cloud.log.bean.constant.ModuleTypeEnum;
import com.xxh.cloud.log.bean.constant.ObjectTypeEnum;
import com.xxh.cloud.log.bean.constant.OperationTypeEnum;
import com.xxh.cloud.log.utils.OperationLogUtils;
import com.xxh.cloud.metadata.client.api.StandardDiseaseClientService;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardDiseaseBO;
import com.xxh.cloud.rule.core.context.XXRuleVariableRoom;
import com.xxh.cloud.rule.core.entity.base.NodeCondition;
import com.xxh.cloud.rule.core.entity.flow.XXFlowRuleBO;
import com.xxh.cloud.rule.flow.variable.XXRuleFlowVariableExecutor;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.function.Predicate;

@Service
public class DataLabelRuleConfigBizServiceImpl implements DataLabelRuleConfigBizService {

    @Autowired
    private DataLabelRuleConfigService dataLabelRuleConfigService;
    @Autowired
    private DataLabelRuleRelInfoService dataLabelRuleRelInfoService;

    @Autowired
    private AdminUserClientService adminUserClientService;
    @Autowired
    private LabelClientService labelClientService;

    @Autowired
    private StandardDiseaseClientService standardDiseaseClientService;

    @Autowired
    private LabelInfoOpMapper labelInfoOpMapper;

    @Autowired
    private DataLabelRuleConfigCacheService dataLabelRuleConfigCacheService;



    @Override
    public PageData<DataLabelRuleConfigVO> query(DataLabelRuleConfigQueryDTO params) {
        ValidatorUtils.validateEntity(params);

        IPage<DataLabelRuleConfig> page = dataLabelRuleConfigService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<DataLabelRuleConfig>()

                        //1.写法
//                        .select(DataLabelRuleConfig.class, new Predicate<TableFieldInfo>() {
//                            @Override
//                            public boolean test(TableFieldInfo tableFieldInfo) {
//                                if("config".equals(tableFieldInfo.getColumn())){
//                                    return false;
//                                }else {
//                                    return true;
//                                }
//                            }
//                        })

                        //2.写法
                        .select(DataLabelRuleConfig.class, info->!info.getColumn().equals("config"))
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
                        .eq(StrUtil.isNotEmpty(params.getProjectCode()),"project_code",params.getProjectCode())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getType()),"type",params.getType())
                        .eq(StrUtil.isNotEmpty(params.getStatus()),"status",params.getStatus())
                        .like(StrUtil.isNotEmpty(params.getLabelId()),"label_ids",params.getLabelId())
                        .eq(StrUtil.isNotEmpty(params.getCategoryId()),"category_id",params.getCategoryId())
                        .like(StrUtil.isNotEmpty(params.getDiseaseId()),"disease_ids",params.getDiseaseId())

                        .eq(StrUtil.isNotEmpty(params.getExtendOne()),"extend_one",params.getExtendOne())
                        .eq(StrUtil.isNotEmpty(params.getExtendTwo()),"extend_two",params.getExtendTwo())
                        .eq(StrUtil.isNotEmpty(params.getExtendThree()),"extend_three",params.getExtendThree())

                        .orderByDesc("orders")
        );


        if (page == null || CollUtil.isEmpty(page.getRecords())) {
            return null;
        }
        Set<String> labelIds=new HashSet<>();
        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();

        page.getRecords().forEach(item->{
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
            if(StrUtil.isNotEmpty(item.getLabelIds())){
                List<String> _ids= StrJoinSplitUtils.split2List(item.getLabelIds());
                labelIds.addAll(_ids);
            }
            if(StrUtil.isNotEmpty(item.getDiseaseIds())){
                List<String> _ids= StrJoinSplitUtils.split2List(item.getDiseaseIds());
                diseaseIds.addAll(_ids);
            }
        });

        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
        Map<String, LabelBO> labelMap= labelClientService.getLabelMapByIds(labelIds);
        Map<String, StandardDiseaseBO> diseaseMap= standardDiseaseClientService.getMap(diseaseIds);

        PageData<DataLabelRuleConfigVO> data= new PageConverter<DataLabelRuleConfig, DataLabelRuleConfigVO>() {
            @Override
            public DataLabelRuleConfigVO converter(DataLabelRuleConfig entity) {
                return DataLabelRuleConfigConvert.getDataLabelRuleConfigVO(entity,labelMap,userMap,diseaseMap);
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public DataLabelRuleConfigVO save(DataLabelRuleConfigSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        if(LabelRuleInitDateType.LRDT_After.getCode().equals (params.getInitDateType())){
            if(params.getInitDateValue()==null || params.getInitDateValue()<=0){
                throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"延迟执行，延迟时间不能为空后小于等于0");
            }
        }

        DataLabelRuleConfig entity=new DataLabelRuleConfig();
        if(StrUtil.isNotEmpty(params.getId())){
            entity.setId(params.getId());
        }else {
            entity.setId(IdUtil.getSnowflake(1,1).nextIdStr());
        }
        entity.setCategoryId(params.getCategoryId());
        entity.setProjectCode(params.getProjectCode());
        entity.setVersionNo(params.getVersionNo());
        entity.setDiseaseIds(StrJoinSplitUtils.join(params.getDiseaseIds()));
        entity.setName(params.getName());
        entity.setType(params.getType());
        entity.setRemark(params.getRemark());
        entity.setLabelIds(StrJoinSplitUtils.join(params.getLabelIds()));
        entity.setInitDateType(params.getInitDateType());
        entity.setInitDateValue(params.getInitDateValue());
        entity.setOrders(params.getOrders());
        entity.setLabelValidityNum(params.getLabelValidityNum());
        entity.setExtendOne(params.getExtendOne());
        entity.setExtendTwo(params.getExtendTwo());
        entity.setExtendThree(params.getExtendThree());
        entity.setExeBatch(params.getExeBatch());
        entity.setStatus(YN.NO.getCode());
        entity.setUpTime(new Date());

        entity.setRowsStatus(YN.YES.getCode());
        if(params.getConfig()!=null){
            entity.setConfig(JSON.toJSONString(params.getConfig()));
        }
        dataLabelRuleConfigService.save(entity);

        //日志
        Map<String, Object> logContent=new HashMap<>();
        logContent.put("ID",entity.getId());
        logContent.put("名称",entity.getName());
        OperationLogUtils.addOpLogGen(OperationTypeEnum.OP_Add, ModuleTypeEnum.MT_DataLabel, ObjectTypeEnum.OT_DataLabel_Rule,entity.getId(),logContent);
        return DataLabelRuleConfigConvert.getDataLabelRuleConfigVO(entity);
    }

    @Override
    public DataLabelRuleConfigVO update(DataLabelRuleConfigUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        if(LabelRuleInitDateType.LRDT_After.getCode().equals (params.getInitDateType())){
            if(params.getInitDateValue()==null || params.getInitDateValue()<=0){
                throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"延迟执行，延迟时间不能为空后小于等于0");
            }
        }
        DataLabelRuleConfig entity=dataLabelRuleConfigService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(YN.YES.getCode().equals(entity.getStatus())){
            throw  new BusinessException(BusinessBaseErrorEnum.Illegal_Operation_ERROR);
        }
        Map<String, Object> logContent=new HashMap<>();
        logContent.put("ID",entity.getId());
        logContent.put("原名称",entity.getName());

        entity.setCategoryId(params.getCategoryId());
        entity.setName(params.getName());
        entity.setType(params.getType());
        entity.setRemark(params.getRemark());
        entity.setLabelIds(StrJoinSplitUtils.join(params.getLabelIds()));
        entity.setDiseaseIds(StrJoinSplitUtils.join(params.getDiseaseIds()));
        entity.setInitDateType(params.getInitDateType());
        entity.setInitDateValue(params.getInitDateValue());
        entity.setOrders(params.getOrders());
        entity.setExtendOne(params.getExtendOne());
        entity.setExtendTwo(params.getExtendTwo());
        entity.setExtendThree(params.getExtendThree());
        entity.setExeBatch(params.getExeBatch());

        if(params.getConfig()!=null){
            entity.setConfig(JSON.toJSONString(params.getConfig()));
        }
        entity.setLabelValidityNum(params.getLabelValidityNum());
        dataLabelRuleConfigService.updateById(entity);

        //
        logContent.put("新名称",entity.getName());
        OperationLogUtils.addOpLogGen(OperationTypeEnum.OP_Edit, ModuleTypeEnum.MT_DataLabel, ObjectTypeEnum.OT_DataLabel_Rule,entity.getId(),logContent);
        return DataLabelRuleConfigConvert.getDataLabelRuleConfigVO(entity);
    }

    @Override
    public DataLabelRuleConfigVO updateConfig(DataLabelRuleConfigUpdateConfigDTO params) {
        ValidatorUtils.validateEntity(params);
        DataLabelRuleConfig entity=dataLabelRuleConfigService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(YN.YES.getCode().equals(entity.getStatus())){
            throw  new BusinessException(BusinessBaseErrorEnum.Illegal_Operation_ERROR);
        }
        entity.setConfig(JSON.toJSONString(params.getConfig()));
        dataLabelRuleConfigService.updateById(entity);

        Map<String, Object> logContent=new HashMap<>();
        logContent.put("名称",entity.getName());
        logContent.put("操作内容","修改条件配置内容");
        OperationLogUtils.addOpLogGen(OperationTypeEnum.OP_Edit, ModuleTypeEnum.MT_DataLabel, ObjectTypeEnum.OT_DataLabel_Rule,entity.getId(),logContent);

        return DataLabelRuleConfigConvert.getDataLabelRuleConfigVO(entity);
    }

    @Override
    public DataLabelRuleConfigVO updateStatus(DataLabelRuleConfigUpdateStatusDTO params) {
        ValidatorUtils.validateEntity(params);
        DataLabelRuleConfig entity=dataLabelRuleConfigService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        if(params.getStatus().equals(entity.getStatus())){
            return DataLabelRuleConfigConvert.getDataLabelRuleConfigVO(entity);
        }

        //日志
        Map<String, Object> logContent=new HashMap<>();
        logContent.put("ID",entity.getId());
        logContent.put("原状态",entity.getStatus());

        entity.setStatus(params.getStatus());
        dataLabelRuleConfigService.updateById(entity);

        if(YN.YES.getCode().equals(params.getStatus())){
            entity.setUpTime(new Date());
            if(LabelRuleInitDateType.LRDT_After.getCode().equals (entity.getInitDateType())){
                entity.setRealRunTime(DateUtil.offsetDay(entity.getUpTime(),entity.getInitDateValue()));
            }else {
                entity.setRealRunTime(entity.getUpTime());
            }
            if(StrUtil.isNotEmpty(entity.getConfig())){
                NodeCondition conditionConfig = JSON.parseObject(entity.getConfig(), NodeCondition.class);
                XXRuleFlowVariableExecutor ruleFlowVariableExecutor=new XXRuleFlowVariableExecutor();
                XXRuleVariableRoom ruleVariableRoom= ruleFlowVariableExecutor.executeByCondition(conditionConfig);
                List<DataLabelRuleRelInfo> configRelList= DataLabelRuleConfigConvert.getDataLabelRuleRelInfo_Add(entity.getId(),ruleVariableRoom);
                dataLabelRuleRelInfoService.remove(new QueryWrapper<DataLabelRuleRelInfo>().eq("config_id",entity.getId()));
                if(CollUtil.isNotEmpty(configRelList)){
                    dataLabelRuleRelInfoService.saveBatch(configRelList);
                }
            }
        }


        logContent.put("新状态",entity.getStatus());
        OperationLogUtils.addOpLogGen(OperationTypeEnum.OP_Edit_Status, ModuleTypeEnum.MT_DataLabel, ObjectTypeEnum.OT_DataLabel_Rule,entity.getId(),logContent);

        dataLabelRuleConfigCacheService.removeDataLabelRuleConfig(entity.getId(),entity.getType(),entity.getLabelIds());

        return DataLabelRuleConfigConvert.getDataLabelRuleConfigVO(entity);
    }

    @Override
    public void updateExeStatus(LabelRuleStatusUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        DataLabelRuleConfig entity=dataLabelRuleConfigService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        if(!LabelRuleInitDateType.LRDT_Before.getCode().equals (entity.getInitDateType())){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"非提前执行，不支持历史数据");
        }

        entity.setInitExeStatus(params.getExeStatus());
        entity.setInitExeTime(new Date());
        dataLabelRuleConfigService.updateById(entity);

    }

    @Override
    public DataLabelRuleConfigVO copy(DataLabelRuleConfigCopyDTO params) {
        ValidatorUtils.validateEntity(params);

        DataLabelRuleConfig entity=dataLabelRuleConfigService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(StrUtil.isEmpty(entity.getLabelIds())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        String newLabelId=StrUtil.format("L{}C",IdUtil.getSnowflake(1,1).nextIdStr());

        String sql= StrUtil.format("        " +
                " INSERT INTO s_md_standard_label_info (" +
                "                id , name ,biz_type ,category_id ,project_code ,version_code ,remark , orders ,`status` ,rows_status ,creator ,create_date ,updater,update_date\n" +
                "        ) " +
                " SELECT" +
                "        '{}' as id , CONCAT (name,'-拷贝') as  name ,biz_type ,category_id ,project_code ,version_code ," +
                "         remark , orders ,'N' as  `status`," +
                "        'Y' as rows_status , '{}' as creator , NOW()  as create_date ,'{}' as updater,NOW() as update_date" +
                "        FROM s_md_standard_label_info where  id='{}'" ,newLabelId ,params.getOpUserId(),params.getOpUserId(),entity.getLabelIds());

        labelInfoOpMapper.copyLabel(sql);


        DataLabelRuleConfig data=new DataLabelRuleConfig();
        data.setId("r_"+newLabelId);
        data.setCategoryId(entity.getCategoryId());
        data.setProjectCode(entity.getProjectCode());
        data.setVersionNo(entity.getVersionNo());
        data.setName(entity.getName()+"拷贝");
        data.setType(entity.getType());
        data.setRemark(entity.getRemark());
        data.setLabelIds(newLabelId);
        data.setLabelValidityNum(entity.getLabelValidityNum());
        data.setDiseaseIds(entity.getDiseaseIds());
        data.setInitDateType(entity.getInitDateType());
        data.setInitDateValue(entity.getInitDateValue());
        data.setOrders(entity.getOrders());
        data.setExtendOne(entity.getExtendOne());
        data.setExtendTwo(entity.getExtendTwo());
        data.setExtendThree(entity.getExtendThree());
        data.setExeBatch(entity.getExeBatch());

        data.setStatus(YN.NO.getCode());
        data.setRowsStatus(YN.YES.getCode());
        data.setConfig(entity.getConfig());
        dataLabelRuleConfigService.save(data);

        return DataLabelRuleConfigConvert.getDataLabelRuleConfigVO(data);
    }



    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR);
        }
        DataLabelRuleConfig entity=dataLabelRuleConfigService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(YN.YES.getCode().equals(entity.getStatus())){
            throw  new BusinessException(BusinessBaseErrorEnum.Illegal_Operation_ERROR);
        }
        entity.setRowsStatus(YN.NO.getCode());
        dataLabelRuleConfigService.updateById(entity);

        Map<String, Object> logContent=new HashMap<>();
        logContent.put("名称",entity.getName());
        OperationLogUtils.addOpLogGen(OperationTypeEnum.OP_Del, ModuleTypeEnum.MT_DataLabel, ObjectTypeEnum.OT_DataLabel_Rule,entity.getId(),logContent);
    }

    @Override
    public DataLabelRuleConfigVO getById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR);
        }
        DataLabelRuleConfig entity=dataLabelRuleConfigService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            return null;
          //  throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        Set<String> labelIds=new HashSet<>();
        Set<String> userIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();

        if(StrUtil.isNotEmpty(entity.getCreator())){
            userIds.add(entity.getCreator());
        }
        if(StrUtil.isNotEmpty(entity.getUpdater())){
            userIds.add(entity.getUpdater());
        }
        if(StrUtil.isNotEmpty(entity.getLabelIds())){
            labelIds=StrJoinSplitUtils.split2Set(entity.getLabelIds());
        }
        if(StrUtil.isNotEmpty(entity.getDiseaseIds())){
            diseaseIds=StrJoinSplitUtils.split2Set(entity.getDiseaseIds());
        }

        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
        Map<String, LabelBO> labelMap= labelClientService.getLabelMapByIds(labelIds);
        Map<String, StandardDiseaseBO> diseaseMap= standardDiseaseClientService.getMap(diseaseIds);
        return DataLabelRuleConfigConvert.getDataLabelRuleConfigVO(entity,labelMap,userMap,diseaseMap);
    }

    @Override
    public DataLabelRuleConfig getBeanById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR);
        }
        DataLabelRuleConfig entity=dataLabelRuleConfigService.getById(id);
        return entity;
    }

    @Override
    public DataLabelRuleConfigVO getFullById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR);
        }
        DataLabelRuleConfig entity=dataLabelRuleConfigService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return DataLabelRuleConfigConvert.getDataLabelRuleConfigVOFull(entity);
    }



}
