package com.xxh.cloud.datalabel.config.manager.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.xxh.cloud.datalabel.config.framework.modules.biz.bean.dto.*;
import com.xxh.cloud.datalabel.config.framework.modules.biz.bean.vos.DataLabelRuleBackUpVO;
import com.xxh.cloud.datalabel.config.framework.modules.biz.bean.vos.DataLabelRuleConfigVO;
import com.xxh.cloud.datalabel.config.framework.modules.biz.service.DataLabelRuleBackUpBizService;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@Log4j2
@RestController
@RequestMapping("/sys/dataLabel/backUpConfig")
@Api(tags = "运营端,数据标签配置备份管理")
public class DataLabelRuleBackUpManagerController {

    @Autowired
    private DataLabelRuleBackUpBizService dataLabelRuleBackUpBizService;


    @SaCheckPermission("sys:dataLabel:backUpConfig:list")
    @PostMapping("/list")
    @ApiOperation(value = "列表", response = DataLabelRuleBackUpVO.class)
    public Result<List<DataLabelRuleBackUpVO>> list(@RequestBody DataLabelRuleBackUpListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<DataLabelRuleBackUpVO> data= dataLabelRuleBackUpBizService.list(params);
        return new Result<List<DataLabelRuleBackUpVO>>().ok(data);
    }


    @SaCheckPermission("sys:dataLabel:backUpConfig:save")
    @PostMapping("/save")
    @ApiOperation(value = "新增备份", response = DataLabelRuleBackUpVO.class)
    public Result<DataLabelRuleBackUpVO> save(@RequestBody DataLabelRuleBackUpSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        DataLabelRuleBackUpVO data= dataLabelRuleBackUpBizService.backup(params);
        return new Result<DataLabelRuleBackUpVO>().ok(data);
    }


    @SaCheckPermission("sys:dataLabel:backUpConfig:update")
    @PostMapping("/update")
    @ApiOperation(value = "修改备份", response = DataLabelRuleBackUpVO.class)
    public Result<DataLabelRuleBackUpVO> update(@RequestBody DataLabelRuleBackUpUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        DataLabelRuleBackUpVO data= dataLabelRuleBackUpBizService.update(params);
        return new Result<DataLabelRuleBackUpVO>().ok(data);
    }


    @SaCheckPermission("sys:dataLabel:backUpConfig:getById")
    @PostMapping("/getById")
    @ApiOperation(value = "查看备份", response = DataLabelRuleBackUpVO.class)
    public Result<DataLabelRuleBackUpVO> getById(@RequestBody DataLabelRuleBackUpGetDTO params) {
        ValidatorUtils.validateEntity(params);
        DataLabelRuleBackUpVO data= dataLabelRuleBackUpBizService.getById(params.getId());
        return new Result<DataLabelRuleBackUpVO>().ok(data);
    }





}
