package com.xxh.cloud.datalabel.config.manager.controller;


import cn.dev33.satoken.annotation.SaCheckPermission;
import com.xxh.cloud.datalabel.config.framework.modules.biz.bean.dto.DataLabelRuleCategoryByIdDTO;
import com.xxh.cloud.datalabel.config.framework.modules.biz.bean.dto.DataLabelRuleCategorySaveDTO;
import com.xxh.cloud.datalabel.config.framework.modules.biz.bean.dto.DataLabelRuleCategoryTreeDTO;
import com.xxh.cloud.datalabel.config.framework.modules.biz.bean.dto.DataLabelRuleCategoryUpdateDTO;
import com.xxh.cloud.datalabel.config.framework.modules.biz.bean.vos.DataLabelRuleCategoryVO;
import com.xxh.cloud.datalabel.config.framework.modules.biz.service.DataLabelRuleCategoryBizService;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@Log4j2
@RestController
@RequestMapping("/sys/dataLabel/ruleCategory")
@Api(tags = "运营端,数据标签规则分类管理")
public class DataLabelRuleCategoryManagerController {

    @Autowired
    private DataLabelRuleCategoryBizService dataLabelRuleCategoryBizService;




    @SaCheckPermission("sys:rule:category:tree")
    @PostMapping("/tree")
    @ApiOperation(value = "树", response = DataLabelRuleCategoryVO.class)
    public Result<List<DataLabelRuleCategoryVO>> tree(@RequestBody DataLabelRuleCategoryTreeDTO params) {
        //todo:
        List<DataLabelRuleCategoryVO> data= dataLabelRuleCategoryBizService.tree(params);
        return new Result<List<DataLabelRuleCategoryVO>>().ok(data);
    }

    @SaCheckPermission("sys:rule:category:save")
    @PostMapping("/save")
    @ApiOperation(value = "新增", response = DataLabelRuleCategoryVO.class)
    public Result<DataLabelRuleCategoryVO> save(@RequestBody DataLabelRuleCategorySaveDTO params) {
        ValidatorUtils.validateEntity(params);
        DataLabelRuleCategoryVO data= dataLabelRuleCategoryBizService.save(params);
        return new Result<DataLabelRuleCategoryVO>().ok(data);
    }

    @SaCheckPermission("sys:rule:category:update")
    @PostMapping("/update")
    @ApiOperation(value = "修改", response = DataLabelRuleCategoryVO.class)
    public Result<DataLabelRuleCategoryVO> update(@RequestBody DataLabelRuleCategoryUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        DataLabelRuleCategoryVO data=dataLabelRuleCategoryBizService.update(params);
        return new Result<DataLabelRuleCategoryVO>().ok(data);
    }

    @SaCheckPermission("sys:rule:category:del")
    @PostMapping("/del")
    @ApiOperation(value = "删除", response = String.class)
    public Result<String> del(@RequestBody DataLabelRuleCategoryByIdDTO params) {
        ValidatorUtils.validateEntity(params);
        dataLabelRuleCategoryBizService.delById(params.getId());
        return new Result<String>().ok("ok");
    }

    @SaCheckPermission("sys:rule:category:info")
    @PostMapping("/info")
    @ApiOperation(value = "获取", response = DataLabelRuleCategoryVO.class)
    public  Result<DataLabelRuleCategoryVO> info(@RequestBody DataLabelRuleCategoryByIdDTO params) {
        ValidatorUtils.validateEntity(params);
        DataLabelRuleCategoryVO data= dataLabelRuleCategoryBizService.infoById(params.getId());
        return new Result<DataLabelRuleCategoryVO>().ok(data);
    }







}
