package com.xxh.cloud.datalabel.config.manager.controller;


import cn.dev33.satoken.annotation.SaCheckPermission;
import com.xxh.cloud.datalabel.config.framework.modules.biz.bean.dto.*;
import com.xxh.cloud.datalabel.config.framework.modules.biz.bean.vos.DataLabelRuleConfigVO;
import com.xxh.cloud.datalabel.config.framework.modules.biz.service.DataLabelRuleConfigBizService;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@Log4j2
@RestController
@RequestMapping("/sys/dataLabel/ruleConfig")
@Api(tags = "运营端,数据标签规则配置管理")
public class DataLabelRuleConfigManagerController {

    @Autowired
    private DataLabelRuleConfigBizService dataLabelRuleConfigBizService;


    @SaCheckPermission("sys:dataLabel:ruleConfig:query")
    @PostMapping("/query")
    @ApiOperation(value = "查询", response = DataLabelRuleConfigVO.class)
    public Result<PageData<DataLabelRuleConfigVO>> query(@RequestBody DataLabelRuleConfigQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<DataLabelRuleConfigVO> data= dataLabelRuleConfigBizService.query(params);
        return new Result<PageData<DataLabelRuleConfigVO>>().ok(data);
    }

    @SaCheckPermission("sys:dataLabel:ruleConfig:save")
    @PostMapping("/save")
    @ApiOperation(value = "新增", response = DataLabelRuleConfigVO.class)
    public Result<DataLabelRuleConfigVO> save(@RequestBody DataLabelRuleConfigSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        DataLabelRuleConfigVO data= dataLabelRuleConfigBizService.save(params);
        return new Result<DataLabelRuleConfigVO>().ok(data);
    }

    @SaCheckPermission("sys:dataLabel:ruleConfig:update")
    @PostMapping("/update")
    @ApiOperation(value = "修改基本信息", response = DataLabelRuleConfigVO.class)
    public Result<DataLabelRuleConfigVO> update(@RequestBody DataLabelRuleConfigUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        DataLabelRuleConfigVO data=dataLabelRuleConfigBizService.update(params);
        return new Result<DataLabelRuleConfigVO>().ok(data);
    }


    @SaCheckPermission("sys:dataLabel:ruleConfig:updateConfig")
    @PostMapping("/updateConfig")
    @ApiOperation(value = "修改配置", response = DataLabelRuleConfigVO.class)
    public Result<DataLabelRuleConfigVO> updateConfig(@RequestBody DataLabelRuleConfigUpdateConfigDTO params) {
        ValidatorUtils.validateEntity(params);
        DataLabelRuleConfigVO data=dataLabelRuleConfigBizService.updateConfig(params);
        return new Result<DataLabelRuleConfigVO>().ok(data);
    }




    @SaCheckPermission("sys:dataLabel:ruleConfig:updateStatus")
    @PostMapping("/updateStatus")
    @ApiOperation(value = "修改上下架状态", response = String.class)
    public Result<String> updateStatus(@RequestBody DataLabelRuleConfigUpdateStatusDTO params) {
        ValidatorUtils.validateEntity(params);
        DataLabelRuleConfigVO data=dataLabelRuleConfigBizService.updateStatus(params);
        return new Result<String>().ok("ok");
    }


    @SaCheckPermission("sys:dataLabel:ruleConfig:del")
    @PostMapping("/del")
    @ApiOperation(value = "删除", response = String.class)
    public Result<String> del(@RequestBody DataLabelRuleConfigGetDTO params) {
        ValidatorUtils.validateEntity(params);
        dataLabelRuleConfigBizService.delById(params.getId());
        return new Result<String>().ok("ok");
    }

    @SaCheckPermission("sys:dataLabel:ruleConfig:info")
    @PostMapping("/info")
    @ApiOperation(value = "获取基本信息", response = DataLabelRuleConfigVO.class)
    public  Result<DataLabelRuleConfigVO> info(@RequestBody DataLabelRuleConfigGetDTO params) {
        ValidatorUtils.validateEntity(params);
        DataLabelRuleConfigVO data= dataLabelRuleConfigBizService.getById(params.getId());
        return new Result<DataLabelRuleConfigVO>().ok(data);
    }


    @SaCheckPermission("sys:dataLabel:ruleConfig:getConfig")
    @PostMapping("/getConfig")
    @ApiOperation(value = "获取配置", response = DataLabelRuleConfigVO.class)
    public Result<DataLabelRuleConfigVO> getConfig(@RequestBody DataLabelRuleConfigGetDTO params) {
        ValidatorUtils.validateEntity(params);
        DataLabelRuleConfigVO data=dataLabelRuleConfigBizService.getFullById(params.getId());
        return new Result<DataLabelRuleConfigVO>().ok(data);
    }


    @SaCheckPermission("sys:dataLabel:ruleConfig:copy")
    @PostMapping("/copy")
    @ApiOperation(value = "拷贝", response = DataLabelRuleConfigVO.class)
    public Result<DataLabelRuleConfigVO> copy(@RequestBody DataLabelRuleConfigCopyDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser user= AuthContextUtils.getCurrentLoginUserCheck();
        params.setOpUserId(user.getId());
        DataLabelRuleConfigVO data= dataLabelRuleConfigBizService.copy(params);
        return new Result<DataLabelRuleConfigVO>().ok(data);
    }



}
