package com.xxh.cloud.datalabel.executor.framework.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.xxh.cloud.datalabel.executor.framework.modules.base.entity.DataLabelRuleExeLog;
import com.xxh.cloud.datalabel.executor.framework.modules.base.entity.DataLabelRuleExeSuccessInfo;
import com.xxh.cloud.datalabel.executor.framework.modules.biz.bean.dto.DataLabelExecutorExtParamDTO;
import com.xxh.cloud.datalabel.executor.framework.modules.biz.bean.vos.DataLabelExeReturnVO;
import com.xxh.cloud.datalabel.executor.framework.modules.biz.convert.DLELogIdUtils;
import com.xxh.cloud.datalabel.executor.framework.modules.biz.service.DataLabelExecuteLogBizService;
import com.xxh.cloud.datalabel.executor.framework.modules.biz.service.DataLabelExecutorBizService;
import com.xxh.cloud.datalabel.executor.framework.modules.biz.utils.DataLabelRuleExeLogUtils;
import com.xxh.cloud.datalabel.executor.framework.modules.cache.service.DataLabelRuleConfigByExecutorCacheService;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.rule.basic.context.XXRuleRoam;
import com.xxh.cloud.rule.core.context.XXRuleContext;
import com.xxh.cloud.rule.core.context.XXRuleNodeLogBean;
import com.xxh.cloud.rule.core.entity.label.XXLabelRuleBO;
import com.xxh.cloud.rule.flow.condition.ConditionExecutorUtils;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service
public class DataLabelExecutorBizServiceImpl implements DataLabelExecutorBizService {


    @Autowired
    private DataLabelRuleConfigByExecutorCacheService dataLabelRuleConfigByExecutorCacheService;

    @Autowired
    private DataLabelExecuteLogBizService dataLabelExecuteLogBizService;


    public DataLabelExeReturnVO executorReturn(String configId , XXRuleRoam data, boolean isLogMQ, DataLabelExecutorExtParamDTO extendParams){
        XXLabelRuleBO ruleConfig = dataLabelRuleConfigByExecutorCacheService.getXXLabelRule2CacheById(configId);
        return this.executorReturnByBean(ruleConfig,data,isLogMQ,extendParams);
    }

    @Override
    public DataLabelExeReturnVO executorReturnByBean(XXLabelRuleBO ruleConfig, XXRuleRoam data, boolean isLogMQ, DataLabelExecutorExtParamDTO extendParams) {
        XXRuleContext context=XXRuleContext.getRuleContext2ExecutorByReturn(ruleConfig,data,false);
        boolean flag= ConditionExecutorUtils.executor(context,ruleConfig);

        //todo:不支持MQ记录日志
        String logId=null;
        if(flag){ //执行成功保存日志
            String idPrefix = DLELogIdUtils.getIdPrefix_DataLabelRuleExe();
            List<DataLabelRuleExeSuccessInfo> list= this.getDataLabelExecuteLogList(idPrefix,data.getPatientId(),data.getDataDate(),flag,ruleConfig,context.getNodeLogList());
            if(CollUtil.isNotEmpty(list)){
                logId = list.get(0).getId();
                dataLabelExecuteLogBizService.saveDLRExeSuccessBatch(idPrefix,list);
            }
        }

        return new DataLabelExeReturnVO(flag,ruleConfig.getId(),ruleConfig.getName(),ruleConfig.getLabelId(),
                ruleConfig.getLabelValidityNum(), ruleConfig.getDiseaseIds(),ruleConfig.getInitDateType(),
                ruleConfig.getInitDateValue(),logId);

    }



    @Override
    public List<DataLabelExeReturnVO> executorReturnByConfigList(List<XXLabelRuleBO> ruleConfigs, XXRuleRoam data, boolean isLogMQ, DataLabelExecutorExtParamDTO extendParams) {
        List<DataLabelExeReturnVO> returnData=new ArrayList<>();
        List<DataLabelRuleExeSuccessInfo> successEntityList=new ArrayList<>();
        List<DataLabelRuleExeLog> dlreLogList=new ArrayList<>();

        String idPrefix = DLELogIdUtils.getIdPrefix_DataLabelRuleExe();

        for(XXLabelRuleBO ruleConfig : ruleConfigs){

            System.out.println("+++++ 执行规则ID = "+ruleConfig.getId());

            DataLabelRuleExeLog exeLog= DataLabelRuleExeLogUtils.getDataLabelRuleExeLog(idPrefix,ruleConfig.getId(),extendParams);
            XXRuleContext context=null;
            boolean flag =false;
            boolean isSuccess=true;
            try {
                 context=XXRuleContext.getRuleContext2ExecutorByReturn(ruleConfig,data,false);
                 flag= ConditionExecutorUtils.executor(context,ruleConfig);
            }catch (Exception e){
                isSuccess=false;
                e.printStackTrace();
            }

            //记录执行日志
            if(flag){
                exeLog.setEndDt(new Date());
                exeLog.setResultStatus(YN.YES.getCode());
                exeLog.setResultRemark("有结果值");
                dlreLogList.add(exeLog);
            }else {
                if(isSuccess){
                    exeLog.setEndDt(new Date());
                    exeLog.setResultStatus(YN.NO.getCode());
                    exeLog.setResultRemark("无结果值");
                    dlreLogList.add(exeLog);
                }else {
                    exeLog.setEndDt(new Date());
                    exeLog.setResultStatus("E");
                    exeLog.setResultRemark("执行报错");
                    dlreLogList.add(exeLog);
                }
            }

            //执行成功保存推理依据
            String logId=null;
            if(flag){
                List<DataLabelRuleExeSuccessInfo> list= this.getDataLabelExecuteLogList(idPrefix,data.getPatientId(),data.getDataDate(),flag,ruleConfig,context.getNodeLogList());
                if(CollUtil.isNotEmpty(list)){
                    logId = list.get(0).getId();
                    successEntityList.addAll(list);
                }
            }

            //返回值
            returnData.add(new DataLabelExeReturnVO(flag,ruleConfig.getId(),ruleConfig.getName(),ruleConfig.getLabelId(),ruleConfig.getLabelValidityNum(),
                    ruleConfig.getDiseaseIds(),ruleConfig.getInitDateType(),ruleConfig.getInitDateValue(),logId));
        }

        if(dlreLogList.size()>0){
            dataLabelExecuteLogBizService.saveDlrExeLogBatch(idPrefix,dlreLogList);
        }

        if(successEntityList.size()>0){
            dataLabelExecuteLogBizService.saveDLRExeSuccessBatch(idPrefix,successEntityList);
        }

        return returnData;
    }


    public List<DataLabelRuleExeSuccessInfo> getDataLabelExecuteLogList(String idPrefix, String patientId ,Date patientDataDt, boolean conditionFlag, XXLabelRuleBO ruleConfig, List<XXRuleNodeLogBean> nodeLogList){
        List<DataLabelRuleExeSuccessInfo> data=new ArrayList<>();
        if(CollUtil.isNotEmpty(nodeLogList)){
            nodeLogList.forEach(item->{
                DataLabelRuleExeSuccessInfo log=getDataLabelExecuteLog(idPrefix,patientId,patientDataDt,conditionFlag,ruleConfig,item);
                data.add(log);
            });
        }
        return data;
    }

    public DataLabelRuleExeSuccessInfo getDataLabelExecuteLog(String idPrefix,String patientId ,Date patientDataDt, boolean conditionFlag, XXLabelRuleBO ruleConfig,XXRuleNodeLogBean nodeLog){
        DataLabelRuleExeSuccessInfo data=new DataLabelRuleExeSuccessInfo();
        data.setId(DLELogIdUtils.getDataLabelRuleExeSuccessId(idPrefix));
        data.setPatientId(patientId);
        data.setPatientDataDt(patientDataDt);
        data.setRuleId(ruleConfig.getId());
        data.setRuleVersion(1);
        //data.setRuleVersion(ruleConfig.getVersion());
        data.setRuleName(ruleConfig.getName());
        data.setStartDate(nodeLog.getStart());
        data.setEndDate(nodeLog.getEnd());
        data.setTimeNum(new Long(nodeLog.getTimeNum()).intValue());
        if(CollUtil.isNotEmpty(nodeLog.getConditionList())){
            data.setExecuteResult(JSON.toJSONString(nodeLog.getConditionList()));
        }
        data.setActionConfig(ruleConfig.getLabelId());

        if(conditionFlag){
            data.setExecuteResultStatus(YN.YES.getCode());
        }else {
            data.setExecuteResultStatus(YN.NO.getCode());
        }
        data.setOpDate(new Date());
        return data;
    }






}
