package com.xxh.cloud.datalabel.executor.framework.modules.cache.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.datalabel.executor.framework.modules.cache.keys.DataLabelCacheKeys;
import com.xxh.cloud.datalabel.executor.framework.modules.cache.service.DataLabelRuleConfigByExecutorCacheService;
import com.xxh.cloud.datalabel.executor.framework.modules.config.service.DataLabelConfigDataGetService;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.rule.core.entity.label.XXLabelRuleBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class DataLabelRuleConfigByExecutorCacheServiceImpl implements DataLabelRuleConfigByExecutorCacheService {

    @Autowired
    private RedisUtils redisUtils;

    @Autowired
    private DataLabelConfigDataGetService dataLabelConfigDataGetService;


    @Override
    public XXLabelRuleBO getXXLabelRule2CacheById(String id) {
        if(StrUtil.isEmpty(id)){
            return null;
        }
        String key= DataLabelCacheKeys.getDataLabelRuleConfigIdKey(id);
        Object json = redisUtils.get(key);
        if (json != null) {
            return JSON.parseObject(json.toString(),XXLabelRuleBO.class);
        }else {
            XXLabelRuleBO data= this.getXXLabelRule2DBById(id);
            if(data!=null){
                this.redisUtils.set(key,JSON.toJSONString(data),RedisUtils.Day_One_EXPIRE);
            }
            return data;
        }
    }

    @Override
    public List<XXLabelRuleBO> getXXLabelRule2CacheByIds(Collection<String> ids) {
        List<XXLabelRuleBO> data=new ArrayList<>();

        if(CollUtil.isEmpty(ids)){
            return data;
        }

        Set<String> keys = new HashSet<>();
        ids.forEach((id) -> {
            keys.add(DataLabelCacheKeys.getDataLabelRuleConfigIdKey(id));
        });

        Map<String, Object> jsonMap = this.redisUtils.getBatchMap(keys);

        if(MapUtil.isEmpty(jsonMap)){
            data = this.getXXLabelRule2DBByIds(ids);
            this.setCache(data);
            return data;
        }

        Set<String> dbIds=new HashSet<>();
        for(String id:ids){
            Object json= jsonMap.get(DataLabelCacheKeys.getDataLabelRuleConfigIdKey(id));
            if(json!=null){
                data.add(JSON.parseObject(json.toString(),XXLabelRuleBO.class));
            }else {
                dbIds.add(id);
            }
        }

        if(dbIds.size()>0){
            List<XXLabelRuleBO> dataList =  this.getXXLabelRule2DBByIds(ids);
            if(CollUtil.isNotEmpty(dataList)){
                data.addAll(dataList);
                this.setCache(dataList);
            }
        }
        return data;
    }

    private void setCache(List<XXLabelRuleBO> list){
        if(CollUtil.isEmpty(list)){
            return;
        }
        Map<String, String> map=new HashMap<>();
        for(XXLabelRuleBO obj:list){
//            map.put(DataLabelCacheKeys.getDataLabelRuleConfigIdKey(obj.getId()),JSON.toJSONString(obj));
            redisUtils.set(DataLabelCacheKeys.getDataLabelRuleConfigIdKey(obj.getId()),JSON.toJSONString(obj),RedisUtils.Day_One_EXPIRE);
        }
//        redisUtils.multiSet(map,RedisUtils.Day_One_EXPIRE);
    }



    @Override
    public List<XXLabelRuleBO> getXXLabelRule2CacheByAll() {
        String key= DataLabelCacheKeys.getDataLabelRuleConfigALLKey();
        Set<String> ids =new HashSet<>();

        Set<Object> cacheIds= this.redisUtils.sGet(key);

        if(CollUtil.isEmpty(cacheIds)){
            ids= this.getXXLabelRuleIdList_All_DB();
            if(CollUtil.isEmpty(ids)){
                return null;
            }
            this.redisUtils.sSet_str(key,ids,RedisUtils.Day_One_EXPIRE);
        }else {
            for(Object item:cacheIds){
                ids.add(item.toString());
            }
        }
        return this.getXXLabelRule2CacheByIds(ids);
    }


    //-----------------------------------------------------------------------------------

    @Override
    public XXLabelRuleBO getXXLabelRule2DBById(String id) {
        return dataLabelConfigDataGetService.getDataLabelConfig(id);
    }

    @Override
    public List<XXLabelRuleBO> getXXLabelRule2DBByIds(Collection<String> ids) {
        return dataLabelConfigDataGetService.getDataLabelConfig(ids);
    }

    @Override
    public List<XXLabelRuleBO> getXXLabelRule2DBByType(String type) {
        return dataLabelConfigDataGetService.getDataLabelConfigByType(type);
    }

    @Override
    public List<XXLabelRuleBO> getXXLabelRule2DBByLabel(String labelId) {
        return dataLabelConfigDataGetService.getDataLabelConfigByLabel(labelId);
    }

    @Override
    public List<XXLabelRuleBO> getXXLabelRule2DBByCategory(String categoryId, String isChild) {
        return dataLabelConfigDataGetService.getDataLabelConfigByCategory(categoryId);
    }

    @Override
    public List<XXLabelRuleBO> getXXLabelRule2DBByAll() {
        return dataLabelConfigDataGetService.getDataLabelConfigByAll();
    }


    //-----------------------------------------------------------------------------------



    private Set<String> getXXLabelRuleIdList_All_DB(){
        return dataLabelConfigDataGetService.getDataLabelConfigIdByAll();
    }



}
