package com.xxh.cloud.datalabel.executor.framework.modules.client.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.sun.media.sound.SoftTuning;
import com.xxh.cloud.datalabel.client.api.RuleDataLabelClientService;
import com.xxh.cloud.datalabel.client.bean.dto.*;
import com.xxh.cloud.datalabel.client.bean.ros.LabelConditionExecuteResultPathRO;
import com.xxh.cloud.datalabel.client.bean.ros.LabelRuleExecuteResultRO;
import com.xxh.cloud.datalabel.executor.framework.modules.biz.bean.dto.DataLabelExecutorExtParamDTO;
import com.xxh.cloud.datalabel.executor.framework.modules.biz.bean.dto.DataLabelRuleCallLogAddDTO;
import com.xxh.cloud.datalabel.executor.framework.modules.biz.bean.vos.DataLabelExeReturnVO;
import com.xxh.cloud.datalabel.executor.framework.modules.biz.service.DataLabelExecuteLogBizService;
import com.xxh.cloud.datalabel.executor.framework.modules.biz.service.DataLabelExecutorAsyncBizService;
import com.xxh.cloud.datalabel.executor.framework.modules.biz.service.DataLabelExecutorBizService;
import com.xxh.cloud.datalabel.executor.framework.modules.biz.utils.RuleDataScopeUtils;
import com.xxh.cloud.datalabel.executor.framework.modules.cache.service.DataLabelRuleConfigByExecutorCacheService;
import com.xxh.cloud.datalabel.executor.framework.modules.client.convert.RuleDataLabelClientConvert;
import com.xxh.cloud.datalabel.executor.framework.modules.config.service.DataLabelConfigDataGetService;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.rule.basic.context.XXRuleRoam;
import com.xxh.cloud.rule.core.entity.label.XXLabelRuleBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service
public class RuleDataLabelClientServiceImpl implements RuleDataLabelClientService {

    @Autowired
    private DataLabelExecutorBizService dataLabelExecutorBizService;

    @Autowired
    private DataLabelRuleConfigByExecutorCacheService dataLabelRuleConfigByExecutorCacheService;


    @Autowired
    private DataLabelExecuteLogBizService dataLabelExecuteLogBizService;


    @Autowired
    private DataLabelExecutorAsyncBizService dataLabelExecutorAsyncBizService;

    @Autowired
    private DataLabelConfigDataGetService dataLabelConfigDataGetService;

    public LabelRuleExecuteResultRO executeByIdTest(LabelRuleExecuteByRuleIdDTO params) {
        XXLabelRuleBO ruleConfig = dataLabelConfigDataGetService.getDataLabelConfig_Test(params.getRuleId());
        if(ruleConfig==null){
            throw new BusinessException(BusinessBaseErrorEnum.NOT_NULL,"配置信息不存在");
        }
        DataLabelExeReturnVO result=null;
        try {
            result =  dataLabelExecutorBizService.executorReturnByBean(ruleConfig, params.getData(), false,null);
        }catch (Exception e){
            e.printStackTrace();
        }

        if(result==null || result.isFlag()==false){
            return null;
        }

        return RuleDataLabelClientConvert.getLabelRuleExecuteResultRO(result);
    }

    @Override
    public LabelRuleExecuteResultRO executeById(LabelRuleExecuteByRuleIdDTO params) {
        if(params.isTest()){
            return this.executeByIdTest(params);
        }

        //log
        DataLabelRuleCallLogAddDTO callLogDTO=new DataLabelRuleCallLogAddDTO(params);

        XXLabelRuleBO ruleConfig = dataLabelRuleConfigByExecutorCacheService.getXXLabelRule2CacheById(params.getRuleId());
        if(ruleConfig==null){
            callLogDTO.setErrMsg("配置信息不存在");
            callLogDTO.setResultFlag("NONE");
            callLogDTO.setEndDt(new Date());
            dataLabelExecuteLogBizService.saveDlrCallLog(callLogDTO);
            throw new BusinessException(BusinessBaseErrorEnum.NOT_NULL,"配置信息不存在");
        }

        //判断是否数据变动
        if(StrUtil.isNotEmpty(params.getDataScopeFlag()) && YN.YES.getCode().equals(params.getDataScopeFlag())){
            boolean exeRuleFlag= RuleDataScopeUtils.isRuleConfigScope(ruleConfig.getVarCodes(),params.getDataScope());
            if(exeRuleFlag==false){
                callLogDTO.setErrMsg("数据变动不在引擎数据范围已内");
                callLogDTO.setResultFlag("NONE");
                callLogDTO.setEndDt(new Date());
                dataLabelExecuteLogBizService.saveDlrCallLog(callLogDTO);
                return null;
            }
        }


        DataLabelExeReturnVO result=null;
        try {
             DataLabelExecutorExtParamDTO extendParams=new DataLabelExecutorExtParamDTO(callLogDTO.getId(),params.getPatientId(),params.getDataDay());
             result =  dataLabelExecutorBizService.executorReturnByBean(ruleConfig, params.getData(), false,extendParams);
        }catch (Exception e){
            callLogDTO.setErrMsg(e.getMessage());
            e.printStackTrace();
        }


        if(result==null || result.isFlag()==false){
            callLogDTO.setResultFlag(YN.NO.getCode());
            callLogDTO.setEndDt(new Date());
            dataLabelExecuteLogBizService.saveDlrCallLog(callLogDTO);
            return null;
        }

        callLogDTO.setResultFlag(YN.YES.getCode());
        callLogDTO.setEndDt(new Date());
        dataLabelExecuteLogBizService.saveDlrCallLog(callLogDTO);

        return RuleDataLabelClientConvert.getLabelRuleExecuteResultRO(result);
    }


//    @Override
//    public List<LabelRuleExecuteResultRO> executeByIds(LabelRuleExecuteByRuleIdsDTO params) {
//        List<XXLabelRuleBO> ruleConfigList = dataLabelRuleConfigByExecutorCacheService.getXXLabelRule2CacheByIds(params.getRuleIds());
//        return this.executeList(ruleConfigList, params.getData());
//    }
//
//
//    @Override
//    public List<LabelRuleExecuteResultRO> executeByCategory(LabelRuleExecuteByCategoryDTO params) {
//        List<XXLabelRuleBO> ruleConfigList = dataLabelRuleConfigByExecutorCacheService.getXXLabelRule2DBByCategory(params.getCategoryId(),params.getChildFlag());
//        return this.executeList(ruleConfigList, params.getData());
//    }
//
//    @Override
//    public List<LabelRuleExecuteResultRO> executeByLabel(LabelRuleExecuteByLabelDTO params) {
//        List<XXLabelRuleBO> ruleConfigList = dataLabelRuleConfigByExecutorCacheService.getXXLabelRule2DBByLabel(params.getLabelId());
//        if(CollUtil.isEmpty(ruleConfigList)){
//            return null;
//        }
//        return this.executeList(ruleConfigList, params.getData());
//    }

    @Override
    public List<LabelRuleExecuteResultRO> executeByAll(LabelRuleExecuteByAllDTO params) {
        System.out.println("执行参数数据范围标记:"+params.getDataScopeFlag());
        //log
        DataLabelRuleCallLogAddDTO callLogDTO=new DataLabelRuleCallLogAddDTO(params);
//        List<XXLabelRuleBO> ruleConfigList = dataLabelRuleConfigByExecutorCacheService.getXXLabelRule2DBByAll();
        List<XXLabelRuleBO> ruleConfigList =dataLabelRuleConfigByExecutorCacheService.getXXLabelRule2CacheByAll();

        //判断是否数据变动
        if(StrUtil.isNotEmpty(params.getDataScopeFlag()) && YN.YES.getCode().equals(params.getDataScopeFlag())){
            List<XXLabelRuleBO> exeRuleConfigList= RuleDataScopeUtils.getRuleConfigListByScope(ruleConfigList,params.getDataScope());
            return this.executeList(exeRuleConfigList, params.getData(),callLogDTO,params.getPatientId(),params.getDataDay());
        }else {
            return this.executeList(ruleConfigList, params.getData(),callLogDTO,params.getPatientId(),params.getDataDay());
        }
    }




    @Override
    public List<LabelConditionExecuteResultPathRO> getResultPath(String logId) {
        //todo:待查询
        return null;
    }




    private List<LabelRuleExecuteResultRO> executeList(List<XXLabelRuleBO> ruleConfigList, XXRuleRoam roamData,DataLabelRuleCallLogAddDTO callLogDTO,String patientId,String patientDate){
        List<LabelRuleExecuteResultRO> data=new ArrayList<>();

        if(CollUtil.isEmpty(ruleConfigList)){
            callLogDTO.setErrMsg("数据变动不在引擎数据范围已内");
            callLogDTO.setResultFlag("NONE");
            callLogDTO.setEndDt(new Date());
            dataLabelExecuteLogBizService.saveDlrCallLog(callLogDTO);
            return data;
        }

        List<DataLabelExeReturnVO> resultList =null;
        try {
            DataLabelExecutorExtParamDTO extendParams=new DataLabelExecutorExtParamDTO(callLogDTO.getId(),patientId,patientDate);
            resultList = dataLabelExecutorBizService.executorReturnByConfigList(ruleConfigList, roamData, false,extendParams);
        }catch (Exception e){
            callLogDTO.setErrMsg(e.getMessage());
            e.printStackTrace();
        }

        if(CollUtil.isNotEmpty(resultList)){
            resultList.forEach(item->{
                if(item.isFlag()){
                    data.add(RuleDataLabelClientConvert.getLabelRuleExecuteResultRO(item));
                }
            });
            callLogDTO.setResultFlag(YN.YES.getCode());
        }else {
            callLogDTO.setResultFlag(YN.NO.getCode());
        }

        callLogDTO.setEndDt(new Date());
        dataLabelExecuteLogBizService.saveDlrCallLog(callLogDTO);

        return data;
    }




}
