package com.xxh.cloud.datalabel.executor.framework.modules.demo.d1;


import org.springframework.transaction.jta.WebSphereUowTransactionManager;

import java.util.Random;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;

/**
 * CompletableFuture
 * 主线程只管提交任务，提交完后就不管了，无需等待任务结果，任务完成后自己回调后续操作。因此，CompletableFuture 来了。
 * */
public class CompletableFutureDemo {


    private static  final Random random=new Random();



    public static void test1() throws InterruptedException{
        CompletableFuture.supplyAsync(()->{
            try {
                int x= random.nextInt(1000);
                Thread.sleep(x);
                return x;
            } catch (InterruptedException e) {
                e.printStackTrace();
                throw new RuntimeException(e);
            }
        }).thenAccept((result)-> System.out.println("异步提交的任务返回值:"+result));

        System.out.println("继续主线程后面的任务");
        System.out.println("--------------Main");
        Thread.sleep(1000);
    }


    /**
     * 串行执行
     * */
    public static void test2() throws InterruptedException {
        CompletableFuture.supplyAsync(()->{
            System.out.println(Thread.currentThread().toString()+" 执行任务1");
            return 1;
        }).thenApply((data)->{
            System.out.println(Thread.currentThread().toString()+" 执行任务2");
            return data+2;
        }).thenApply((data)->{
            System.out.println(Thread.currentThread().toString()+" 执行任务3");
            return data+2;
        }).thenAccept((result)-> System.out.println("异步串行任务结果: "+result));

        System.out.println("继续主线程任务");
        Thread.sleep(2000);
    }

    public static  void test3() throws ExecutionException, InterruptedException {
        CompletableFuture<Integer>[] tasks=new CompletableFuture[10];
        for (int i = 0; i < 10; i++) {
            tasks[i]=CompletableFuture.supplyAsync(new CompletableFutureTask(i));
        }

        CompletableFuture<Void> voidCompletableFuture =CompletableFuture
//                 .anyOf(tasks)
                .allOf(tasks)
                //allOf result 未空
                //anyof result 未第一个完成的任务结果
                .thenAccept((result) -> System.out.println("执行完成 - "+result));

        voidCompletableFuture.get();


    }

    public static void main(String[] args) throws InterruptedException, ExecutionException {
//        CompletableFutureDemo.test1();
//        CompletableFutureDemo.test2();
        CompletableFutureDemo.test3();

    }


}
