package com.xxh.cloud.rule.basic.api.bean.dto;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.rule.basic.api.MethodExecuteByCallApi;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

@Getter
@Setter
public class WorkFlowNodeExecuteDTO {



    private String id;

    /**
     * 节点ID
     * */
    private String nodeId;

    /**
     * 节点名称
     * */
    private String nodeName;

    /**
     * 节点类型
     * */
    private String nodeType;

    /**
     * 节点类型
     * */
    private String nodeRuleType;

    /**
     * 节点内容
     * */
    private String nodeContent;

    /**
     * 上一节点ID
     * */
    private String preNodeId;



    //====================================


    /**
     * 是否有执行人
     * */
    private String isExecutor;

    /**
     * 执行人(NodeExecutorType)
     * */
    private String executorType;

    /**
     * 执行对象
     * */
    private Set<String> executorIds;

    /**
     * 进度设置
     * */
    private int advanceRatio;


    //================================
    /**
     * 审批方式[或签、会签](NodeCheckWay)
     * */
    private String checkWay;

    /**
     * 会签比例
     * */
    private int checkRatio;

    //==============================

    /**
     * 是否定时触发
     * */
    private String isTrigger;

    @ApiModelProperty(value = "开始时间:审核后数量")
    private int triggerStartNum;

    @ApiModelProperty(value = "截止时间:审核后数量")
    private int triggerEndNum;

    @ApiModelProperty(value = "截止时间:审核后数量单位[DateTimeUnitTypeEnum]")
    private String triggerEndUnit;

    @ApiModelProperty(value = "触发时间类型[仅触发一次，循环触发][NodeTriggerTypeEnum]")
    private String triggerType;

    @ApiModelProperty(value = "循环触发数量")
    private int triggerLoopNum;

    @ApiModelProperty(value = "循环触发数量单位[DateTimeUnitTypeEnum]")
    private String triggerLoopUnit;



    //==============================


    @ApiModelProperty(value = "动作(函数组)ID")
    private String actionId;

    @ApiModelProperty(value = "动作(函数组)名称")
    private String actionName;

    @ApiModelProperty(value = "方法(函数)编号")
    private String methodCode;

    @ApiModelProperty(value = "方法(函数)名称")
    private String methodName;

    /**
     * 方法参数
     * */
    private List<MethodExecuteParamDTO> params;





    public WorkFlowNodeExecuteDTO(String nodeId, String nodeType,String nodeRuleType,String nodeName,
                                   String nodeContent,String prevNodeId){
        this.id= StrUtil.format("WFTN{}",IdUtil.getSnowflake(1,2).nextIdStr());
         this.nodeId=nodeId;
         this.nodeName=nodeName;
         this.nodeType=nodeType;
         this.nodeRuleType=nodeRuleType;
         this.nodeContent=nodeContent;
         this.preNodeId=preNodeId;
    }


//    public WorkFlowNodeExecuteDTO(){
//
//    }


    public void addExecutorId(String userId){
        if(this.executorIds==null){
            this.executorIds=new HashSet<>();
        }
        if(StrUtil.isNotEmpty(userId)){
            this.executorIds.add(userId);
        }
    }

    public void addExecutorId(List<String> userIds){
        if(this.executorIds==null){
            this.executorIds=new HashSet<>();
        }
        if(CollUtil.isNotEmpty(userIds)){
            this.executorIds.addAll(userIds);
        }
    }


    public void addParam(MethodExecuteParamDTO item){
        if(this.params==null){
            this.params=new ArrayList<>();
        }
        if(item!=null){
            this.params.add(item);
        }
    }

}
