package com.xxh.cloud.rule.basic.context;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.rule.basic.constant.XXRuleRoamConstant;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;


/**
 *
 * kv结构
 *
 * key: 日期，basic
 * value: Map<String,XXRuleRoamHealthItem>
 * */
public class XXRuleRoam extends ConcurrentHashMap<String, Object> {


    /**
     *  数据时间
     * */
    private Date ddt;

    /**
     * 初始化日健康数据
     * @param dayDate ：日期（yyyyMMdd）
     * @param dayDataList : 患者每日数据列表
     * key：yyyymmdd ,  val:   Map<String,XXRuleRoamHealthItem>
     * */
    public void initDayHealth(String dayDate, List<XXRuleRoamHealthItem> dayDataList){
        if(CollUtil.isEmpty(dayDataList)){
            return;
        }
        Map<String,XXRuleRoamHealthItem> dayData=new HashMap<>();
        dayDataList.forEach(item->{
            dayData.put(item.getMdc(),item);
        });
        this.put(dayDate,dayData);
    }

    /**
     * 初始化日健康数据
     * @param dayDate ：日期（yyyyMMdd）
     * @param dayDataMap : 患者每日数据列表
     * */
    public void initDayHealth(String dayDate, Map<String,XXRuleRoamHealthItem> dayDataMap){
        if(MapUtil.isEmpty(dayDataMap)){
            return;
        }
        this.put(dayDate,dayDataMap);
    }


    /**
     * 新增健康数据
     * @param map: yyyymmdd:List
     * */
    public void addHealthData(Map<String,List<XXRuleRoamHealthItem>> map){
        if(MapUtil.isEmpty(map)){
            return;
        }
        map.forEach((dayDate,dayDataList)->{
            addDayHealth(dayDate,dayDataList);
        });
    }



    /**
     * 新增日健康数据
     * @param dayDate ：日期（yyyyMMdd）
     * @param dayDataList : 患者每日数据列表
     * */
    public void addDayHealth(String dayDate, List<XXRuleRoamHealthItem> dayDataList){
        if(CollUtil.isEmpty(dayDataList)){
            return;
        }
        Object dayDataObj= this.get(dayDate);
        Map<String,XXRuleRoamHealthItem> dayData;

        if(dayDataObj==null){
            dayData=new HashMap<>();
            this.put(dayDate,dayData);
        }else {
            dayData=(Map<String,XXRuleRoamHealthItem>)dayDataObj;
        }

        dayDataList.forEach(item->{
            dayData.put(item.getMdc(),item);
        });

    }


    /**
     * 新增日健康数据
     * @param dayDate ：日期（yyyyMMdd）
     * @param dayDataItem : 患者每日数据列表
     * */
    public void addDayHealth(String dayDate, XXRuleRoamHealthItem dayDataItem){
        if(dayDataItem==null){
            return;
        }
        Object dayDataObj= this.get(dayDate);
        Map<String,XXRuleRoamHealthItem> dayData;
        if(dayDataObj==null){
            dayData=new HashMap<>();
            this.put(dayDate,dayData);
        }else {
            dayData=(Map<String,XXRuleRoamHealthItem>)dayDataObj;
        }

        dayData.put(dayDataItem.getMdc(),dayDataItem);
    }


    /**
     * 获取日健康数据
     * */
    public  Map<String,XXRuleRoamHealthItem> getDayHealth(String dayDate){
        if(StrUtil.isEmpty(dayDate)){
            return null;
        }
        Object dayDataObj= this.get(dayDate);
        if(dayDataObj==null){
        }
        if(dayDataObj!=null){
            return (Map<String,XXRuleRoamHealthItem>)dayDataObj;
        }
        return null;
    }



    /**
     * 新增基本信息
     * */
    public void addBasicHealthData(Map<String,XXRuleRoamHealthItem> basicData){
        this.put(XXRuleRoamConstant.Basic_key,basicData);
    }

    /**
     * 新增基本信息
     * */
    public void addBasicHealthData(String dataKey, XXRuleRoamHealthItem dayDataItem){
        if(dayDataItem==null){
            return;
        }
        Map<String,XXRuleRoamHealthItem> basicData= this.getBasicHealthData();

        if(basicData==null){
            basicData=new HashMap<>();
            this.put(XXRuleRoamConstant.Basic_key,basicData);
        }
        basicData.put(dataKey,dayDataItem);
    }



    /**
     * 获取基本信息
     * */
     public Map<String,XXRuleRoamHealthItem> getBasicHealthData(){
         Object dayDataObj= this.get(XXRuleRoamConstant.Basic_key);
         if(dayDataObj!=null){
             return (Map<String,XXRuleRoamHealthItem>)dayDataObj;
         }
         return null;
     }

    /**
     * 获取基本信息
     * */
     public XXRuleRoamHealthItem getBasicHealthData(String key){
         Map<String,XXRuleRoamHealthItem> dataMap=this.getBasicHealthData();
         if(CollUtil.isEmpty(dataMap)){
             return null;
         }
         return dataMap.get(key);
     }


    /**
     * 获取基本信息
     * */
    public Object getBasicHealthDataValue(String key){
        XXRuleRoamHealthItem dataItem = this.getBasicHealthData(key);
        if(dataItem==null){
            return null;
        }
        return dataItem.getMdv();
    }



    /**
     * 获取基本信息
     * */
     public String getBasicHealthDataValueToStr(String key){
         XXRuleRoamHealthItem dataItem = this.getBasicHealthData(key);
         if(dataItem==null){
             return null;
         }
         return Convert.toStr(dataItem.getMdv());
     }

    /**
     * 获取基本信息
     * */
    public Integer getBasicHealthDataValueToInt(String key){
        XXRuleRoamHealthItem dataItem = this.getBasicHealthData(key);
        if(dataItem==null){
            return null;
        }
        return Convert.toInt(dataItem.getMdv());
    }

     /**
      * 获取基本信息 : 患者ID
      * */
     public String getPatientId(){
        return this.getBasicHealthDataValueToStr(XXRuleRoamConstant.Basic_Key_patient_id);
     }

    /**
     * 获取基本信息 : 患者姓名
     * */
    public String getPatientName(){
        return this.getBasicHealthDataValueToStr(XXRuleRoamConstant.Basic_Key_patient_name);
    }


    /**
     *
     * */
    public void setDataDate(Date date){
        this.ddt=date;
    }
    /**
     *
     * */
    public Date getDataDate(){
        return this.ddt;
    }



}
