package com.xxh.cloud.rule.framework.modules.action.biz.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.log.bean.constant.ModuleTypeEnum;
import com.xxh.cloud.log.bean.constant.ObjectTypeEnum;
import com.xxh.cloud.log.bean.constant.OperationTypeEnum;
import com.xxh.cloud.log.utils.OperationLogUtils;
import com.xxh.cloud.rule.framework.modules.action.biz.RuleActionInfoBizService;
import com.xxh.cloud.rule.framework.modules.action.entity.dos.RuleActionInfo;
import com.xxh.cloud.rule.framework.modules.action.entity.dos.RuleActionMethodInfo;
import com.xxh.cloud.rule.framework.modules.action.entity.dos.RuleActionMethodParameterInfo;
import com.xxh.cloud.rule.framework.modules.action.entity.dto.RuleActionInfoListDTO;
import com.xxh.cloud.rule.framework.modules.action.entity.dto.RuleActionInfoQueryDTO;
import com.xxh.cloud.rule.framework.modules.action.entity.dto.RuleActionInfoSaveDTO;
import com.xxh.cloud.rule.framework.modules.action.entity.dto.RuleActionInfoUpdateDTO;
import com.xxh.cloud.rule.framework.modules.action.entity.vos.RuleActionVO;
import com.xxh.cloud.rule.framework.modules.action.helper.RuleActionHelper;
import com.xxh.cloud.rule.framework.modules.action.service.RuleActionInfoService;
import com.xxh.cloud.rule.framework.modules.action.service.RuleActionMethodInfoService;
import com.xxh.cloud.rule.framework.modules.action.service.RuleActionMethodParameterInfoService;
import com.xxh.cloud.rule.framework.modules.cache.helper.RuleCacheKeysHelper;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class RuleActionInfoBizServiceImpl implements RuleActionInfoBizService {

    @Autowired
    private RedisUtils redisUtils;
    @Autowired
    private RuleActionInfoService ruleActionInfoService;
    @Autowired
    private RuleActionMethodInfoService ruleActionMethodInfoService;
    @Autowired
    private RuleActionMethodParameterInfoService ruleActionMethodParameterInfoService;
    @Autowired
    private AdminUserClientService adminUserClientService;


    @Override
    public PageData<RuleActionVO> query(RuleActionInfoQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        IPage<RuleActionInfo> page = ruleActionInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<RuleActionInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getType()),"type",params.getType())
                        .eq(StrUtil.isNotEmpty(params.getProjectCode()),"project_code",params.getProjectCode())
                        .orderByDesc("id")
        );
        if (page == null || CollUtil.isEmpty(page.getRecords())) {
            return null;
        }
        Set<String> userIds=new HashSet<>();
        page.getRecords().forEach(item->{
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
        });
        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
        PageData<RuleActionVO> data= new PageConverter<RuleActionInfo, RuleActionVO>() {
            @Override
            public RuleActionVO converter(RuleActionInfo entity) {
                return RuleActionHelper.getRuleActionVO(entity,userMap);
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<RuleActionVO> list(RuleActionInfoListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<RuleActionInfo> list = ruleActionInfoService.list(
                new QueryWrapper<RuleActionInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getType()),"type",params.getType())
                        .eq(StrUtil.isNotEmpty(params.getProjectCode()),"project_code",params.getProjectCode())
                        .orderByDesc("id")
        );
        if (CollUtil.isEmpty(list)) {
            return null;
        }
        Set<String> userIds=new HashSet<>();
        list.forEach(item->{
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
        });
        Map<String, AdminUserBO>  userMap= adminUserClientService.getUserMapByIds(userIds);
        List<RuleActionVO> data=new ArrayList<>();
        list.forEach(item->{
            data.add(RuleActionHelper.getRuleActionVO(item,userMap));
        });
        return data;
    }

    @Override
    public RuleActionVO save(RuleActionInfoSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        long num= ruleActionInfoService.count(
                new QueryWrapper<RuleActionInfo>()
                        .eq("id",params.getId())
                        .eq("rows_status", YN.YES.getCode())
        );
        if(num>0){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_EXISTS,"动作编号已存在");
        }
        RuleActionInfo entity=new RuleActionInfo();
        entity.setId(params.getId());
        entity.setProjectCode(params.getProjectCode());
        entity.setName(params.getName());
        entity.setType(params.getType());
        entity.setRemark(params.getRemark());
        entity.setRowsStatus(YN.YES.getCode());
        ruleActionInfoService.save(entity);


        Map<String, Object> logContent=new HashMap<>();
        logContent.put("所属项目:",params.getProjectCode());
        logContent.put("编号",entity.getId());
        logContent.put("名称",entity.getName());
        OperationLogUtils.addOpLogGen(OperationTypeEnum.OP_Add, ModuleTypeEnum.MT_Rule, ObjectTypeEnum.OT_RuleAction,entity.getId(),logContent);
        return RuleActionHelper.getRuleActionVO(entity);
    }

    @Override
    public RuleActionVO update(RuleActionInfoUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        RuleActionInfo entity=ruleActionInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        Map<String, Object> logContent=new HashMap<>();
        logContent.put("编号",entity.getId());
        logContent.put("原名称",entity.getName());

        entity.setName(params.getName());
        entity.setRemark(params.getRemark());
        entity.setType(params.getType());
        ruleActionInfoService.updateById(entity);
        redisUtils.delete(RuleCacheKeysHelper.getRuleActionKey(entity.getId()));

        logContent.put("新名称",entity.getName());
        OperationLogUtils.addOpLogGen(OperationTypeEnum.OP_Edit, ModuleTypeEnum.MT_Rule, ObjectTypeEnum.OT_RuleAction,entity.getId(),logContent);

        return RuleActionHelper.getRuleActionVO(entity);
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        RuleActionInfo entity=ruleActionInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        ruleActionMethodInfoService.remove(
                new QueryWrapper<RuleActionMethodInfo>()
                        .eq("action_id",entity.getId())
        );
        ruleActionMethodParameterInfoService.remove(
                new QueryWrapper<RuleActionMethodParameterInfo>()
                        .eq("action_id",entity.getId())
        );
        redisUtils.delete(RuleCacheKeysHelper.getRuleActionKey(entity.getId()));
        ruleActionInfoService.removeById(id);

        Map<String, Object> logContent=new HashMap<>();
        logContent.put("编号",entity.getId());
        logContent.put("名称",entity.getName());
        OperationLogUtils.addOpLogGen(OperationTypeEnum.OP_Del, ModuleTypeEnum.MT_Rule, ObjectTypeEnum.OT_RuleAction,entity.getId(),logContent);
    }

    @Override
    public RuleActionVO infoById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        RuleActionInfo entity=ruleActionInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return this.info(entity,false);
    }

    @Override
    public RuleActionVO info2NullById(String id) {
        if(StrUtil.isEmpty(id)){
            return null;
        }
        RuleActionInfo entity=ruleActionInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            return null;
        }
        return this.info(entity,false);
    }

    @Override
    public RuleActionVO infoFullById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        RuleActionInfo entity=ruleActionInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return this.info(entity,true);
    }


    private RuleActionVO info(RuleActionInfo entity,boolean isFull){
        Set<String> userIds=new HashSet<>();
        if(StrUtil.isNotEmpty(entity.getCreator())){
            userIds.add(entity.getCreator());
        }
        if(StrUtil.isNotEmpty(entity.getUpdater())){
            userIds.add(entity.getUpdater());
        }
        Map<String, AdminUserBO>  userMap= adminUserClientService.getUserMapByIds(userIds);

        if(isFull){
            //todo:
//            ruleActionMethodInfoService.list()
        }else {

        }
        return RuleActionHelper.getRuleActionVO(entity,userMap);
    }
}
