package com.xxh.cloud.rule.framework.modules.action.biz.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.log.bean.constant.ModuleTypeEnum;
import com.xxh.cloud.log.bean.constant.ObjectTypeEnum;
import com.xxh.cloud.log.bean.constant.OperationTypeEnum;
import com.xxh.cloud.log.utils.OperationLogUtils;
import com.xxh.cloud.rule.framework.modules.action.biz.RuleActionMethodInfoBizService;
import com.xxh.cloud.rule.framework.modules.action.entity.dos.RuleActionMethodInfo;
import com.xxh.cloud.rule.framework.modules.action.entity.dos.RuleActionMethodParameterInfo;
import com.xxh.cloud.rule.framework.modules.action.entity.dto.RuleActionMethodListDTO;
import com.xxh.cloud.rule.framework.modules.action.entity.dto.RuleActionMethodSaveDTO;
import com.xxh.cloud.rule.framework.modules.action.entity.dto.RuleActionMethodUpdateDTO;
import com.xxh.cloud.rule.framework.modules.action.entity.vos.RuleActionMethodVO;
import com.xxh.cloud.rule.framework.modules.action.entity.vos.RuleActionVO;
import com.xxh.cloud.rule.framework.modules.action.helper.RuleActionHelper;
import com.xxh.cloud.rule.framework.modules.action.helper.RuleActionMethodHelper;
import com.xxh.cloud.rule.framework.modules.action.service.RuleActionMethodInfoService;
import com.xxh.cloud.rule.framework.modules.action.service.RuleActionMethodParameterInfoService;
import com.xxh.cloud.rule.framework.modules.cache.helper.RuleCacheKeysHelper;
import com.xxh.cloud.rule.framework.modules.cache.service.RuleActionCacheService;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class RuleActionMethodInfoBizServiceImpl implements RuleActionMethodInfoBizService {


    @Autowired
    private RuleActionMethodInfoService ruleActionMethodInfoService;

    @Autowired
    private RuleActionMethodParameterInfoService ruleActionMethodParameterInfoService;

    @Autowired
    private RuleActionCacheService ruleActionCacheService;

    @Autowired
    private AdminUserClientService adminUserClientService;

    @Autowired
    private RedisUtils redisUtils;


    @Override
    public List<RuleActionMethodVO> list(RuleActionMethodListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<RuleActionMethodInfo> list = ruleActionMethodInfoService.list(new QueryWrapper<RuleActionMethodInfo>()
                .eq(StrUtil.isNotEmpty(params.getActionId()),"action_id",params.getActionId())
                .in(CollUtil.isNotEmpty(params.getActionIds()),"action_id",params.getActionIds())

                .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())

                .eq(StrUtil.isNotEmpty(params.getCode()),"code",params.getCode())
                .in(CollUtil.isNotEmpty(params.getCodes()),"code",params.getCodes())

                .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())

                .eq(StrUtil.isNotEmpty(params.getProjectCode()),"project_code",params.getProjectCode())

                .orderByDesc("action_id")
        );
        if(CollUtil.isEmpty(list)){
            return null;
        }
        Set<String> userIds=new HashSet<>();
        Set<String> actionIds=new HashSet<>();
        Set<String> methodIds=new HashSet<>();

        list.forEach(item->{
            methodIds.add(item.getId());
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
            if(StrUtil.isNotEmpty(item.getActionId())){
                actionIds.add(item.getActionId());
            }
        });
        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
        Map<String, RuleActionVO> actionMap= ruleActionCacheService.getMapByIds(actionIds);

        List<RuleActionMethodVO> data = new ArrayList<>();

        if(params.isFull()){
            List<RuleActionMethodParameterInfo> methodParamList= ruleActionMethodParameterInfoService.list(
                    new QueryWrapper<RuleActionMethodParameterInfo>()
                            .in("method_id",methodIds)
            );

            list.forEach(item -> {
                RuleActionMethodVO ram= RuleActionMethodHelper.getRuleActionMethodVO(item,methodParamList,actionMap,userMap);
                data.add(ram);
            });
        }else {
            list.forEach(item -> {
                data.add(RuleActionMethodHelper.getRuleActionMethodVO(item,actionMap,userMap));
            });
        }

        return data;
    }


    @Override
    public RuleActionMethodVO save(RuleActionMethodSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        long num= ruleActionMethodInfoService.count(new QueryWrapper<RuleActionMethodInfo>()
                .eq("action_id",params.getActionId())
                .eq("code",params.getCode())
        );
        if(num>0){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_EXISTS,"方法已存在");
        }

        RuleActionMethodInfo entity =new RuleActionMethodInfo();
        entity.setId(RuleActionHelper.getRuleActionMethodId());
        entity.setProjectCode(params.getProjectCode());
        entity.setActionId(params.getActionId());
        entity.setCode(params.getCode());
        entity.setName(params.getName());
        entity.setRemark(params.getRemark());

        Map<String, Object> logContent=new HashMap<>();
        logContent.put("所属应用",entity.getProjectCode());
        logContent.put("编号",entity.getCode());
        logContent.put("名称",entity.getName());

        List<RuleActionMethodParameterInfo> paramsList=new ArrayList<>();
        if(CollUtil.isNotEmpty(params.getInParams())){
            params.getInParams().forEach(item->{
                RuleActionMethodParameterInfo paramEntity=new RuleActionMethodParameterInfo();
                paramEntity.setId(RuleActionHelper.getRuleActionMethodParameterId());
                paramEntity.setActionId(entity.getActionId());
                paramEntity.setMethodId(entity.getId());
                paramEntity.setMethodCode(entity.getCode());
                paramEntity.setCode(item.getCode());
                paramEntity.setName(item.getName());
                paramEntity.setRemark(item.getRemark());
                paramEntity.setType(item.getType());
                if(item.getParaCfg()!=null){
                    paramEntity.setParaCfg(JSON.toJSONString(item.getParaCfg()));
                }
                paramEntity.setIsNeed(item.getIsNeed());
                paramEntity.setParaInputValType(item.getParaInputValType());
                paramEntity.setDefVal(item.getDefVal());
                paramEntity.setIsShow(item.getIsShow());
                paramsList.add(paramEntity);
                logContent.put("参数-"+paramEntity.getCode(),StrUtil.format("编号:{},名称:{}",paramEntity.getCode(), paramEntity.getName()));
            });
        }

        ruleActionMethodInfoService.save(entity);
        if(paramsList.size()>0){
            ruleActionMethodParameterInfoService.saveBatch(paramsList);
        }


        OperationLogUtils.addOpLogGen(OperationTypeEnum.OP_Add, ModuleTypeEnum.MT_Rule, ObjectTypeEnum.OT_RuleActionMethod,entity.getId(),logContent);

        return RuleActionMethodHelper.getRuleActionMethodVO(entity);
    }

    @Override
    public RuleActionMethodVO update(RuleActionMethodUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        RuleActionMethodInfo entity = ruleActionMethodInfoService.getById(params.getId());
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        Map<String, Object> logContent=new HashMap<>();
        logContent.put("原编号",entity.getCode());
        logContent.put("原名称",entity.getName());

        entity.setName(params.getName());
        entity.setRemark(params.getRemark());

        logContent.put("新编号",entity.getCode());
        logContent.put("新名称",entity.getName());

        List<RuleActionMethodParameterInfo> paramsList=new ArrayList<>();
        if(CollUtil.isNotEmpty(params.getInParams())){
            params.getInParams().forEach(item->{
                RuleActionMethodParameterInfo paramEntity=new RuleActionMethodParameterInfo();
                paramEntity.setId(RuleActionHelper.getRuleActionMethodParameterId());
                paramEntity.setActionId(entity.getActionId());
                paramEntity.setMethodId(entity.getId());
                paramEntity.setMethodCode(entity.getCode());
                paramEntity.setCode(item.getCode());
                paramEntity.setName(item.getName());
                paramEntity.setRemark(item.getRemark());
                paramEntity.setType(item.getType());
                if(item.getParaCfg()!=null){
                    paramEntity.setParaCfg(JSON.toJSONString(item.getParaCfg()));
                }
                paramEntity.setIsNeed(item.getIsNeed());
                paramEntity.setParaInputValType(item.getParaInputValType());
                paramEntity.setDefVal(item.getDefVal());
                paramEntity.setIsShow(item.getIsShow());
                paramsList.add(paramEntity);
                logContent.put("新参数-"+paramEntity.getCode(),StrUtil.format("编号:{},名称:{}",paramEntity.getCode(), paramEntity.getName()));
            });
        }
        ruleActionMethodInfoService.updateById(entity);
        if(paramsList.size()>0){
            ruleActionMethodParameterInfoService.remove(
                    new QueryWrapper<RuleActionMethodParameterInfo>()
                            .eq("method_id",entity.getId())
            );
            ruleActionMethodParameterInfoService.saveBatch(paramsList);
        }
        redisUtils.delete(RuleCacheKeysHelper.getRuleActionMethodKey(entity.getId()));

        OperationLogUtils.addOpLogGen(OperationTypeEnum.OP_Edit, ModuleTypeEnum.MT_Rule, ObjectTypeEnum.OT_RuleActionMethod,entity.getId(),logContent);


        return RuleActionMethodHelper.getRuleActionMethodVO(entity);
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        RuleActionMethodInfo entity=ruleActionMethodInfoService.getById(id);
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        ruleActionMethodParameterInfoService.remove(
                new QueryWrapper<RuleActionMethodParameterInfo>()
                        .eq("method_id",entity.getId())
        );
        ruleActionMethodInfoService.removeById(id);
        redisUtils.delete(RuleCacheKeysHelper.getRuleActionMethodKey(entity.getId()));

        Map<String, Object> logContent=new HashMap<>();
        logContent.put("编号",entity.getCode());
        logContent.put("名称",entity.getName());
        OperationLogUtils.addOpLogGen(OperationTypeEnum.OP_Del, ModuleTypeEnum.MT_Rule, ObjectTypeEnum.OT_RuleActionMethod,entity.getId(),logContent);





    }

    @Override
    public RuleActionMethodVO infoFullById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        RuleActionMethodInfo entity=ruleActionMethodInfoService.getById(id);
        if(entity==null ){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return this.info(entity);
    }

    @Override
    public RuleActionMethodVO info2NullFullById(String id) {
        if(StrUtil.isEmpty(id)){
            return null;
        }
        RuleActionMethodInfo entity=ruleActionMethodInfoService.getById(id);
        if(entity==null){
            return null;
        }
        return this.info(entity);
    }

    @Override
    public RuleActionMethodVO infoFullByCode(String actionId, String code) {
        if (StrUtil.isEmpty(actionId) || StrUtil.isEmpty(code)) {
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"编号不能为空");
        }
        RuleActionMethodInfo entity = ruleActionMethodInfoService.getOne(new QueryWrapper<RuleActionMethodInfo>()
                .eq("code",code)
                .eq("action_id", actionId)
        );
        if(entity==null ){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return this.info(entity);
    }

    private RuleActionMethodVO info(RuleActionMethodInfo entity){

        Set<String> userIds=new HashSet<>();
        Set<String> actionIds=new HashSet<>();

        if(StrUtil.isNotEmpty(entity.getCreator())){
            userIds.add(entity.getCreator());
        }
        if(StrUtil.isNotEmpty(entity.getUpdater())){
            userIds.add(entity.getUpdater());
        }
        if(StrUtil.isNotEmpty(entity.getActionId())){
            actionIds.add(entity.getActionId());
        }

        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
        Map<String, RuleActionVO> actionMap= ruleActionCacheService.getMapByIds(actionIds);

        List<RuleActionMethodParameterInfo> methodParamList= ruleActionMethodParameterInfoService.list(
                new QueryWrapper<RuleActionMethodParameterInfo>()
                        .eq("method_id",entity.getId())
        );

        return RuleActionMethodHelper.getRuleActionMethodVO(entity,methodParamList,actionMap,userMap);
    }



}
