package com.xxh.cloud.rule.framework.modules.rule.biz.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.tree.TreeUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.rule.framework.modules.cache.helper.RuleCacheKeysHelper;
import com.xxh.cloud.rule.framework.modules.rule.biz.RuleCategoryInfoBizService;
import com.xxh.cloud.rule.framework.modules.rule.entity.dos.RuleCategoryInfo;
import com.xxh.cloud.rule.framework.modules.rule.entity.dos.RuleConfigInfo;
import com.xxh.cloud.rule.framework.modules.rule.entity.dto.RuleCategoryListDTO;
import com.xxh.cloud.rule.framework.modules.rule.entity.dto.RuleCategorySaveDTO;
import com.xxh.cloud.rule.framework.modules.rule.entity.dto.RuleCategoryTreeDTO;
import com.xxh.cloud.rule.framework.modules.rule.entity.dto.RuleCategoryUpdateDTO;
import com.xxh.cloud.rule.framework.modules.rule.entity.vos.RuleCategoryVO;
import com.xxh.cloud.rule.framework.modules.rule.helper.RuleCategoryHelper;
import com.xxh.cloud.rule.framework.modules.rule.service.RuleCategoryInfoService;
import com.xxh.cloud.rule.framework.modules.rule.service.RuleConfigInfoService;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class RuleCategoryInfoBizServiceImpl implements RuleCategoryInfoBizService {


    @Autowired
    private RuleCategoryInfoService ruleCategoryInfoService;
    @Autowired
    private AdminUserClientService adminUserClientService;
    @Autowired
    private RedisUtils redisUtils;
    @Autowired
    private RuleConfigInfoService ruleConfigInfoService;



    @Override
    public List<RuleCategoryVO> tree(RuleCategoryTreeDTO params) {
        List<RuleCategoryInfo> list= ruleCategoryInfoService.list();
        if(CollUtil.isEmpty(list)){
            return null;
        }

        Set<String> userIds=new HashSet<>();
        list.forEach(item->{
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
        });
        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);

        List<RuleCategoryVO> dataList=new ArrayList<>();
        list.forEach(item->{
            dataList.add(RuleCategoryHelper.getRuleCategoryVO(item,userMap));
        });

        List<RuleCategoryVO> data= TreeUtils.build(dataList);
        return data;
    }

    @Override
    public List<RuleCategoryVO> list(RuleCategoryListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<RuleCategoryInfo> list= ruleCategoryInfoService.list(
                new QueryWrapper<RuleCategoryInfo>()
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
        );
        if(CollUtil.isEmpty(list)){
            return null;
        }
        Set<String> userIds=new HashSet<>();
        list.forEach(item->{
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }
        });
        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);

        List<RuleCategoryVO> data=new ArrayList<>();

        list.forEach(item->{
            data.add(RuleCategoryHelper.getRuleCategoryVO(item,userMap));
        });
        return data;
    }

    @Override
    public RuleCategoryVO save(RuleCategorySaveDTO params) {
        ValidatorUtils.validateEntity(params);
        RuleCategoryInfo entity=new RuleCategoryInfo();
        if(StrUtil.isEmpty(params.getId())){
            entity.setId(RuleCategoryHelper.getRuleCategoryId());
        }else {
            entity.setId(params.getId());
        }
        entity.setProjectCode(params.getProjectCode());
        entity.setName(params.getName());
        entity.setRemark(params.getRemark());
        entity.setOrders(params.getOrders());
        entity.setPid(params.getPid());
        ruleCategoryInfoService.save(entity);
        return RuleCategoryHelper.getRuleCategoryVO(entity);
    }

    @Override
    public RuleCategoryVO update(RuleCategoryUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        RuleCategoryInfo entity=ruleCategoryInfoService.getById(params.getId());
        if(entity==null ){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setName(params.getName());
        entity.setOrders(params.getOrders());
        entity.setRemark(params.getRemark());
        ruleCategoryInfoService.updateById(entity);
        redisUtils.delete(RuleCacheKeysHelper.getRuleCategoryKey(entity.getId()));
        return RuleCategoryHelper.getRuleCategoryVO(entity);
    }

    @Override
    public RuleCategoryVO infoById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        RuleCategoryInfo entity=ruleCategoryInfoService.getById(id);
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return this.info(entity);
    }

    @Override
    public RuleCategoryVO info2NullById(String id) {
        if(StrUtil.isEmpty(id)){
            return null;
        }
        RuleCategoryInfo entity=ruleCategoryInfoService.getById(id);
        if(entity==null){
            return null;
        }
        return this.info(entity);
    }

    private RuleCategoryVO info(RuleCategoryInfo entity){
        Set<String> userIds=new HashSet<>();
        if(StrUtil.isNotEmpty(entity.getCreator())){
            userIds.add(entity.getCreator());
        }
        if(StrUtil.isNotEmpty(entity.getUpdater())){
            userIds.add(entity.getUpdater());
        }
        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
        return RuleCategoryHelper.getRuleCategoryVO(entity,userMap);
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        RuleCategoryInfo entity=ruleCategoryInfoService.getById(id);
        if(entity==null ){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        long num= ruleConfigInfoService.count(
                new QueryWrapper<RuleConfigInfo>()
                        .eq("category_id",entity.getId())
                        .eq("rows_status", YN.YES.getCode())
        );
        if(num>0){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_EXISTS,"规则配置已存在");
        }
        ruleCategoryInfoService.removeById(id);
        redisUtils.delete(RuleCacheKeysHelper.getRuleCategoryKey(entity.getId()));
    }


}
