package com.xxh.cloud.rule.framework.modules.rule.biz.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.log.bean.constant.ModuleTypeEnum;
import com.xxh.cloud.log.bean.constant.ObjectTypeEnum;
import com.xxh.cloud.log.bean.constant.OperationTypeEnum;
import com.xxh.cloud.log.utils.OperationLogUtils;
import com.xxh.cloud.rule.framework.modules.rule.biz.RuleConfigBackUpInfoBizService;
import com.xxh.cloud.rule.framework.modules.rule.biz.RuleDecisionFlowConfigInfoBizService;
import com.xxh.cloud.rule.framework.modules.rule.biz.RuleDecisionTreeConfigInfoBizService;
import com.xxh.cloud.rule.framework.modules.rule.entity.dos.RuleConfigBackUpInfo;
import com.xxh.cloud.rule.framework.modules.rule.entity.dos.RuleConfigInfo;
import com.xxh.cloud.rule.framework.modules.rule.entity.dos.RuleDecisionFlowConfigInfo;
import com.xxh.cloud.rule.framework.modules.rule.entity.dto.RuleConfigBackUpAddDTO;
import com.xxh.cloud.rule.framework.modules.rule.entity.dto.RuleConfigBackUpListDTO;
import com.xxh.cloud.rule.framework.modules.rule.entity.vos.RuleConfigBackUpVO;
import com.xxh.cloud.rule.framework.modules.rule.helper.RuleConfigBackUpConvert;
import com.xxh.cloud.rule.framework.modules.rule.service.RuleConfigBackUpInfoService;
import com.xxh.cloud.rule.framework.modules.rule.service.RuleConfigInfoService;
import com.xxh.cloud.rule.framework.modules.rule.service.RuleDecisionFlowConfigInfoService;
import com.xxh.cloud.rule.framework.modules.rule.service.RuleDecisionTreeNodeConfigInfoService;
import io.swagger.annotations.ApiModelProperty;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Log4j2
@Service
public class RuleConfigBackUpInfoBizServiceImpl implements RuleConfigBackUpInfoBizService {

    @Autowired
    private RuleConfigBackUpInfoService ruleConfigBackUpInfoService;
    @Autowired
    private RuleConfigInfoService ruleConfigInfoService;
    @Autowired
    private RuleDecisionFlowConfigInfoService ruleDecisionFlowConfigInfoService;



    @Override
    public RuleConfigBackUpVO backUp(RuleConfigBackUpAddDTO params) {
        ValidatorUtils.validateEntity(params);
        RuleConfigInfo ruleConfig= ruleConfigInfoService.getById(params.getRuleConfigId());
        if(ruleConfig==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        RuleDecisionFlowConfigInfo flowConfig= ruleDecisionFlowConfigInfoService.getById(params.getRuleConfigId());
        if(flowConfig==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS,"规则为配置,不需备份");
        }

        RuleConfigBackUpInfo ruleConfigBackUpInfo= ruleConfigBackUpInfoService.getOne(
                new QueryWrapper<RuleConfigBackUpInfo>()
                        .select("max(rule_version_no) as ruleVersionNo")
                        .eq("rule_config_id",params.getRuleConfigId())
        );

        Integer version;
        if(ruleConfigBackUpInfo==null || ruleConfigBackUpInfo.getRuleVersionNo()==null){
            version=ruleConfig.getVersionNo();
        }else {
            version=ruleConfigBackUpInfo.getRuleVersionNo()+1;
        }

        RuleConfigBackUpInfo entity=new RuleConfigBackUpInfo();
        entity.setId(IdUtil.getSnowflake(5,2).nextIdStr());
        entity.setProjectCode(ruleConfig.getProjectCode());
        entity.setRuleConfigId(ruleConfig.getId());
        entity.setRuleVersionNo(version);
        entity.setCategoryId(ruleConfig.getCategoryId());
        entity.setName(ruleConfig.getName());
        entity.setType(ruleConfig.getType());
        entity.setBizType(ruleConfig.getBizType());
        entity.setEventId(ruleConfig.getEventId());
        entity.setRelType(ruleConfig.getRelType());
        entity.setRelTwoType(ruleConfig.getRelTwoType());
        entity.setRelId(ruleConfig.getRelId());
        entity.setSource(ruleConfig.getSource());
        entity.setBackUpRemark(params.getBackUpRemark());
        entity.setConfig(flowConfig.getContent());
        ruleConfigBackUpInfoService.save(entity);

        Map<String, Object> logContent=new HashMap<>();
        logContent.put("名称",entity.getName());
        logContent.put("版本号",entity.getRuleVersionNo());
        logContent.put("备注",entity.getBackUpRemark());
//        OperationLogUtils.addOpLogGen(OperationTypeEnum.OP_Add, ModuleTypeEnum.MT_Rule, ObjectTypeEnum.OT_RuleBackUp,entity.getId(),logContent);

        return RuleConfigBackUpConvert.getRuleConfigBackUpVOSimple(entity);
    }

    @Override
    public List<RuleConfigBackUpVO> list(RuleConfigBackUpListDTO params) {
        ValidatorUtils.validateEntity(params);

        List<RuleConfigBackUpInfo> list=ruleConfigBackUpInfoService.list(
                new QueryWrapper<RuleConfigBackUpInfo>()
                        .select("id as id " ,
                                "rule_config_id as ruleConfigId " ,
                                "rule_version_no as ruleVersionNo ",
                                "back_up_remark as backUpRemark " ,
                                "creator as creator " ,
                                "create_date as createDate " )
                        .eq("rule_config_id",params.getRuleConfigId())
                        .orderByDesc("rule_version_no")

        );

        List<RuleConfigBackUpVO> data=new ArrayList<>();
        if(CollUtil.isEmpty(list)){
            return data;
        }
        list.forEach(item->{
            data.add(RuleConfigBackUpConvert.getRuleConfigBackUpVOSimple(item));
        });

        return data;
    }

    @Override
    public RuleConfigBackUpVO infoById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        RuleConfigBackUpInfo entity= ruleConfigBackUpInfoService.getById(id);
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return RuleConfigBackUpConvert.getRuleConfigBackUpVO(entity);
    }

    @Override
    public void redo(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        RuleConfigBackUpInfo entity= ruleConfigBackUpInfoService.getById(id);
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        //todo:

    }


}
