package com.xxh.cloud.rule.framework.modules.rule.biz.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.metadata.client.api.StandardDiseaseClientService;
import com.xxh.cloud.metadata.client.api.StandardEventClientService;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardDiseaseBO;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardEventBO;
import com.xxh.cloud.rule.core.constant.RuleConfigTypeEnum;
import com.xxh.cloud.rule.core.context.XXRuleVariableRoom;
import com.xxh.cloud.rule.core.entity.flow.XXFlowNodeBO;
import com.xxh.cloud.rule.core.entity.flow.XXFlowRuleBO;
import com.xxh.cloud.rule.flow.variable.XXRuleFlowVariableExecutor;
import com.xxh.cloud.rule.framework.modules.cache.helper.RuleCacheKeysHelper;
import com.xxh.cloud.rule.framework.modules.cache.service.RuleCategoryCacheService;
import com.xxh.cloud.rule.framework.modules.rule.biz.RuleConfigInfoBizService;
import com.xxh.cloud.rule.framework.modules.rule.entity.dos.RuleConfigInfo;
import com.xxh.cloud.rule.framework.modules.rule.entity.dos.RuleConfigRelInfo;
import com.xxh.cloud.rule.framework.modules.rule.entity.dos.RuleDecisionFlowConfigInfo;
import com.xxh.cloud.rule.framework.modules.rule.entity.dto.*;
import com.xxh.cloud.rule.framework.modules.rule.entity.vos.RuleCategoryVO;
import com.xxh.cloud.rule.framework.modules.rule.entity.vos.RuleConfigVO;
import com.xxh.cloud.rule.framework.modules.rule.helper.RuleConfigConvert;
import com.xxh.cloud.rule.framework.modules.rule.helper.RuleConfigHelper;
import com.xxh.cloud.rule.framework.modules.rule.helper.RuleDecisionFlowConfigHelper;
import com.xxh.cloud.rule.framework.modules.rule.service.RuleConfigInfoService;
import com.xxh.cloud.rule.framework.modules.rule.service.RuleConfigRelInfoService;
import com.xxh.cloud.rule.framework.modules.rule.service.RuleDecisionFlowConfigInfoService;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class RuleConfigInfoBizServiceImpl implements RuleConfigInfoBizService {

    @Autowired
    private RuleConfigInfoService ruleConfigInfoService;
    @Autowired
    private AdminUserClientService adminUserClientService;
    @Autowired
    private RuleCategoryCacheService ruleCategoryCacheService;
    @Autowired
    private RedisUtils redisUtils;

    @Autowired
    private RuleDecisionFlowConfigInfoService ruleDecisionFlowConfigInfoService;
    @Autowired
    private RuleConfigRelInfoService ruleConfigRelInfoService;
    @Autowired
    private StandardEventClientService standardEventClientService;

    @Autowired
    private StandardDiseaseClientService standardDiseaseClientService;



    @Override
    public PageData<RuleConfigVO> query(RuleConfigInfoQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        IPage<RuleConfigInfo> page = ruleConfigInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<RuleConfigInfo>()
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .eq(StrUtil.isNotEmpty(params.getCategoryId()),"category_id",params.getCategoryId())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getType()),"type",params.getType())
                        .eq(StrUtil.isNotEmpty(params.getBizType()),"biz_type",params.getBizType())

                        .eq(StrUtil.isNotEmpty(params.getEventId()),"event_id",params.getEventId())

                        .eq(StrUtil.isNotEmpty(params.getRelType()),"rel_type",params.getRelType())
                        .eq(StrUtil.isNotEmpty(params.getRelTwoType()),"rel_two_type",params.getRelTwoType())
                        .like(StrUtil.isNotEmpty(params.getRelId()),"rel_id",params.getRelId())

                        .eq(StrUtil.isNotEmpty(params.getStatus()),"status",params.getStatus())
                        .eq(StrUtil.isNotEmpty(params.getSource()),"source",params.getSource())
                        .eq(StrUtil.isNotEmpty(params.getProjectCode()),"project_code",params.getProjectCode())

                        .eq(StrUtil.isNotEmpty(params.getExtendOne()),"extend_one",params.getExtendOne())
                        .eq(StrUtil.isNotEmpty(params.getExtendTwo()),"extend_two",params.getExtendTwo())
                        .eq(StrUtil.isNotEmpty(params.getExtendThree()),"extend_three",params.getExtendThree())


                        .eq("rows_status", YN.YES.getCode())
                        .orderByDesc("orders")
        );


        if (page == null || CollUtil.isEmpty(page.getRecords())) {
            return null;
        }

        Set<String> categoryIds=new HashSet<>();
        Set<String> userIds=new HashSet<>();
        Set<String> eventIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();


        page.getRecords().forEach(item->{
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }

            if(StrUtil.isNotEmpty(item.getCategoryId())){
                categoryIds.add(item.getCategoryId());
            }

            if(StrUtil.isNotEmpty(item.getEventId())){
                eventIds.add(item.getEventId());
            }

            if(StrUtil.isNotEmpty(item.getRelId())){
                Set<String> _ids= StrJoinSplitUtils.split2Set(item.getRelId());
                diseaseIds.addAll(_ids);
            }

        });

        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
        Map<String, RuleCategoryVO> categoryMap= ruleCategoryCacheService.getMapByIds(categoryIds);
        Map<String, StandardEventBO> eventMap= standardEventClientService.getMap(eventIds);
        Map<String, StandardDiseaseBO> diseaseMap= standardDiseaseClientService.getMap(diseaseIds);

        PageData<RuleConfigVO> data= new PageConverter<RuleConfigInfo, RuleConfigVO>() {
            @Override
            public RuleConfigVO converter(RuleConfigInfo entity) {
                return RuleConfigConvert.getRuleConfigVO(entity,categoryMap,userMap,eventMap,diseaseMap);
            }
        }.getPageData(page);

        return data;
    }




    @Override
    public List<RuleConfigVO> list(RuleConfigInfoListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<RuleConfigInfo> list = ruleConfigInfoService.list(
                new QueryWrapper<RuleConfigInfo>()
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .eq(StrUtil.isNotEmpty(params.getCategoryId()),"category_id",params.getCategoryId())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getType()),"type",params.getType())
                        .eq(StrUtil.isNotEmpty(params.getBizType()),"biz_type",params.getBizType())


                        .eq(StrUtil.isNotEmpty(params.getEventId()),"event_id",params.getEventId())

                        .eq(StrUtil.isNotEmpty(params.getRelType()),"rel_type",params.getRelType())
                        .eq(StrUtil.isNotEmpty(params.getRelTwoType()),"rel_two_type",params.getRelTwoType())
                        .like(StrUtil.isNotEmpty(params.getRelId()),"rel_id",params.getRelId())

                        .eq(StrUtil.isNotEmpty(params.getStatus()),"status",params.getStatus())
                        .eq(StrUtil.isNotEmpty(params.getSource()),"source",params.getSource())
                        .eq(StrUtil.isNotEmpty(params.getProjectCode()),"project_code",params.getProjectCode())

                        .eq(StrUtil.isNotEmpty(params.getExtendOne()),"extend_one",params.getExtendOne())
                        .eq(StrUtil.isNotEmpty(params.getExtendTwo()),"extend_two",params.getExtendTwo())
                        .eq(StrUtil.isNotEmpty(params.getExtendThree()),"extend_three",params.getExtendThree())

                        .eq("rows_status", YN.YES.getCode())
                        .orderByDesc("orders")
        );

        if(CollUtil.isEmpty(list)){
            return null;
        }

        Set<String> categoryIds=new HashSet<>();
        Set<String> userIds=new HashSet<>();
        Set<String> eventIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();

        list.forEach(item->{
            if(StrUtil.isNotEmpty(item.getCreator())){
                userIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                userIds.add(item.getUpdater());
            }

            if(StrUtil.isNotEmpty(item.getCategoryId())){
                categoryIds.add(item.getCategoryId());
            }

            if(StrUtil.isNotEmpty(item.getEventId())){
                eventIds.add(item.getEventId());
            }

            if(StrUtil.isNotEmpty(item.getRelId())){
                Set<String> _ids= StrJoinSplitUtils.split2Set(item.getRelId());
                diseaseIds.addAll(_ids);
            }

        });

        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
        Map<String, RuleCategoryVO> categoryMap= ruleCategoryCacheService.getMapByIds(categoryIds);
        Map<String, StandardEventBO> eventMap= standardEventClientService.getMap(eventIds);
        Map<String, StandardDiseaseBO> diseaseMap= standardDiseaseClientService.getMap(diseaseIds);


        List<RuleConfigVO> data=new ArrayList<>();

        if(CollUtil.isNotEmpty(list)){
            list.forEach(item->{
                RuleConfigVO dataItem= RuleConfigConvert.getRuleConfigVO(item,categoryMap,userMap,eventMap,diseaseMap);
                if(dataItem!=null){
                    data.add(dataItem);
                }
            });
        }

        return data;
    }

    @Override
    public RuleConfigVO save(RuleConfigInfoSaveDTO params) {
        ValidatorUtils.validateEntity(params);

        RuleConfigInfo entity= new RuleConfigInfo();
        if(StrUtil.isEmpty(params.getId())){
            entity.setId(RuleConfigConvert.getRuleConfigId());
        }else {
            entity.setId(params.getId());
        }
        entity.setProjectCode(params.getProjectCode());
        entity.setCategoryId(params.getCategoryId());
        entity.setName(params.getName());
        entity.setType(params.getType());
        entity.setBizType(params.getBizType());
        entity.setEventId(params.getEventId());
        entity.setRelType(params.getRelType());
        entity.setRelTwoType(params.getRelTwoType());
        entity.setRelId(StrJoinSplitUtils.join(params.getRelId()));
        entity.setStatus(YN.NO.getCode());
        entity.setRemark(params.getRemark());
        entity.setOrders(params.getOrders());
        entity.setSource(params.getSource());
        entity.setVersionNo(1);
        entity.setExtendOne(params.getExtendOne());
        entity.setExtendTwo(params.getExtendTwo());
        entity.setExtendThree(params.getExtendThree());
        entity.setRowsStatus(YN.YES.getCode());
        ruleConfigInfoService.save(entity);
        return RuleConfigConvert.getRuleConfigVO(entity);
    }

    @Override
    public RuleConfigVO update(RuleConfigInfoUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        RuleConfigInfo entity= ruleConfigInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(YN.YES.getCode().equals(entity.getStatus())){
            throw  new BusinessException(BusinessBaseErrorEnum.Illegal_Operation_ERROR);
        }
        entity.setCategoryId(params.getCategoryId());
        entity.setName(params.getName());

        entity.setEventId(params.getEventId());
        entity.setRelType(params.getRelType());
        entity.setRelTwoType(params.getRelTwoType());
        entity.setRelId(StrJoinSplitUtils.join(params.getRelId()));
        entity.setRemark(params.getRemark());
        entity.setOrders(params.getOrders());
        entity.setSource(params.getSource());
        entity.setExtendOne(params.getExtendOne());
        entity.setExtendTwo(params.getExtendTwo());
        entity.setExtendThree(params.getExtendThree());
        ruleConfigInfoService.updateById(entity);
        return RuleConfigConvert.getRuleConfigVO(entity);
    }

    @Override
    public RuleConfigVO updateStatus(RuleConfigInfoUpdateStatusDTO params) {
        ValidatorUtils.validateEntity(params);
        RuleConfigInfo entity= ruleConfigInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(params.getStatus().equals(entity.getStatus())){
            return RuleConfigConvert.getRuleConfigVO(entity);
        }

        entity.setStatus(params.getStatus());
        ruleConfigInfoService.updateById(entity);

        //todo:delete
        redisUtils.delete(RuleCacheKeysHelper.getRuleScriptKey(entity.getId()));
        redisUtils.delete(RuleCacheKeysHelper.getRuleScriptRunMainKey());


        //上架:引用参数
        if(YN.YES.getCode().equals(params.getStatus())){
            if(RuleConfigTypeEnum.RCT_Decision_Flow.getCode().equals(entity.getType())){

                RuleDecisionFlowConfigInfo flowConfig= ruleDecisionFlowConfigInfoService.getById(entity.getId());
                if(flowConfig==null || StrUtil.isEmpty(flowConfig.getContent())){
                    throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS,"规则配置错误");
                }
                XXFlowNodeBO ruleNode= JSON.parseObject(flowConfig.getContent(), XXFlowNodeBO.class);

                XXFlowRuleBO flowRule=new XXFlowRuleBO();
                flowRule.setId(entity.getId());
                flowRule.setName(entity.getName());
                flowRule.setVersion(entity.getVersionNo());
                flowRule.setConfig(ruleNode);

                XXRuleFlowVariableExecutor ruleFlowVariableExecutor=new XXRuleFlowVariableExecutor();
                XXRuleVariableRoom ruleVariableRoom= ruleFlowVariableExecutor.execute(flowRule,false);

                List<RuleConfigRelInfo> ruleConfigRelList= RuleConfigHelper.getRuleConfigRelInfo_Add(entity.getId(),ruleVariableRoom);

                ruleConfigRelInfoService.remove(new QueryWrapper<RuleConfigRelInfo>().eq("rule_config_id",entity.getId()));
                if(CollUtil.isNotEmpty(ruleConfigRelList)){
                    ruleConfigRelInfoService.saveBatch(ruleConfigRelList);
                }


            }else if(RuleConfigTypeEnum.RCT_Decision_Tree.getCode().equals(entity.getType())){
                //todo:暂不支持
            }
        }






        return RuleConfigConvert.getRuleConfigVO(entity);
    }

    @Override
    public RuleConfigVO infoById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        RuleConfigInfo entity=ruleConfigInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return this.info(entity);
    }


    private RuleConfigVO info(RuleConfigInfo entity){
        Set<String> categoryIds=new HashSet<>();
        Set<String> userIds=new HashSet<>();
        Set<String> eventIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();

        if(StrUtil.isNotEmpty(entity.getCreator())){
            userIds.add(entity.getCreator());
        }
        if(StrUtil.isNotEmpty(entity.getUpdater())){
            userIds.add(entity.getUpdater());
        }
        if(StrUtil.isNotEmpty(entity.getCategoryId())){
            categoryIds.add(entity.getCategoryId());
        }
        if(StrUtil.isNotEmpty(entity.getEventId())){
            eventIds.add(entity.getEventId());
        }

        if(StrUtil.isNotEmpty(entity.getRelId())){
             diseaseIds= StrJoinSplitUtils.split2Set(entity.getRelId());
        }

        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
        Map<String, RuleCategoryVO> categoryMap= ruleCategoryCacheService.getMapByIds(categoryIds);

        Map<String, StandardEventBO> eventMap= standardEventClientService.getMap(eventIds);
        Map<String, StandardDiseaseBO> diseaseMap= standardDiseaseClientService.getMap(diseaseIds);

        return RuleConfigConvert.getRuleConfigVO(entity,categoryMap,userMap,eventMap,diseaseMap);
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        RuleConfigInfo entity=ruleConfigInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(YN.YES.getCode().equals(entity.getStatus())){
            throw  new BusinessException(BusinessBaseErrorEnum.Illegal_Operation_ERROR);
        }
        entity.setRowsStatus(YN.NO.getCode());
        ruleConfigInfoService.updateById(entity);
    }

    @Override
    public RuleConfigVO copy(RuleConfigInfoCopyDTO params) {
        ValidatorUtils.validateEntity(params);
        RuleConfigInfo entity=ruleConfigInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        RuleDecisionFlowConfigInfo configEntity= ruleDecisionFlowConfigInfoService.getById(params.getId());
        if(configEntity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }


        RuleConfigInfo dataEntity=new RuleConfigInfo();
        dataEntity.setId(RuleConfigConvert.getRuleConfigId());
        dataEntity.setProjectCode(entity.getProjectCode());
        dataEntity.setCategoryId(entity.getCategoryId());
        dataEntity.setName(entity.getName());
        dataEntity.setType(entity.getType());
        dataEntity.setBizType(entity.getBizType());
        dataEntity.setEventId(entity.getEventId());
        dataEntity.setRelType(entity.getRelType());
        dataEntity.setRelTwoType(entity.getRelTwoType());
        dataEntity.setRelId(entity.getRelId());
        dataEntity.setStatus(YN.NO.getCode());
        dataEntity.setRemark(entity.getRemark());
        dataEntity.setOrders(entity.getOrders());
        dataEntity.setSource(entity.getSource());
        dataEntity.setVersionNo(1);
        dataEntity.setRowsStatus(YN.YES.getCode());
        ruleConfigInfoService.save(dataEntity);

        RuleDecisionFlowConfigInfo dataConfig=new RuleDecisionFlowConfigInfo();
        dataConfig.setId(dataEntity.getId());
        dataConfig.setRuleConfigId(dataEntity.getId());
        dataConfig.setContent(configEntity.getContent());
        ruleDecisionFlowConfigInfoService.save(dataConfig);

        return RuleConfigConvert.getRuleConfigVO(entity);
    }


}
