package com.xxh.cloud.rule.framework.modules.rule.biz.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.rule.core.constant.RuleConfigTypeEnum;
import com.xxh.cloud.rule.framework.modules.action.service.RuleActionMethodParameterInfoService;
import com.xxh.cloud.rule.framework.modules.rule.biz.RuleDecisionTreeConfigInfoBizService;
import com.xxh.cloud.rule.framework.modules.rule.entity.dos.RuleConfigInfo;
import com.xxh.cloud.rule.framework.modules.rule.entity.dos.RuleDecisionTreeNodeConfigInfo;
import com.xxh.cloud.rule.framework.modules.rule.entity.dto.DecisionTreeConfigSave2EditDTO;
import com.xxh.cloud.rule.framework.modules.rule.entity.vos.RuleDecisionTreeConfigVO;
import com.xxh.cloud.rule.framework.modules.rule.helper.DecisionTreeNodeConfigConvert;
import com.xxh.cloud.rule.framework.modules.rule.helper.DecisionTreeNodeConfigHelper;
import com.xxh.cloud.rule.framework.modules.rule.service.RuleConfigInfoService;
import com.xxh.cloud.rule.framework.modules.rule.service.RuleDecisionTreeNodeConfigInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class RuleDecisionTreeConfigInfoBizServiceImpl implements RuleDecisionTreeConfigInfoBizService {


    @Autowired
    private RuleConfigInfoService ruleConfigInfoService;
    @Autowired
    private RuleActionMethodParameterInfoService ruleActionMethodParameterInfoService;
    @Autowired
    private RuleDecisionTreeNodeConfigInfoService ruleDecisionTreeNodeConfigInfoService;

    @Override
    public List<RuleDecisionTreeConfigVO> tree(String ruleConfigId) {
        List<RuleDecisionTreeNodeConfigInfo> list = ruleDecisionTreeNodeConfigInfoService.list(
                new QueryWrapper<RuleDecisionTreeNodeConfigInfo>()
                        .eq("rule_config_id",ruleConfigId)
        );
        if(CollUtil.isEmpty(list)){
            return null;
        }
        return DecisionTreeNodeConfigConvert.getTree(list,null);
    }

    @Override
    public void save2Update(DecisionTreeConfigSave2EditDTO params) {
        ValidatorUtils.validateEntity(params);
        this.validate(params);
        RuleConfigInfo entity= ruleConfigInfoService.getById(params.getRuleConfigId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus()) ||
                !RuleConfigTypeEnum.RCT_Decision_Tree.getCode().equals(entity.getType())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        if(YN.YES.getCode().equals(entity.getStatus())){
            throw  new BusinessException(BusinessBaseErrorEnum.Illegal_Operation_ERROR,"已发布不能操作");
        }

        if(!RuleConfigTypeEnum.RCT_Decision_Tree.getCode().equals(entity.getType())){
            throw  new BusinessException(BusinessBaseErrorEnum.Config_GET_ERROR,"不支持的规则类型不能操作");
        }

        List<RuleDecisionTreeNodeConfigInfo>  list = DecisionTreeNodeConfigHelper.getRuleDecisionTreeNodeConfigInfoList(entity,params.getNodes());
        ruleDecisionTreeNodeConfigInfoService.remove(new QueryWrapper<RuleDecisionTreeNodeConfigInfo>()
                .eq("rule_config_id",params.getRuleConfigId())
        );
        ruleDecisionTreeNodeConfigInfoService.saveBatch(list);
    }

    private void validate(DecisionTreeConfigSave2EditDTO params){
        if(CollUtil.isEmpty(params.getNodes())){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"节点不能为空");
        }
        //todo:
    }


}
