package com.xxh.cloud.rule.sys.manager.controller;


import cn.dev33.satoken.annotation.SaCheckPermission;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.rule.framework.modules.action.biz.RuleActionInfoBizService;
import com.xxh.cloud.rule.framework.modules.action.entity.dto.*;
import com.xxh.cloud.rule.framework.modules.action.entity.vos.RuleActionVO;
import com.xxh.cloud.rule.framework.modules.action.service.RuleActionInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

//@SaCheckPermission("sys:rule:action")
@Log4j2
@RestController
@RequestMapping("/sys/rule/action")
@Api(tags = "运营端,动作管理")
public class RuleActionManagerController {


    @Autowired
    private RuleActionInfoBizService ruleActionInfoService;


    @SaCheckPermission("sys:rule:action:query")
    @PostMapping("/query")
    @ApiOperation(value = "查询", response = RuleActionVO.class)
    public Result<PageData<RuleActionVO>> query(@RequestBody RuleActionInfoQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<RuleActionVO> data= ruleActionInfoService.query(params);
        return new Result<PageData<RuleActionVO>>().ok(data);
    }

    @SaCheckPermission("sys:rule:action:list")
    @PostMapping("/list")
    @ApiOperation(value = "列表", response = RuleActionVO.class)
    public Result<List<RuleActionVO>> list(@RequestBody RuleActionInfoListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<RuleActionVO> data= ruleActionInfoService.list(params);
        return new Result<List<RuleActionVO>>().ok(data);
    }

    @SaCheckPermission("sys:rule:action:save")
    @PostMapping("/save")
    @ApiOperation(value = "新增", response = RuleActionVO.class)
    public Result<RuleActionVO> save(@RequestBody RuleActionInfoSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        RuleActionVO data= ruleActionInfoService.save(params);
        return new Result<RuleActionVO>().ok(data);
    }

    @SaCheckPermission("sys:rule:action:update")
    @PostMapping("/update")
    @ApiOperation(value = "修改", response = RuleActionVO.class)
    public Result<RuleActionVO> update(@RequestBody RuleActionInfoUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        RuleActionVO data=ruleActionInfoService.update(params);
        return new Result<RuleActionVO>().ok(data);
    }

    @SaCheckPermission("sys:rule:action:del")
    @PostMapping("/del")
    @ApiOperation(value = "删除", response = String.class)
    public Result<String> del(@RequestBody RuleActionInfoByIdDTO params) {
        ValidatorUtils.validateEntity(params);
        ruleActionInfoService.delById(params.getId());
        return new Result<String>().ok("ok");
    }

    @SaCheckPermission("sys:rule:action:info")
    @PostMapping("/info")
    @ApiOperation(value = "获取", response = RuleActionVO.class)
    public  Result<RuleActionVO> info(@RequestBody RuleActionInfoByIdDTO params) {
        ValidatorUtils.validateEntity(params);
        RuleActionVO data= ruleActionInfoService.infoById(params.getId());
        return new Result<RuleActionVO>().ok(data);
    }






}
