package com.xxh.cloud.rule.sys.manager.controller;


import cn.dev33.satoken.annotation.SaCheckPermission;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.metadata.framework.modules.common.constant.enums.RuleMethodParamDataType;
import com.xxh.cloud.rule.framework.modules.action.biz.RuleActionMethodInfoBizService;
import com.xxh.cloud.rule.framework.modules.action.entity.dto.*;
import com.xxh.cloud.rule.framework.modules.action.entity.vos.RuleActionMethodVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

//@SaCheckPermission("sys:rule:method")
@Log4j2
@RestController
@RequestMapping("/sys/rule/method")
@Api(tags = "运营端,动作方法管理")
public class RuleActionMethodManagerController {

    @Autowired
    private RuleActionMethodInfoBizService ruleActionMethodInfoService;

    @SaCheckPermission("sys:rule:method:list")
    @PostMapping("/list")
    @ApiOperation(value = "列表", response = RuleActionMethodVO.class)
    public Result<List<RuleActionMethodVO>> list(@RequestBody RuleActionMethodListDTO params) {
        ValidatorUtils.validateEntity(params);

        List<RuleActionMethodVO> data= ruleActionMethodInfoService.list(params);
        return new Result<List<RuleActionMethodVO>>().ok(data);
    }

    @SaCheckPermission("sys:rule:method:save")
    @PostMapping("/save")
    @ApiOperation(value = "新增", response = RuleActionMethodVO.class)
    public Result<RuleActionMethodVO> save(@RequestBody RuleActionMethodSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        RuleActionMethodVO data= ruleActionMethodInfoService.save(params);
        return new Result<RuleActionMethodVO>().ok(data);
    }

    @SaCheckPermission("sys:rule:method:update")
    @PostMapping("/update")
    @ApiOperation(value = "修改", response = RuleActionMethodVO.class)
    public Result<RuleActionMethodVO> update(@RequestBody RuleActionMethodUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        RuleActionMethodVO data=ruleActionMethodInfoService.update(params);
        return new Result<RuleActionMethodVO>().ok(data);
    }

    @SaCheckPermission("sys:rule:method:del")
    @PostMapping("/del")
    @ApiOperation(value = "删除", response = String.class)
    public Result<String> del(@RequestBody RuleActionMethodByIdDTO params) {
        ValidatorUtils.validateEntity(params);
        ruleActionMethodInfoService.delById(params.getId());
        return new Result<String>().ok("ok");
    }

    @SaCheckPermission("sys:rule:method:info")
    @PostMapping("/info")
    @ApiOperation(value = "获取", response = RuleActionMethodVO.class)
    public  Result<RuleActionMethodVO> info(@RequestBody RuleActionMethodByIdDTO params) {
        ValidatorUtils.validateEntity(params);
        RuleActionMethodVO data= ruleActionMethodInfoService.infoFullById(params.getId());
        return new Result<RuleActionMethodVO>().ok(data);
    }



    @SaCheckPermission("sys:rule:method:infoByCode")
    @PostMapping("/infoByCode")
    @ApiOperation(value = "获取", response = RuleActionMethodVO.class)
    public  Result<RuleActionMethodVO> infoByCode(@RequestBody RuleActionMethodByCodeDTO params) {
        ValidatorUtils.validateEntity(params);
        RuleActionMethodVO data= ruleActionMethodInfoService.infoFullByCode(params.getActionId(), params.getCode());
        return new Result<RuleActionMethodVO>().ok(data);
    }


}
