package com.xxh.cloud.rule.sys.manager.controller;


import cn.dev33.satoken.annotation.SaCheckPermission;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.rule.framework.modules.rule.biz.RuleCategoryInfoBizService;
import com.xxh.cloud.rule.framework.modules.rule.entity.dto.RuleCategoryByIdDTO;
import com.xxh.cloud.rule.framework.modules.rule.entity.dto.RuleCategorySaveDTO;
import com.xxh.cloud.rule.framework.modules.rule.entity.dto.RuleCategoryTreeDTO;
import com.xxh.cloud.rule.framework.modules.rule.entity.dto.RuleCategoryUpdateDTO;
import com.xxh.cloud.rule.framework.modules.rule.entity.vos.RuleCategoryVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

//@SaCheckPermission("sys:rule:category")
@Log4j2
@RestController
@RequestMapping("/sys/rule/category")
@Api(tags = "运营端,规则分类管理")
public class RuleCategoryManagerController {

    @Autowired
    private RuleCategoryInfoBizService ruleCategoryInfoService;


    @SaCheckPermission("sys:rule:category:tree")
    @PostMapping("/tree")
    @ApiOperation(value = "树", response = RuleCategoryVO.class)
    public Result<List<RuleCategoryVO>> tree(@RequestBody RuleCategoryTreeDTO params) {
        //todo:
        List<RuleCategoryVO> data= ruleCategoryInfoService.tree(params);
        return new Result<List<RuleCategoryVO>>().ok(data);
    }

    @SaCheckPermission("sys:rule:category:save")
    @PostMapping("/save")
    @ApiOperation(value = "新增", response = RuleCategoryVO.class)
    public Result<RuleCategoryVO> save(@RequestBody RuleCategorySaveDTO params) {
        ValidatorUtils.validateEntity(params);
        RuleCategoryVO data= ruleCategoryInfoService.save(params);
        return new Result<RuleCategoryVO>().ok(data);
    }

    @SaCheckPermission("sys:rule:category:update")
    @PostMapping("/update")
    @ApiOperation(value = "修改", response = RuleCategoryVO.class)
    public Result<RuleCategoryVO> update(@RequestBody RuleCategoryUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        RuleCategoryVO data=ruleCategoryInfoService.update(params);
        return new Result<RuleCategoryVO>().ok(data);
    }

    @SaCheckPermission("sys:rule:category:del")
    @PostMapping("/del")
    @ApiOperation(value = "删除", response = String.class)
    public Result<String> del(@RequestBody RuleCategoryByIdDTO params) {
        ValidatorUtils.validateEntity(params);
        ruleCategoryInfoService.delById(params.getId());
        return new Result<String>().ok("ok");
    }

    @SaCheckPermission("sys:rule:category:info")
    @PostMapping("/info")
    @ApiOperation(value = "获取", response = RuleCategoryVO.class)
    public  Result<RuleCategoryVO> info(@RequestBody RuleCategoryByIdDTO params) {
        ValidatorUtils.validateEntity(params);
        RuleCategoryVO data= ruleCategoryInfoService.infoById(params.getId());
        return new Result<RuleCategoryVO>().ok(data);
    }








}
