package com.xxh.cloud.rule.sys.manager.controller;


import cn.dev33.satoken.annotation.SaCheckPermission;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.rule.framework.modules.rule.biz.RuleConfigInfoBizService;
import com.xxh.cloud.rule.framework.modules.rule.entity.dto.*;
import com.xxh.cloud.rule.framework.modules.rule.entity.vos.RuleConfigVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

//@SaCheckPermission("sys:rule:config")
@Log4j2
@RestController
@RequestMapping("/sys/rule/config")
@Api(tags = "运营端,规则配置管理")
public class RuleConfigInfoManagerController {

    @Autowired
    private RuleConfigInfoBizService ruleConfigInfoService;


    @SaCheckPermission("sys:rule:config:query")
    @PostMapping("/query")
    @ApiOperation(value = "查询", response = RuleConfigVO.class)
    public Result<PageData<RuleConfigVO>> query(@RequestBody RuleConfigInfoQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<RuleConfigVO> data= ruleConfigInfoService.query(params);
        return new Result<PageData<RuleConfigVO>>().ok(data);
    }

    @SaCheckPermission("sys:rule:config:list")
    @PostMapping("/list")
    @ApiOperation(value = "列表", response = RuleConfigVO.class)
    public Result<List<RuleConfigVO>> list(@RequestBody RuleConfigInfoListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<RuleConfigVO> data= ruleConfigInfoService.list(params);
        return new Result<List<RuleConfigVO>>().ok(data);
    }

    @SaCheckPermission("sys:rule:config:save")
    @PostMapping("/save")
    @ApiOperation(value = "新增", response = RuleConfigVO.class)
    public Result<RuleConfigVO> save(@RequestBody RuleConfigInfoSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        RuleConfigVO data= ruleConfigInfoService.save(params);
        return new Result<RuleConfigVO>().ok(data);
    }

    @SaCheckPermission("sys:rule:config:update")
    @PostMapping("/update")
    @ApiOperation(value = "修改", response = RuleConfigVO.class)
    public Result<RuleConfigVO> update(@RequestBody RuleConfigInfoUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        RuleConfigVO data=ruleConfigInfoService.update(params);
        return new Result<RuleConfigVO>().ok(data);
    }


    @SaCheckPermission("sys:rule:config:updateStatus")
    @PostMapping("/updateStatus")
    @ApiOperation(value = "修改状态", response = RuleConfigVO.class)
    public Result<RuleConfigVO> updateStatus(@RequestBody RuleConfigInfoUpdateStatusDTO params) {
        ValidatorUtils.validateEntity(params);
        RuleConfigVO data=ruleConfigInfoService.updateStatus(params);
        return new Result<RuleConfigVO>().ok(data);
    }

    @SaCheckPermission("sys:rule:config:del")
    @PostMapping("/del")
    @ApiOperation(value = "删除", response = String.class)
    public Result<String> del(@RequestBody RuleConfigInfoByIdDTO params) {
        ValidatorUtils.validateEntity(params);
        ruleConfigInfoService.delById(params.getId());
        return new Result<String>().ok("ok");
    }

    @SaCheckPermission("sys:rule:config:info")
    @PostMapping("/info")
    @ApiOperation(value = "获取", response = RuleConfigVO.class)
    public  Result<RuleConfigVO> info(@RequestBody RuleConfigInfoByIdDTO params) {
        ValidatorUtils.validateEntity(params);
        RuleConfigVO data= ruleConfigInfoService.infoById(params.getId());
        return new Result<RuleConfigVO>().ok(data);
    }


    @SaCheckPermission("sys:rule:config:copy")
    @PostMapping("/copy")
    @ApiOperation(value = "拷贝", response = RuleConfigVO.class)
    public  Result<RuleConfigVO> copy(@RequestBody RuleConfigInfoCopyDTO params) {
        ValidatorUtils.validateEntity(params);
        RuleConfigVO data= ruleConfigInfoService.copy(params);
        return new Result<RuleConfigVO>().ok(data);
    }


}
