package com.xxh.cloud.rule.sys.manager.controller;


import cn.dev33.satoken.annotation.SaCheckPermission;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.metadata.framework.modules.common.constant.enums.RuleCondtionDataTypeEnum;
import com.xxh.cloud.rule.framework.modules.rule.entity.dto.RuleConfigInfoByIdDTO;
import com.xxh.cloud.rule.framework.modules.rule.entity.vos.CondtionOneDataVO;
import com.xxh.cloud.rule.framework.modules.rule.entity.vos.RuleDecisionTreeConfigVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;

@Log4j2
@RestController
@RequestMapping("/sys/rule/dataConfig")
@Api(tags = "运营端,规则数据配置管理")
public class RuleDataConfigManagerController {



    @SaCheckPermission("sys:rule:dataConfig:condtionOneData")
    @PostMapping("/condtionOneData")
    @ApiOperation(value = "条件属性一级规则配置", response = CondtionOneDataVO.class)
    public Result<List<CondtionOneDataVO>> condtionOneData() {

        List<CondtionOneDataVO> data=new ArrayList<>();

        //报告
        CondtionOneDataVO report= new CondtionOneDataVO("报告","报告", YN.NO,YN.YES);
        report.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Report_Laboratory.getCode(),"检验报告"));
        report.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Report_Examine.getCode(),"检查报告"));
        //report.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Report_Pathology.getCode(),"病理报告"));
        //todo:有数据，未实现
        //report.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Report_PhysicalExamine.getCode(),"体检报告[暂不支持]"));
        data.add(report);

        //医生操作
        CondtionOneDataVO doctorOp= new CondtionOneDataVO("医生操作","医生操作", YN.NO,YN.YES);
        doctorOp.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Apply_Examine.getCode(),"申请检查"));
        doctorOp.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Apply_Laboratory_Project.getCode(),"申请检验"));
        //doctorOp.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Apply_Pathology.getCode(),"申请病理"));


        //todo:无同步数据
        //doctorOp.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Apply_Inpatient.getCode(),"申请住院[无数据]"));
        //todo:无同步数据
        //doctorOp.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Apply_Discharge.getCode(),"申请出院[无数据]"));

        doctorOp.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Apply_Operation.getCode(),"申请手术",YN.NO));
        //申请治疗，用治疗记录
        doctorOp.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Therapy.getCode(),"申请治疗"));

        //todo:无数据
        //doctorOp.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Apply_Nurse.getCode(),"申请护理[无数据]"));

        doctorOp.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Medicine_MZ.getCode(),"门诊用药"));
        doctorOp.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Medicine_JZ.getCode(),"急诊用药"));
        doctorOp.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Medicine_ZY.getCode(),"住院用药"));
        //doctorOp.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Medicine_CY.getCode(),"出院带药"));
//        doctorOp.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Diagnosis.getCode(),"下诊断"));
        doctorOp.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.OperationRecord.getCode(),"执行手术"));
        data.add(doctorOp);


        //患者信息
        CondtionOneDataVO baseInfo= new CondtionOneDataVO("患者信息","患者信息", YN.NO,YN.YES);
        baseInfo.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Diagnosis.getCode(),"历史诊断"));
        //todo:无数据
        //baseInfo.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Medical_Record.getCode(),"病历[无数据]"));
        //todo:无数据
        //baseInfo.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Physical_Sign.getCode(),"体征[无数据]"));
        baseInfo.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.MDT_Patient.getCode(),"患者基础信息"));
        baseInfo.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.ScaleResult.getCode(),"量表结果"));
        //baseInfo.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.ScaleItemResult.getCode(),"量表问题"));

        data.add(baseInfo);



        CondtionOneDataVO recordInfo= new CondtionOneDataVO("患者就医记录","患者就医记录", YN.NO,YN.YES);
        recordInfo.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.VisitRecord.getCode(),"门急诊就诊"));
        recordInfo.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Reserve_RegisterRecord.getCode(),"预约挂号"));
        //recordInfo.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Reserve_Examine.getCode(),"预约检查"));

        //recordInfo.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Reserve_Inpatient.getCode(),"预约住院"));
        recordInfo.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Reserve_Inpatient_Not.getCode(),"预约住院"));

        //recordInfo.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Inpatient.getCode(),"入院登记"));
        recordInfo.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Inpatient_Not.getCode(),"入院登记"));

        //todo:测试环境发包
        recordInfo.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Change_Dept.getCode(),"转科室"));

        recordInfo.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Discharge.getCode(),"出院"));

        //recordInfo.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Online_Inquiry.getCode(),"在线问诊"));
        //recordInfo.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Change_Ward.getCode(),"转病区[无数据]"));
        data.add(recordInfo);

//        CondtionOneDataVO advice= new CondtionOneDataVO("文书记录","文书记录", YN.NO,YN.YES);
//        advice.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.DischargeSummary.getCode(),"出院小结"));
//        advice.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.OperationText.getCode(),"手术文书"));
//        data.add(advice);


        //CondtionOneDataVO actionEvent= new CondtionOneDataVO("患者就医事件","患者就医事件", YN.NO,YN.YES);
        //actionEvent.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.MDS_Event.getCode(),"事件",YN.NO));
        //data.add(actionEvent);

        CondtionOneDataVO emrInfo= new CondtionOneDataVO("电子病例数据","电子病例数据", YN.NO,YN.YES);
        emrInfo.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.EMR_SymptomZS.getCode(),"主诉"));
        emrInfo.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.EMR_SymptomJWS.getCode(),"既往史"));
        emrInfo.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.EMR_SymptomXBS.getCode(),"现病史"));
        emrInfo.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Physical_Sign.getCode(),"体征"));
        //
//        emrInfo.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.EMR_Temperature.getCode(),"体温"));
//        emrInfo.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.EMR_HeartRate.getCode(),"心率(次/分)"));
//        emrInfo.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.EMR_BloodPressure.getCode(),"血压(BP mmHg)"));
//        emrInfo.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.EMR_Weight.getCode(),"体重(kg)"));
//        emrInfo.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.EMR_Height.getCode(),"身高(cm)"));
//        emrInfo.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.EMR_BMI.getCode(),"BMI"));
        data.add(emrInfo);


        CondtionOneDataVO opData= new CondtionOneDataVO("运营数据","运营数据", YN.NO,YN.YES);
        opData.addChildren(new CondtionOneDataVO(RuleCondtionDataTypeEnum.Label.getCode(),"标签"));
        data.add(opData);


        return new Result<List<CondtionOneDataVO>>().ok(data);
    }



//    om.xxh.cloud.datalabel.config.framework.modules.base.mapper.DataLabelRuleRelInfoMapper.insert (batch index #1) failed. Cause:
//    java.sql.BatchUpdateException: Field 'creator' doesn't have a default value ;
//    Field 'creator' doesn't have a default value; nested exception is java.sql.BatchUpdateException: Field 'creator' doesn't have a default value



}
