package com.xxh.cloud.rule.core.context;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.rule.basic.api.MethodExecuteByCallApi;
import com.xxh.cloud.rule.basic.api.MethodExecuteByMQApi;
import com.xxh.cloud.rule.basic.api.bean.dto.MethodExecuteDTO;
import com.xxh.cloud.rule.basic.api.bean.dto.WorkFlowNodeExecuteDTO;
import com.xxh.cloud.rule.basic.context.XXRuleRoam;
import com.xxh.cloud.rule.core.constant.FlowResultEnum;
import com.xxh.cloud.rule.core.constant.MethodExecuteWayEnum;
import com.xxh.cloud.rule.core.constant.RuleConfigBizTypeEnum;
import com.xxh.cloud.rule.core.constant.RuleConfigTypeEnum;
import com.xxh.cloud.rule.core.entity.flow.XXFlowRuleBO;
import com.xxh.cloud.rule.core.entity.label.XXLabelRuleBO;
import lombok.Getter;
import lombok.Setter;

import java.util.*;

@Getter
@Setter
public class XXRuleContext {


    /**
     * 规则ID
     * */
    private String ruleId;

    /**
     * 规则名称
     * */
    private String ruleName;

    /**
     * 规则分类[RuleConfigTypeEnum]
     * */
    private String ruleType= RuleConfigTypeEnum.RCT_Decision_Flow.getCode();


    /**
     * 规则业务分类[RuleConfigBizTypeEnum]
     * */
    private String ruleBizType= RuleConfigBizTypeEnum.CBT_Inference.getCode();

    /**
     * 规则版本号
     * */
    private Integer ruleVersion;


    /**
     * 规则时间戳
     * */
    private long ruleTimestamp;


    /**
     * 规则分类ID
     * */
    private String ruleCategoryId;


    /**
     *  条件执行是否断言
     *  默认：断言执行
     * */
    private boolean isAssert=true;


    //----------------------------------- 数据

    /**
     * 执行的数据日期
     * 默认：当前
     * */
    private Date dataDate=new Date();

    /**
     *  引擎调用数据
     * */
    private XXRuleRoam roam;


    /**
     *  引擎调用数据 key 排序
     * */
    //todo:优化
    private List<String> roamKeyList;


    /**
     * 部门负责人
     * */
    private Map<String,List<String>> deptUserMap;

    /**
     * 岗位用户
     * */
    private Map<String,List<String>> postUserMap;


    //-----------------------------------
    /**
     * 请求执行ID
     * */
    private String traceId;

    /**
     * 处理开始时间(毫秒)
     * */
    private Date requestStartTime;

    /**
     * 处理截止时间(毫秒)
     * */
    private Date requestEndTime;

    /**
     * 处理时间(毫秒)
     * */
    private long requestTime;

    /**
     * 是否debug模式
     * */
    private boolean isDebug;


    /**
     * debug模式下，模拟调用次数
     * */
    private Long callNum;



    /**
     * 节点:引用变量(返回)
     * */
    private XXRuleVariableRoom variable;

    /**
     * 节点:日志记录(返回)
     * */
    private List<XXRuleNodeLogBean> nodeLogList=new ArrayList<>();


    /**
     * 引擎方法执行方式定义
     * */
    private MethodExecuteWayEnum methodExecuteWay=MethodExecuteWayEnum.MEW_Return;


    /**
     * 引擎方法执行方式 : MQ
     * */
    private MethodExecuteByMQApi methodExecuteByMQApi;


    /**
     * 引擎方法执行方式 : Call
     * */
    private MethodExecuteByCallApi methodExecuteByCallApi;

    /**
     * 【推理】引擎方法执行方法定义返回(返回)
     * */
    private List<MethodExecuteDTO> returnMethodExecuteList;

    /**
     * 【流程】
     * */
    private List<WorkFlowNodeExecuteDTO> returnWorkFlowNodeList;


    /**
     * 流程执行结果状态
     * */
    private FlowResultEnum executeResult;





    public boolean isRuleEnd(){
        if(this.executeResult!=null && this.executeResult== FlowResultEnum.End){
            return true; //终止
        }else {
            return false;
        }
    }




    //

    /**
     * 创建引擎执行上下文
     * */
    public static XXRuleContext getRuleContext2ExecutorByReturn(XXLabelRuleBO rule , XXRuleRoam userData, boolean isDebug){
        XXRuleContext context=new XXRuleContext();
        context.setRuleId(rule.getId());
        context.setRuleName(rule.getName());
        context.setRuleType(rule.getType());
        context.setRuleVersion(rule.getVersion());
        context.setRuleCategoryId(rule.getCategoryId());
        context.setTraceId(IdUtil.simpleUUID());
        context.setRequestStartTime(new Date());
        context.setDebug(isDebug);
        context.setRoam(userData);
        context.setMethodExecuteWay(MethodExecuteWayEnum.MEW_Return);
        return context;
    }



    /**
     * 创建引擎执行上下文
     * */
    public static XXRuleContext getRuleContext2ExecutorByReturn(XXFlowRuleBO rule ,XXRuleRoam userData,boolean isDebug){
        XXRuleContext context=new XXRuleContext();
        context.setRuleId(rule.getId());
        context.setRuleName(rule.getName());
        context.setRuleType(rule.getType());
        context.setRuleBizType(rule.getBizType());
        context.setRuleVersion(rule.getVersion());
        context.setRuleCategoryId(rule.getRuleCategoryId());
        context.setTraceId(IdUtil.simpleUUID());
        context.setRequestStartTime(new Date());
        context.setDebug(isDebug);
        context.setRoam(userData);
        context.setMethodExecuteWay(MethodExecuteWayEnum.MEW_Return);
        return context;
    }

    /**
     * 创建引擎执行上下文
     * */
    public static XXRuleContext getRuleContext2ExecutorByMQ(XXFlowRuleBO rule ,XXRuleRoam userData,MethodExecuteByMQApi mqApi,boolean isDebug){
        XXRuleContext context=new XXRuleContext();
        context.setRuleId(rule.getId());
        context.setRuleName(rule.getName());
        context.setRuleType(rule.getType());
        context.setRuleBizType(rule.getBizType());
        context.setRuleVersion(rule.getVersion());
        context.setTraceId(IdUtil.simpleUUID());
        context.setRequestStartTime(new Date());
        context.setDebug(isDebug);
        context.setRoam(userData);
        context.setMethodExecuteWay(MethodExecuteWayEnum.MEW_MQ);
        context.setMethodExecuteByMQApi(mqApi);
        return context;
    }

    /**
     * 创建引擎执行上下文
     * */
    public static XXRuleContext getRuleContext2ExecutorByCall(XXFlowRuleBO rule ,XXRuleRoam userData, MethodExecuteByCallApi methodCallApi,boolean isDebug){
        XXRuleContext context=new XXRuleContext();
        context.setRuleId(rule.getId());
        context.setRuleName(rule.getName());
        context.setRuleType(rule.getType());
        context.setRuleBizType(rule.getBizType());
        context.setRuleVersion(rule.getVersion());
        context.setTraceId(IdUtil.simpleUUID());
        context.setRequestStartTime(new Date());
        context.setDebug(isDebug);
        context.setRoam(userData);
        context.setMethodExecuteWay(MethodExecuteWayEnum.MEW_Call);
        context.setMethodExecuteByCallApi(methodCallApi);
        return context;
    }

//    public static XXRuleContext getWorkFlowContext2ExecutorByCall(String taskId ,XXFlowRuleBO rule ,XXRuleRoam userData,boolean isDebug){
//        XXRuleContext context=new XXRuleContext();
//        context.setRuleId(rule.getId());
//        context.setRuleName(rule.getName());
//        context.setRuleType(rule.getType());
//        context.setRuleBizType(rule.getBizType());
//        context.setRuleVersion(rule.getVersion());
//        context.setTraceId(taskId);
//        context.setRequestStartTime(new Date());
//        context.setDebug(isDebug);
//        context.setRoam(userData);
//        context.setMethodExecuteWay(MethodExecuteWayEnum.MEW_Call);
//        return context;
//    }



    /**
     * 创建获取参数上下文
     * */
    public static XXRuleContext getRuleContext2Variable(XXFlowRuleBO rule,boolean isDebug){
        XXRuleContext context=new XXRuleContext();
        context.setRuleId(rule.getId());
        context.setRuleName(rule.getName());
        context.setRuleType(rule.getType());
        context.setRuleBizType(rule.getBizType());
        context.setRuleVersion(rule.getVersion());
//        context.setRuleTimestamp();
        context.setTraceId(IdUtil.simpleUUID());
        context.setRequestStartTime(new Date());
        context.setDebug(isDebug);
        context.setVariable(new XXRuleVariableRoom());
        return context;
    }


    /**
     * 新增调用方法
     * */
    public void addReturnMethod(MethodExecuteDTO item){
        if(this.returnMethodExecuteList==null){
            this.returnMethodExecuteList=new ArrayList<>();
        }
        if(item!=null){
            this.returnMethodExecuteList.add(item);
        }
    }

    /**
     * 新增流程执行节点
     * */
    public void addReturnWorkFlowNode(WorkFlowNodeExecuteDTO item){
        if(this.returnWorkFlowNodeList==null){
            this.returnWorkFlowNodeList=new ArrayList<>();
        }
        if(item!=null){
            this.returnWorkFlowNodeList.add(item);
        }
    }


    /**
     *
     * */
    public void addNodeLog(XXRuleNodeLogBean item){
        if(this.nodeLogList==null){
            this.nodeLogList=new ArrayList<>();
        }
        if(item!=null){
            this.nodeLogList.add(item);
        }
    }

    /**
     *
     * */
    public void addRoam(String key,Object val){
        if(this.roam==null){
            this.roam=new XXRuleRoam();
        }
        this.roam.put(key,val);
    }



    /**
     * 新增部门人员
     * */
    public void addDeptUser(String deptId,List<String> userIds){
        if(this.deptUserMap==null){
            this.deptUserMap=new HashMap<>();
        }
        if(StrUtil.isNotEmpty(deptId) && CollUtil.isNotEmpty(userIds)){
            this.deptUserMap.put(deptId,userIds);
        }
    }

    /**
     * 新增岗位人员
     * */
    public void addPostUser(String postId,List<String> userIds){
        if(this.postUserMap==null){
            this.postUserMap=new HashMap<>();
        }
        if(StrUtil.isNotEmpty(postId) && CollUtil.isNotEmpty(userIds)){
            this.postUserMap.put(postId,userIds);
        }
    }

}
