package com.xxh.cloud.rule.core.executor.condition.process;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.utils.ConsoleUtils;
import com.xxh.cloud.rule.core.entity.base.*;
import com.xxh.cloud.rule.core.constant.ConditionNodeTypeEnum;
import com.xxh.cloud.rule.core.constant.ConditionRelationTypeEnum;
import com.xxh.cloud.rule.core.context.XXRuleContext;
import com.xxh.cloud.rule.core.executor.condition.context.ConditionContext;
import com.xxh.cloud.rule.core.executor.condition.context.ConditionContextItemBO;
import com.xxh.cloud.rule.core.executor.condition.healthOperate.XXHOperateUtils;
import com.xxh.cloud.rule.core.executor.condition.operate.XXOperate;
import com.xxh.cloud.rule.core.executor.condition.operate.XXOperateFactory;
import com.xxh.cloud.rule.core.executor.data.PropertyBaseGetUtils;

/**
 * 条件节点处理-执行
 * */
public class DefaultConditionNodeExecuteProcess extends AbstractConditionNodeProcess{


    @Override
    public boolean process(XXRuleContext context, ConditionContext conditionContext, String nodeId, String nodeName,String nodeType, String nodeRuleType, NodeCondition condition) {
        boolean flag=  this.condition(context,conditionContext,condition,"0");
        return flag;
    }

    /**
     *  执行条件
     * */
    private boolean condition(XXRuleContext context,ConditionContext conditionContext,NodeCondition condition,String conditionParentId){
        if(condition==null ){
            ConsoleUtils.log("+++++ 条件配置未空 。");
            return false;
        }
        if(StrUtil.isEmpty(condition.getType())){
            ConsoleUtils.log("+++++ 条件配置未空: {} ", JSON.toJSONString(condition));
            return false;
        }

        if(ConditionNodeTypeEnum.Property.getCode().equals(condition.getType())){
            System.out.println(">>>>>  执行节点开始");
            boolean flag =this.condition_property(context,conditionContext,condition,condition.getProperty(),conditionParentId); //属性条件
            System.out.println(StrUtil.format(">>>>>  执行节点完成,结果:{} .  ID:{},参数:{} ; {}  ",
                    flag,condition.getId(),condition.getProperty().getMdCode(),condition.getProperty().getMdName()));

//            System.out.println("执行条件属性,开始 *********************");
//            System.out.println(JSON.toJSONString(condition.getProperty()));
//            System.out.println("****** 执行条件属性结果:"+flag);
//            System.out.println("执行条件属性,截止  *********************");

            return flag;
        }
//      else if(ConditionNodeTypeEnum.Method.getCode().equals(condition.getType())){
//           return this.condition_method(context,conditionContext,condition.getMethod());  //方法条件
//      }

        else {
            //关系条件
            //且:所有T，返回T
            if(ConditionRelationTypeEnum.And.getCode().equals(condition.getRelation())){
                System.out.println(">>>>> And 开始");
                for(NodeCondition childCondition: condition.getChildren()){
                    boolean flag =  condition(context,conditionContext,childCondition,condition.getId());
                    if(flag==false){
                        //记录关系节点日志
                        conditionContext.addConditionItem(new ConditionContextItemBO(condition.getId(),condition.getType(),condition.getRelation(),conditionParentId,false));
                        System.out.println(">>>>> And 截止 false");
                        return false;
                    }
                }
                //记录关系节点日志
                conditionContext.addConditionItem(new ConditionContextItemBO(condition.getId(),condition.getType(),condition.getRelation(),conditionParentId,true));
                System.out.println(">>>>> And 截止 true");
                return true;
            }


            //或:有一个T，返回T
            if(ConditionRelationTypeEnum.Or.getCode().equals(condition.getRelation())){
                System.out.println(">>>>> Or 开始");
                for(NodeCondition childCondition : condition.getChildren()){
                    boolean flag = condition(context,conditionContext,childCondition,condition.getId());
                    if(flag){
                        //记录关系节点日志
                        conditionContext.addConditionItem(new ConditionContextItemBO(condition.getId(),condition.getType(),condition.getRelation(),conditionParentId,true));
                        System.out.println(">>>>> Or 截止 true");
                        return true;
                    }
                }
                //记录关系节点日志
                conditionContext.addConditionItem(new ConditionContextItemBO(condition.getId(),condition.getType(),condition.getRelation(),conditionParentId,false));
                System.out.println(">>>>> Or 截止 false");
                return false;
            }
            return false;
        }
    }



    /**
     * 判断属性条件
     * */
    private boolean condition_property(XXRuleContext context,ConditionContext conditionContext,NodeCondition condition, Object object,String conditionParentId){
        if(object instanceof PropertyConditionBaseBean){
            PropertyConditionBaseBean property=(PropertyConditionBaseBean)object;
            return this.condition_property_Base(context,conditionContext,condition,property,conditionParentId);
        }else if(object instanceof PropertyConditionHealthBean){
            PropertyConditionHealthBean property=(PropertyConditionHealthBean)object;
            return this.condition_property_Health(context,conditionContext,condition,property,conditionParentId);
        }else {
            throw new BusinessException(BusinessBaseErrorEnum.Run_Rule_ERROR,"不支持的条件属性配置");
        }
    }

    /**
     *  判断属性条件 - 基础
     * */
    private boolean condition_property_Base(XXRuleContext context,ConditionContext conditionContext,NodeCondition condition, PropertyConditionBaseBean property,String conditionParentId){
        //操作符对象
        XXOperate xxOperate= XXOperateFactory.getInstall(property.getOp());
        //左值
        Object leftValObj= context.getRoam().get(property.getVarCode());
        //比较操作
        boolean flag= xxOperate.operate(property.getVarType(),
                leftValObj,
                PropertyBaseGetUtils.getValueBeanObj(context,property.getVarType(),property.getOvo()),
                PropertyBaseGetUtils.getValueBeanObj(context,property.getVarType(),property.getOvt()),
                PropertyBaseGetUtils.getValueBeanList(context,property.getVarType(),property.getOvs())
        );
        //todo:条件路径日志
        return flag;
    }

    /**
     *  判断属性条件 - 健康
     * */
    private boolean condition_property_Health(XXRuleContext context,ConditionContext conditionContext,NodeCondition condition, PropertyConditionHealthBean property,String conditionParentId){
        boolean flag= XXHOperateUtils.operate(context,conditionContext,condition,property,conditionParentId);
        return flag;
    }


    /**
     * 判断方法条件 - [不支持]
     * */
    private boolean condition_method(XXRuleContext context,ConditionContext conditionContext, MethodBaseBean method){
        //todo:未实现
        return true;
    }




}
