package com.xxh.cloud.rule.core.executor.method.process;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.rule.basic.api.bean.dto.MethodExecuteDTO;
import com.xxh.cloud.rule.basic.api.bean.dto.MethodExecuteParamDTO;
import com.xxh.cloud.rule.basic.api.bean.dto.WorkFlowNodeExecuteDTO;
import com.xxh.cloud.rule.core.constant.NodeCheckWay;
import com.xxh.cloud.rule.core.constant.NodeExecutorType;
import com.xxh.cloud.rule.core.context.XXRuleContext;
import com.xxh.cloud.rule.core.entity.base.MethodBaseBean;
import com.xxh.cloud.rule.core.entity.base.MethodBaseParamBean;
import com.xxh.cloud.rule.core.entity.base.NodeMethod;
import com.xxh.cloud.rule.core.executor.method.utils.MethodExecuteUtils;
import io.swagger.annotations.ApiModelProperty;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.Map;

/**
 * 流程引擎动作方法执行
 * */
public class WorkFlowMethodNodeExecuteProcess{

    private static final Logger logger = LoggerFactory.getLogger(WorkFlowMethodNodeExecuteProcess.class);


    public WorkFlowNodeExecuteDTO doResult(XXRuleContext context, String nodeId, String nodeType,String nodeRuleType,String nodeName,
                                           String nodeContent,String prevNodeId,NodeMethod action){


        WorkFlowNodeExecuteDTO data= new WorkFlowNodeExecuteDTO(nodeId,nodeType,nodeRuleType,nodeName,nodeContent,prevNodeId);
        data.setAdvanceRatio(action.getAdvanceRatio());

        if(StrUtil.isEmpty(action.getCheckWay()) || NodeCheckWay.Check_Or.getCode().equals(action.getCheckWay())){
            data.setCheckWay(NodeCheckWay.Check_Or.getCode());
        }else {
            data.setCheckWay(NodeCheckWay.Check_All.getCode());
             data.setCheckRatio(action.getCheckRatio());
        }


        //自动执行(执行人)
        if(StrUtil.isEmpty(action.getExecutorType()) || NodeExecutorType.NE_None.getCode().equals(action.getExecutorType())){
            data.setIsExecutor(YN.NO.getCode());
            data.setExecutorType(NodeExecutorType.NE_None.getCode());
        }else {
            data.setIsExecutor(YN.YES.getCode());
            if(NodeExecutorType.NE_User.getCode().equals(action.getExecutorType())){
                data.setExecutorType(action.getExecutorType());
                action.getExecutorIds().forEach(item->{
                    data.addExecutorId(item.getVal());
                });
            }

            if(NodeExecutorType.NE_Post.getCode().equals(action.getExecutorType())){
                data.setExecutorType(action.getExecutorType());
                action.getExecutorIds().forEach(item->{
                    if(MapUtil.isNotEmpty(context.getPostUserMap())){
                        List<String> userIds= context.getPostUserMap().get(item.getVal());
                        data.addExecutorId(userIds);
                    }
                });
            }

            if(NodeExecutorType.Check_Director.getCode().equals(action.getExecutorType())){
                data.setExecutorType(action.getExecutorType());
                action.getExecutorIds().forEach(item->{
                    if(MapUtil.isNotEmpty(context.getDeptUserMap())){
                        List<String> userIds= context.getDeptUserMap().get(item.getVal());
                        data.addExecutorId(userIds);
                    }
                });
            }
        }

        //审核后定时
        if(StrUtil.isEmpty(action.getCronTriggerYN()) || YN.NO.getCode().equals(action.getCronTriggerYN())){
            data.setIsTrigger(YN.NO.getCode());
        }else {
            data.setIsTrigger(YN.YES.getCode());
            data.setTriggerStartNum(action.getTriggerStartNum());
            data.setTriggerEndNum(action.getTriggerEndNum());
            data.setTriggerEndUnit(action.getTriggerEndUnit());
            data.setTriggerType(action.getTriggerType());
            data.setTriggerLoopNum(action.getTriggerLoopNum());
            data.setTriggerLoopUnit(action.getTriggerLoopUnit());
        }

        //动作
        data.setActionId(action.getActionId());
        data.setActionName(action.getActionName());
        data.setMethodCode(action.getMethodCode());
        data.setMethodName(action.getMethodName());
        for(MethodBaseParamBean param : action.getParams()){
            MethodExecuteParamDTO methodExecuteParamDTO=  MethodExecuteUtils.getMethodExecuteParam(context.getRoam(),param);
            data.addParam(methodExecuteParamDTO);
        }
        return data;
    }





}
