package com.xxh.cloud.rule.core.utils;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.utils.XXEnumUtil;
import com.xxh.cloud.rule.core.constant.HealthDateLimitUnitEnum;
import com.xxh.cloud.rule.core.constant.HealthDateRangeTypeEnum;

import java.util.*;

public class DateBetweenUtils {




    /**
     *  时间单位
     * */
    private static HealthDateLimitUnitEnum getHealthDateLimitUnitEnum(String unitCode){
        Map<String,HealthDateLimitUnitEnum> unitMap= XXEnumUtil.getEnumMap(HealthDateLimitUnitEnum.class);
        HealthDateLimitUnitEnum unit = unitMap.get(unitCode);
        if(unit==null){
            throw new BusinessException(BusinessBaseErrorEnum.Config_GET_ERROR,"不支持的时间定义");
        }
        return unit;
    }


    /**
     * 相对时间转绝对时间
     * */
    private static DateTime getDate(Date date, HealthDateLimitUnitEnum unit, int num){
        switch (unit){
            case UN_Hour:
                return DateUtil.offsetHour(date,0-num);
            case UN_Day:
                return DateUtil.offsetDay(date,0-num);
            case UN_Week:
                return DateUtil.offsetWeek(date,0-num);
            case UN_Month:
                return DateUtil.offsetMonth(date, 0-num);
        }
        throw new BusinessException(BusinessBaseErrorEnum.Config_GET_ERROR,"不支持的时间定义");
    }



    private static List<String> between(Date start, Date end, HealthDateLimitUnitEnum unit){
        long num;
        if(HealthDateLimitUnitEnum.UN_Hour==unit){
            num= DateUtil.between(start,end, DateUnit.HOUR);
        }else {
            num= DateUtil.between(start,end, DateUnit.DAY);
        }

        List<String> data=new ArrayList<>();
        for (int i = 0; i <= num; i++) {
            if(HealthDateLimitUnitEnum.UN_Hour==unit){
                System.out.println(" i = " +i +" data = "+DateUtil.formatDateTime(start));
                DateTime dt = DateUtil.offsetHour(start,i);
                data.add(DateUtil.format(dt,"yyyyMMddHH"));
            }else {
                DateTime dt = DateUtil.offsetDay(start,i);
                data.add(DateUtil.format(dt, "yyyyMMdd"));
            }
        }

        Collections.sort(data, new Comparator<String>() {
            @Override
            public int compare(String str1, String str2) {
                return str2.compareTo(str1);
            }
        });

        return data;
    }




    /**
     * 相对值
     * */
    public static List<String> between_relative(String unit,String startNum, String endNum){




        HealthDateLimitUnitEnum unitEnum = getHealthDateLimitUnitEnum(unit);
        int start=  Integer.parseInt(startNum);
        int end = Integer.parseInt(endNum);


        Date date=new Date();
        //
        Date startDate;
        if(start<=0){
            startDate=new Date();
        }else {
            startDate=getDate(date,unitEnum,start);
        }

        //
        Date endDate;
        if(end<=0){
            endDate=new Date();
        }else {
            endDate=getDate(date,unitEnum,end);
        }

        return between(startDate,endDate,unitEnum);
    }


    /**
     * 绝对值
     * @param  startDate : yyyy-MM-dd
     * @param  endDate : yyyy-MM-dd
     * */
    public static List<String> between_absolute(String startDate, String endDate){
        DateTime start= DateUtil.parseDate(startDate);

        DateTime end;
        if(StrUtil.isNotEmpty(endDate)){
             end=DateUtil.parseDate(endDate);
        }else {
             end= DateTime.now();
        }
        return between(start,end,HealthDateLimitUnitEnum.UN_Day);
    }


    public static List<String> between(String type,String unit, String start, String end){
        if(StrUtil.isEmpty(type) || StrUtil.isEmpty(unit) || StrUtil.isEmpty(start) || StrUtil.isEmpty(end)){
            return new ArrayList<>();
        }

        if(HealthDateRangeTypeEnum.HDRT_Absolute.getCode().equals(type)){
            return  between_absolute(start,end);
        }else {
            return between_relative(unit,start,end);
        }
    }




    public static void main(String[] args) {

//        List<String> list= DateBetweenUtils.between(4,1,HealthDateLimitUnitEnum.UN_Day);
//        System.out.println(JSON.toJSONString(list));
//
//        Collections.sort(list, new Comparator<String>() {
//            @Override
//            public int compare(String str1, String str2) {
//                return str2.compareTo(str1);
//            }
//        });
//        System.out.println(JSON.toJSONString(list));

//        XXRuleRoam roam=new XXRuleRoam();
//        roam.put("1","1");
//        roam.put("2","2");
//        Enumeration<String> enu= roam.keys();
//
//        while (enu.hasMoreElements()) {
//            System.out.println(enu.nextElement());
//        }



        List<String> list= DateBetweenUtils.between("relative","D","0","0");
        list.forEach(item->{
            System.out.println(item);
        });


    }




}
