package com.xxh.cloud.rule.execute.framework.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.rule.core.entity.flow.XXFlowRuleBO;
import com.xxh.cloud.rule.execute.framework.modules.base.entity.FlowTaskInfo;
import com.xxh.cloud.rule.execute.framework.modules.base.entity.FlowTaskNodeInfo;
import com.xxh.cloud.rule.execute.framework.modules.base.entity.RuleFlowExecuteLog;
import com.xxh.cloud.rule.execute.framework.modules.base.service.FlowTaskInfoService;
import com.xxh.cloud.rule.execute.framework.modules.base.service.FlowTaskNodeInfoService;
import com.xxh.cloud.rule.execute.framework.modules.base.service.RuleFlowExecuteLogService;
import com.xxh.cloud.rule.execute.framework.modules.biz.bean.vos.RuleFlowExecuteLogVO;
import com.xxh.cloud.rule.execute.framework.modules.biz.convert.RuleFlowExecuteLogConvert;
import com.xxh.cloud.rule.execute.framework.modules.biz.service.RuleFlowExecuteLogBizService;
import com.xxh.cloud.rule.execute.framework.modules.cache.RuleWorkFlowCacheService;
import com.xxh.cloud.rule.execute.framework.modules.workflow.service.impl.FlowNodeTriggerWorkFlowServiceImpl;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Log4j2
@Service
public class RuleFlowExecuteLogBizServiceImpl implements RuleFlowExecuteLogBizService {

    @Autowired
    private RuleFlowExecuteLogService ruleFlowExecuteLogService;


    @Autowired
    private FlowTaskInfoService flowTaskInfoService;


    @Autowired
    private FlowTaskNodeInfoService flowTaskNodeInfoService;

    @Autowired
    private RuleWorkFlowCacheService ruleWorkFlowCacheService;


    @Override
    public RuleFlowExecuteLogVO getRuleFlowExecuteLogByTask(String taskId, String nodeCode) {
        List<RuleFlowExecuteLog> list= ruleFlowExecuteLogService.list(
                new QueryWrapper<RuleFlowExecuteLog>()
                        .eq("task_id",taskId)
                        .eq("node_code",nodeCode)
        );
        if(CollUtil.isEmpty(list)){
            return null;
        }



        RuleFlowExecuteLog entity= list.get(0);
        return RuleFlowExecuteLogConvert.getRuleFlowExecuteLogVO(entity);
    }


    private RuleFlowExecuteLog getRuleFlowExecuteLog(String taskId, String nodeCode){
        List<RuleFlowExecuteLog> list= ruleFlowExecuteLogService.list(
                new QueryWrapper<RuleFlowExecuteLog>()
                        .eq("task_id",taskId)
                        .eq("node_code",nodeCode)
        );
        if(CollUtil.isEmpty(list)){
            return null;
        }
        RuleFlowExecuteLog entity= list.get(0);
        return entity;
    }

    @Override
    public List<RuleFlowExecuteLogVO> getRuleFlowExecuteLogListByNodeTask(String nodeTaskId) {
        FlowTaskNodeInfo entity= flowTaskNodeInfoService.getById(nodeTaskId);
        if(entity==null){
            return null;
        }
        return this.getRuleFlowExecuteLogListByTask(entity.getTaskId(),entity.getNodeCode());
    }


    @Override
    public List<RuleFlowExecuteLogVO> getRuleFlowExecuteLogListByTask(String taskId,String nodeCode) {
        RuleFlowExecuteLog ruleFlowExecuteLog = this.getRuleFlowExecuteLog(taskId,nodeCode);
        if(ruleFlowExecuteLog==null){
            return null;
        }
        List<RuleFlowExecuteLog> list= ruleFlowExecuteLogService.list(
                new QueryWrapper<RuleFlowExecuteLog>()
                        .eq("task_id",taskId)
                        .le("id",ruleFlowExecuteLog.getId())
                        .orderByAsc("id")
        );

        if(CollUtil.isEmpty(list)){
            return null;
        }

        List<RuleFlowExecuteLogVO> data=new ArrayList<>();

        list.forEach(item->{
            data.add(RuleFlowExecuteLogConvert.getRuleFlowExecuteLogVO(item));
        });

        return data;

    }




}
