package com.xxh.cloud.rule.execute.framework.modules.biz.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.rule.core.entity.flow.XXFlowRuleBO;
import com.xxh.cloud.rule.execute.framework.modules.base.entity.RuleInferenceExecuteMainItemLog;
import com.xxh.cloud.rule.execute.framework.modules.base.entity.RuleInferenceExecuteMainLog;
import com.xxh.cloud.rule.execute.framework.modules.base.service.RuleInferenceExecuteMainItemLogService;
import com.xxh.cloud.rule.execute.framework.modules.base.service.RuleInferenceExecuteMainLogService;
import com.xxh.cloud.rule.execute.framework.modules.biz.bean.dto.RuleInferenceExecuteMainLogQueryDTO;
import com.xxh.cloud.rule.execute.framework.modules.biz.bean.vos.RuleInferenceExecuteMainLogVO;
import com.xxh.cloud.rule.execute.framework.modules.biz.convert.RuleInferenceExecuteMainLogConvert;
import com.xxh.cloud.rule.execute.framework.modules.biz.service.RuleInferenceExecuteLogBizService;
import com.xxh.cloud.rule.execute.framework.modules.cache.RuleInferenceCacheService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class RuleInferenceExecuteLogBizServiceImpl implements RuleInferenceExecuteLogBizService {


    @Autowired
    private RuleInferenceExecuteMainLogService ruleInferenceExecuteMainLogService;
    @Autowired
    private RuleInferenceExecuteMainItemLogService ruleInferenceExecuteMainItemLogService;

    @Autowired
    private RuleInferenceCacheService ruleInferenceCacheService;


    @Override
    public PageData<RuleInferenceExecuteMainLogVO> query(RuleInferenceExecuteMainLogQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        IPage<RuleInferenceExecuteMainLog> page = ruleInferenceExecuteMainLogService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<RuleInferenceExecuteMainLog>()
                        .eq(StrUtil.isNotEmpty(params.getPatientId()),"patient_id",params.getPatientId())
                        .eq(StrUtil.isNotEmpty(params.getRuleId()),"rule_id",params.getRuleId())
                        .like(StrUtil.isNotEmpty(params.getRuleName()),"rule_name",params.getRuleName())
                        .eq(StrUtil.isNotEmpty(params.getRuleCategoryId()),"rule_category_id",params.getRuleCategoryId())
                        .ge(StrUtil.isNotEmpty(params.getStartDateGe()),"start_date",params.getStartDateGe())
                        .le(StrUtil.isNotEmpty(params.getStartDateLe()),"start_date",params.getStartDateLe())
                        .orderByDesc("create_date")
        );




        PageData<RuleInferenceExecuteMainLogVO> data= new PageConverter<RuleInferenceExecuteMainLog, RuleInferenceExecuteMainLogVO>() {
            @Override
            public RuleInferenceExecuteMainLogVO converter(RuleInferenceExecuteMainLog entity) {
                return RuleInferenceExecuteMainLogConvert.getRuleInferenceExecuteMainLogVO(entity);
            }
        }.getPageData(page);

        return data;
    }


    @Override
    public RuleInferenceExecuteMainLogVO getById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        RuleInferenceExecuteMainLog entity= ruleInferenceExecuteMainLogService.getById(id);
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        XXFlowRuleBO ruleConfig = ruleInferenceCacheService.getFlowRule(entity.getRuleId());


        List<RuleInferenceExecuteMainItemLog> list= ruleInferenceExecuteMainItemLogService.list(
                new QueryWrapper<RuleInferenceExecuteMainItemLog>()
                        .eq("exe_log_id",id)
        );

        return RuleInferenceExecuteMainLogConvert.getRuleInferenceExecuteMainLogVO(entity,list);
    }
}
