package com.xxh.cloud.rule.execute.framework.modules.client.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.rule.basic.context.XXRuleRoam;
import com.xxh.cloud.rule.execute.framework.modules.base.entity.FlowTaskNodeInfo;
import com.xxh.cloud.rule.execute.framework.modules.client.convert.WorkFlowExecuteResultConvert;
import com.xxh.cloud.rule.execute.framework.modules.workflow.bean.ros.PatientExeRuleIdRO;
import com.xxh.cloud.rule.execute.framework.modules.workflow.bean.vos.Execute2WorkFlowByReturnVO;
import com.xxh.cloud.rule.execute.framework.modules.workflow.bean.vos.FlowNodeProcessWorkFlowReturnVO;
import com.xxh.cloud.rule.execute.framework.modules.workflow.service.FlowNodeProcessWorkFlowService;
import com.xxh.cloud.rule.execute.framework.modules.workflow.service.FlowNodeTriggerWorkFlowService;
import com.xxh.cloud.rule.execute.framework.modules.workflow.service.FlowStartProcessWorkFlowService;
import com.xxh.cloud.rule.execute.framework.modules.workflow.service.FlowTaskExecuteQueryService;
import com.xxh.cloud.rule.execute.inference.client.api.RuleExecute2WorkFlowClientService;
import com.xxh.cloud.rule.execute.inference.client.bean.bos.*;
import com.xxh.cloud.rule.execute.inference.client.bean.dto.WorkFlowExecuteAllDTO;
import com.xxh.cloud.rule.execute.inference.client.bean.dto.WorkFlowExecuteByRuleDTO;
import com.xxh.cloud.rule.execute.inference.client.bean.dto.WorkFlowNodeTaskExecuteDTO;
import com.xxh.cloud.rule.execute.inference.client.bean.dto.WorkFlowTriggerNodeTaskExecuteDTO;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

@Log4j2
@Service
public class RuleExecute2WorkFlowClientServiceImpl implements RuleExecute2WorkFlowClientService {


    @Autowired
    private FlowStartProcessWorkFlowService flowStartProcessWorkFlowService;
    @Autowired
    private FlowNodeProcessWorkFlowService flowNodeProcessWorkFlowService;

    @Autowired
    private FlowNodeTriggerWorkFlowService flowNodeTriggerWorkFlowService;


    @Autowired
    private FlowTaskExecuteQueryService flowTaskExecuteQueryService;


    @Override
    public WorkFlowNodeTaskResultBO getWorkFlowNodeTaskResult(String nodeTaskId) {
        FlowTaskNodeInfo taskNode = flowTaskExecuteQueryService.getFlowTaskNodeById(nodeTaskId);
        if(taskNode==null){
            return null;
        }
        WorkFlowNodeTaskResultBO data=new WorkFlowNodeTaskResultBO();
        BeanUtil.copyProperties(taskNode,data);
        return data;
    }

    @Override
    public List<FlowEventTriggerNodeTaskResultBO> getTriggerNodeTaskByDay(String date) {
        List<FlowEventTriggerNodeTaskResultBO> data= flowTaskExecuteQueryService.getTriggerNodeTaskListByDay(date);
        return data;
    }

    @Override
    public void processTriggerNodeTask2Timeout() {
        //todo:待执行
    }


    @Override
    public WorkFlowExecuteProcessAllResultBO executeWorkFlowAll(WorkFlowExecuteAllDTO params) {
        WorkFlowExecuteProcessAllResultBO data=new WorkFlowExecuteProcessAllResultBO();

        PatientExeRuleIdRO patientExeRule= flowTaskExecuteQueryService.getPatientExe2WorkFlowList(params.getPatientUserId());
        //System.out.println("++++ 执行规则:"+ JSON.toJSONString(patientExeRule));

        //
        if(CollUtil.isNotEmpty(patientExeRule.getTriggerNodeTaskIds())){
            for(String triggerNodeTaskId : patientExeRule.getTriggerNodeTaskIds()){
                WorkFlowTriggerNodeTaskExecuteDTO triggerParams=new WorkFlowTriggerNodeTaskExecuteDTO();
                triggerParams.setNodeTaskId(triggerNodeTaskId);
                triggerParams.setData(params.getData());
                triggerParams.setTriggerExeDate(DateUtil.now());
                List<WorkFlowExecuteResultBO> list= this.processTriggerNode(triggerParams);
                data.addStartResult(list);
            }
        }

        //
        if(CollUtil.isNotEmpty(patientExeRule.getExeNotRuleConfigIds())){
            for(String ruleConfigId:patientExeRule.getExeNotRuleConfigIds()){
                WorkFlowExecuteByRuleDTO startParams=new WorkFlowExecuteByRuleDTO(ruleConfigId,params.getData(),params.getOpUserId());
                List<WorkFlowExecuteResultBO> list =this.startExecute(startParams);
                data.addTriggerResult(list);
            }
        }

        return data;
    }


    @Override
    public List<WorkFlowExecuteResultBO> startExecute(WorkFlowExecuteByRuleDTO params) {
        List<WorkFlowExecuteResultBO> data=new ArrayList<>();
        List<Execute2WorkFlowByReturnVO> list= flowStartProcessWorkFlowService.executeByReturn(params.getRuleConfigId(),params.getData(),false,"",params.getOpUserId());
        if(CollUtil.isEmpty(list)){
            return data;
        }
        list.forEach(item->{
            data.add(WorkFlowExecuteResultConvert.getWorkFlowExecuteResultBO(item));
        });
        return data;
    }

    @Override
    public WorkFlowExecuteProcessResultBO processNode(WorkFlowNodeTaskExecuteDTO params) {
        WorkFlowExecuteProcessResultBO data=new WorkFlowExecuteProcessResultBO();
        FlowNodeProcessWorkFlowReturnVO returnVO= flowNodeProcessWorkFlowService.executeByReturn(
                params.getNodeTaskId(),params.getHandleStatus(),params.getOpUserId(), params.getData(),
                false,"","");
        data.setTaskStatus(returnVO.getTaskStatus());
        data.setNodeTaskStatus(returnVO.getNodeTaskStatus());
        if(CollUtil.isNotEmpty(returnVO.getNext())){
            returnVO.getNext().forEach(item->{
                data.addNextTask(WorkFlowExecuteResultConvert.getWorkFlowExecuteResultBO(item));
            });
        }
        return data;
    }

    @Override
    public List<WorkFlowExecuteResultBO> processTriggerNode(WorkFlowTriggerNodeTaskExecuteDTO params) {
        List<WorkFlowExecuteResultBO> data=new ArrayList<>();
        List<Execute2WorkFlowByReturnVO> list= flowNodeTriggerWorkFlowService.executeByReturn(params.getNodeTaskId(),params.getTriggerExeDate(),params.getData(),false);
        if(CollUtil.isEmpty(list)){
            return data;
        }
        list.forEach(item->{
            data.add(WorkFlowExecuteResultConvert.getWorkFlowExecuteResultBO(item));
        });
        return data;
    }




}
