package com.xxh.cloud.rule.execute.framework.modules.meta.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.rule.core.constant.RuleConfigBizTypeEnum;
import com.xxh.cloud.rule.core.entity.flow.XXFlowNodeBO;
import com.xxh.cloud.rule.core.entity.flow.XXFlowRuleBO;
import com.xxh.cloud.rule.execute.framework.modules.meta.bean.FlowRuleConfigGetBO;
import com.xxh.cloud.rule.execute.framework.modules.meta.mapper.FlowRuleConfigGetMapper;
import com.xxh.cloud.rule.execute.framework.modules.meta.service.FlowRuleConfigGetService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class FlowRuleConfigGetServiceImpl implements FlowRuleConfigGetService {


    @Autowired
    private FlowRuleConfigGetMapper flowRuleConfigGetMapper;


    private XXFlowRuleBO getXXFlowRuleBO(FlowRuleConfigGetBO entity){
        if(entity==null){
            return null;
        }
        XXFlowRuleBO data=new XXFlowRuleBO();
        data.setId(entity.getId());
        data.setName(entity.getName());
        data.setRuleCategoryId(entity.getCategoryId());
        data.setVersion(entity.getVersionNo());
        data.setType(entity.getType());
        data.setBizType(entity.getBizType());
        data.setRelType(entity.getRelType());
        data.setRelTwoType(entity.getRelTwoType());
        data.setRelIds(StrJoinSplitUtils.split2List(entity.getRelId()));
        data.setEventId(entity.getEventId());
        if(StrUtil.isNotEmpty(entity.getContent())){
            data.setConfig(JSON.parseObject(entity.getContent(),XXFlowNodeBO.class));
        }
        return data;
    }

    @Override
    public XXFlowRuleBO getById(String ruleConfigId) {
        FlowRuleConfigGetBO entity= flowRuleConfigGetMapper.getFlowRuleConfig2Content(
                new QueryWrapper<FlowRuleConfigGetBO>()
                        .eq("a.id",ruleConfigId)
                        .eq("a.rows_status", YN.YES.getCode())
                        .eq("a.status",YN.YES.getCode())
        );
        return this.getXXFlowRuleBO(entity);
    }

    @Override
    public Map<String, XXFlowRuleBO> getMapByIds(List<String> ruleConfigIds) {
        Map<String, XXFlowRuleBO> data=new HashMap<>();
        List<FlowRuleConfigGetBO> list= flowRuleConfigGetMapper.getFlowRuleConfig2ContentList(
                new QueryWrapper<FlowRuleConfigGetBO>()
                        .in("a.id",ruleConfigIds)
                        .eq("a.rows_status", YN.YES.getCode())
                        .eq("a.status",YN.YES.getCode())
        );
        if(CollUtil.isNotEmpty(list)){
            list.forEach(item->{
                data.put(item.getId(),this.getXXFlowRuleBO(item));
            });
        }
        return data;
    }

    @Override
    public List<XXFlowRuleBO> getListByIds(List<String> ruleConfigIds) {
        List<XXFlowRuleBO> data=new ArrayList<>();
        List<FlowRuleConfigGetBO> list= flowRuleConfigGetMapper.getFlowRuleConfig2ContentList(
                new QueryWrapper<FlowRuleConfigGetBO>()
                        .in("a.id",ruleConfigIds)
                        .eq("a.rows_status", YN.YES.getCode())
                        .eq("a.status",YN.YES.getCode())
        );
        if(CollUtil.isNotEmpty(list)){
            list.forEach(item->{
                data.add(this.getXXFlowRuleBO(item));
            });
        }
        return data;
    }


    @Override
    public List<XXFlowRuleBO> getByBizType(String bizType) {
        List<XXFlowRuleBO> data=new ArrayList<>();
        List<FlowRuleConfigGetBO> list= flowRuleConfigGetMapper.getFlowRuleConfigList(
                new QueryWrapper<FlowRuleConfigGetBO>()
                        .eq("biz_type",bizType)
                        .eq("rows_status", YN.YES.getCode())
                        .eq("status",YN.YES.getCode())
        );
        if(CollUtil.isNotEmpty(list)){
            list.forEach(item->{
                data.add(this.getXXFlowRuleBO(item));
            });
        }
        return data;
    }

    @Override
    public List<XXFlowRuleBO> getByBizType2Inference() {
        return this.getByBizType(RuleConfigBizTypeEnum.CBT_Inference.getCode());
    }

    @Override
    public List<XXFlowRuleBO> getByBizType2WorkFlow() {
        return this.getByBizType(RuleConfigBizTypeEnum.CBT_WorkFlow.getCode());
    }

    @Override
    public List<XXFlowRuleBO> getByBizType2EventWorkFlow() {
        return this.getByBizType(RuleConfigBizTypeEnum.CBT_Event_WorkFlow.getCode());
    }

    @Override
    public List<XXFlowRuleBO> getByBizType2EventWorkFlow(String eventId) {
        List<XXFlowRuleBO> data=new ArrayList<>();
        List<FlowRuleConfigGetBO> list= flowRuleConfigGetMapper.getFlowRuleConfigList(
                new QueryWrapper<FlowRuleConfigGetBO>()
                        .eq("biz_type",RuleConfigBizTypeEnum.CBT_Event_WorkFlow.getCode())
                        .eq("event_id",eventId)
                        .eq("rows_status", YN.YES.getCode())
                        .eq("status",YN.YES.getCode())
        );
        if(CollUtil.isNotEmpty(list)){
            list.forEach(item->{
                data.add(this.getXXFlowRuleBO(item));
            });
        }
        return data;
    }


//    @Override
//    public List<XXFlowRuleBO> getByBizType(String bizType, String relType) {
//        return null;
//    }
//
//    @Override
//    public List<XXFlowRuleBO> getByBizType(String bizType, String relType, String relId) {
//        return null;
//    }
//    @Override
//    public List<XXFlowRuleBO> getListByCategory(String ruleCategoryId) {
//
//        return null;
//    }



}
