package com.xxh.cloud.rule.execute.framework.modules.workflow.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.rule.basic.api.MethodExecuteByCallApi;
import com.xxh.cloud.rule.basic.api.bean.dto.WorkFlowNodeExecuteDTO;
import com.xxh.cloud.rule.basic.constant.WorkFlowNodeHandleStatusEnum;
import com.xxh.cloud.rule.basic.constant.WorkFlowNodeStatusEnum;
import com.xxh.cloud.rule.basic.constant.WorkFlowNodeTriggerStatus;
import com.xxh.cloud.rule.basic.constant.WorkFlowStatusEnum;
import com.xxh.cloud.rule.basic.context.XXRuleRoam;
import com.xxh.cloud.rule.core.constant.FlowResultEnum;
import com.xxh.cloud.rule.core.constant.NodeCheckWay;
import com.xxh.cloud.rule.core.context.XXRuleContext;
import com.xxh.cloud.rule.core.entity.flow.XXFlowRuleBO;
import com.xxh.cloud.rule.execute.framework.modules.base.entity.FlowTaskInfo;
import com.xxh.cloud.rule.execute.framework.modules.base.entity.FlowTaskNodeInfo;
import com.xxh.cloud.rule.execute.framework.modules.base.entity.FlowTaskOperatorInfo;
import com.xxh.cloud.rule.execute.framework.modules.base.entity.RuleFlowExecuteLog;
import com.xxh.cloud.rule.execute.framework.modules.base.service.FlowTaskInfoService;
import com.xxh.cloud.rule.execute.framework.modules.base.service.FlowTaskNodeInfoService;
import com.xxh.cloud.rule.execute.framework.modules.base.service.FlowTaskOperatorInfoService;
import com.xxh.cloud.rule.execute.framework.modules.base.service.RuleFlowExecuteLogService;
import com.xxh.cloud.rule.execute.framework.modules.cache.RuleWorkFlowCacheService;
import com.xxh.cloud.rule.execute.framework.modules.workflow.bean.vos.Execute2WorkFlowByReturnVO;
import com.xxh.cloud.rule.execute.framework.modules.workflow.bean.vos.FlowNodeProcessWorkFlowReturnVO;
import com.xxh.cloud.rule.execute.framework.modules.workflow.bean.vos.WorkFlowNodeTriggerConfig;
import com.xxh.cloud.rule.execute.framework.modules.workflow.service.FlowNodeProcessWorkFlowService;
import com.xxh.cloud.rule.execute.framework.modules.workflow.utils.FlowTaskNodeTriggerRO;
import com.xxh.cloud.rule.execute.framework.modules.workflow.utils.FlowTaskNodeUtils;
import com.xxh.cloud.rule.flow.workflow.XXWorkFlowExecutor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;

@Service
public class FlowNodeProcessWorkFlowServiceImpl implements FlowNodeProcessWorkFlowService {

    @Autowired
    private RuleWorkFlowCacheService ruleWorkFlowCacheService;
    @Autowired
    private FlowTaskInfoService flowTaskInfoService;
    @Autowired
    private FlowTaskNodeInfoService flowTaskNodeInfoService;
    @Autowired
    private FlowTaskOperatorInfoService flowTaskOperatorInfoService;
    @Autowired
    private RuleFlowExecuteLogService ruleFlowExecuteLogService;


    /**
     * WorkFlowNodeHandleStatusEnum
     * */
    @Override
    public FlowNodeProcessWorkFlowReturnVO executeByReturn(String nodeTaskId, String handleStatus, String handleUserId, XXRuleRoam data, boolean isLogMQ, String callWay, String callUserId) {

        FlowNodeProcessWorkFlowReturnVO resultData=new FlowNodeProcessWorkFlowReturnVO();


        FlowTaskNodeInfo flowTaskNode= flowTaskNodeInfoService.getById(nodeTaskId);
        if(flowTaskNode==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        if(WorkFlowNodeStatusEnum.Complete.getCode().equals(flowTaskNode.getStatus())
                || WorkFlowNodeStatusEnum.Kill.getCode().equals(flowTaskNode.getStatus()) ){

            resultData.setNodeTaskStatus(flowTaskNode.getStatus());
            return resultData;
        }
        FlowTaskInfo flowTask= flowTaskInfoService.getById(flowTaskNode.getTaskId());
        //执行
        Set<String> exeUserIds = StrJoinSplitUtils.split2Set(flowTaskNode.getExecuteUserIds());
        Set<String> yesExeUserIds = StrJoinSplitUtils.split2Set(flowTaskNode.getExecuteYesUserIds());
        Set<String> notExeUserIds = StrJoinSplitUtils.split2Set(flowTaskNode.getExecuteNoUserIds());


        if(!notExeUserIds.contains(handleUserId)){
            throw new BusinessException(BusinessBaseErrorEnum.Config_GET_ERROR,"不支持的执行人");
        }

        FlowTaskOperatorInfo entity=new FlowTaskOperatorInfo();
        entity.setId(IdUtil.getSnowflake(1,4).nextIdStr());
        entity.setTaskId(flowTaskNode.getTaskId());
        entity.setTaskNodeId(flowTaskNode.getId());
        entity.setNodeCode(flowTaskNode.getNodeCode());
        entity.setNodeName(flowTaskNode.getNodeName());
        entity.setNodeType(flowTaskNode.getNodeType());
        entity.setHandleUserId(handleUserId);
        entity.setHandleStatus(handleStatus);
        entity.setHandleTime(new Date());
        flowTaskOperatorInfoService.save(entity);

        if(WorkFlowNodeHandleStatusEnum.Kill.getCode().equals(handleStatus)){
            //终止
            notExeUserIds.remove(handleUserId);
            yesExeUserIds.add(handleUserId);

            if(CollUtil.isEmpty(notExeUserIds)){
                flowTaskNode.setExecuteNoUserIds("");
            }else {
                flowTaskNode.setExecuteNoUserIds(StrJoinSplitUtils.join(notExeUserIds));
            }
            flowTaskNode.setExecuteYesUserIds(StrJoinSplitUtils.join(yesExeUserIds));
            flowTaskNode.setStatus(WorkFlowNodeStatusEnum.Kill.getCode());
            flowTaskNodeInfoService.updateById(flowTaskNode);

            flowTask.setStatus(WorkFlowStatusEnum.Fail.getCode());
            flowTask.setEndTime(new Date());
            flowTask.setCompletion(flowTaskNode.getNodeAdvanceRatio());
            flowTaskInfoService.updateById(flowTask);

            resultData.setTaskStatus(flowTask.getStatus());
            resultData.setNodeTaskStatus(flowTaskNode.getStatus());
            return resultData;

        }else if(WorkFlowNodeHandleStatusEnum.Ignore.getCode().equals(handleStatus)){
            //忽略
            notExeUserIds.remove(handleUserId);
            yesExeUserIds.add(handleUserId);

            if(CollUtil.isEmpty(notExeUserIds)){
                flowTaskNode.setExecuteNoUserIds("");
            }else {
                flowTaskNode.setExecuteNoUserIds(StrJoinSplitUtils.join(notExeUserIds));
            }

            flowTaskNode.setExecuteYesUserIds(StrJoinSplitUtils.join(yesExeUserIds));

            if(NodeCheckWay.Check_Or.getCode().equals(flowTaskNode.getNodeCheckWay())){

                flowTaskNode.setStatus(WorkFlowNodeStatusEnum.Kill.getCode());
                flowTaskNode.setEndTime(new Date());
                flowTaskNodeInfoService.updateById(flowTaskNode);

                flowTask.setStatus(WorkFlowStatusEnum.Fail.getCode());
                flowTask.setEndTime(new Date());
                flowTask.setCompletion(flowTaskNode.getNodeAdvanceRatio());
                flowTaskInfoService.updateById(flowTask);

                resultData.setTaskStatus(flowTask.getStatus());
                resultData.setNodeTaskStatus(flowTaskNode.getStatus());
                return resultData;

            }else {
                //todo:判断执行成功的个数
                boolean flag= XXWorkFlowExecutor.getExeCompleteFlag(flowTaskNode.getNodeCheckRatio(),yesExeUserIds.size(), exeUserIds.size());
//                if(flag){
//                    flowTaskNode.setStatus(WorkFlowNodeStatusEnum.Complete.getCode());
//                    flowTaskNode.setEndTime(new Date());
//                    flowTaskNodeInfoService.updateById(flowTaskNode);
//                }else {
//                    flowTaskNode.setStatus(WorkFlowNodeStatusEnum.Ing.getCode());
//                    flowTaskNode.setEndTime(new Date());
//                    flowTaskNodeInfoService.updateById(flowTaskNode);
//                }

            }


        }else if(WorkFlowNodeHandleStatusEnum.Success.getCode().equals(handleStatus)){
            //提交
            notExeUserIds.remove(handleUserId);
            yesExeUserIds.add(handleUserId);

            if(CollUtil.isEmpty(notExeUserIds)){
                flowTaskNode.setExecuteNoUserIds("");
            }else {
                flowTaskNode.setExecuteNoUserIds(StrJoinSplitUtils.join(notExeUserIds));
            }
            //flowTaskNode.setExecuteNoUserIds(StrJoinSplitUtils.join(notExeUserIds));
            flowTaskNode.setExecuteYesUserIds(StrJoinSplitUtils.join(yesExeUserIds));
            if(NodeCheckWay.Check_Or.getCode().equals(flowTaskNode.getNodeCheckWay())){
                flowTaskNode.setStatus(WorkFlowNodeStatusEnum.Complete.getCode());
                flowTaskNode.setEndTime(new Date());
                this.flowTaskNodeTrigger(flowTaskNode);
                flowTaskNodeInfoService.updateById(flowTaskNode);

                resultData.setNodeTaskStatus(flowTaskNode.getStatus());
            }else {
                //todo:判断执行成功的个数
                boolean flag= XXWorkFlowExecutor.getExeCompleteFlag(flowTaskNode.getNodeCheckRatio(),yesExeUserIds.size(), exeUserIds.size());
                if(flag){
                    flowTaskNode.setStatus(WorkFlowNodeStatusEnum.Complete.getCode());
                    flowTaskNode.setEndTime(new Date());
                    this.flowTaskNodeTrigger(flowTaskNode);
                    flowTaskNodeInfoService.updateById(flowTaskNode);
                }else {
                    flowTaskNode.setStatus(WorkFlowNodeStatusEnum.Ing.getCode());
                    flowTaskNode.setEndTime(new Date());
                    //this.flowTaskNodeTrigger(flowTaskNode);
                    flowTaskNodeInfoService.updateById(flowTaskNode);
                }
                resultData.setNodeTaskStatus(flowTaskNode.getStatus());
            }

        }


        if(WorkFlowNodeStatusEnum.Complete.getCode().equals(flowTaskNode.getStatus())){
            if(WorkFlowNodeTriggerStatus.TS_NONE.getCode().equals(flowTaskNode.getTriggerStatus())){
                //执行下一个节点

                XXFlowRuleBO ruleConfig = ruleWorkFlowCacheService.getWorkFlowRule(flowTask.getRuleFlowId());
                XXRuleContext context= XXWorkFlowExecutor.next(flowTask.getId(),ruleConfig,flowTaskNode.getNodeCode(),data);
                List<Execute2WorkFlowByReturnVO> nextReturnData = this.nextNodeExecute(context,flowTask.getRuleFlowId(),flowTask.getId());

                if(CollUtil.isEmpty(nextReturnData) || context.isRuleEnd()){
                    flowTask.setStatus(WorkFlowStatusEnum.Success.getCode());
                    flowTask.setEndTime(new Date());
                    flowTask.setCompletion(flowTaskNode.getNodeAdvanceRatio());
                    flowTaskInfoService.updateById(flowTask);

                    resultData.setTaskStatus(flowTask.getStatus());
                    resultData.setNext(nextReturnData);
                    return resultData;
                }

                resultData.setTaskStatus(flowTask.getStatus());
                resultData.setNext(nextReturnData);
                return resultData;

            }else {
                //等待自动执行
                resultData.setTaskStatus(flowTask.getStatus());
                return resultData;
            }
        }

        resultData.setTaskStatus(flowTask.getStatus());
        return resultData;
    }

    private void flowTaskNodeTrigger(FlowTaskNodeInfo flowTaskNode){
        if(!WorkFlowNodeStatusEnum.Complete.getCode().equals(flowTaskNode.getStatus())){
            return;
        }

        if(!YN.YES.getCode().equals(flowTaskNode.getIsCronTrigger())){
            return;
        }

        if(WorkFlowNodeTriggerStatus.TS_NOT.getCode().equals(flowTaskNode.getTriggerStatus()) && StrUtil.isEmpty(flowTaskNode.getTriggerDateList())){
            WorkFlowNodeTriggerConfig workFlowNodeTriggerConfig=JSON.parseObject(flowTaskNode.getTriggerConfig(),WorkFlowNodeTriggerConfig.class);
            FlowTaskNodeTriggerRO taskNodeTriggerRO = FlowTaskNodeUtils.getTriggerDateList2(workFlowNodeTriggerConfig);
            String triggerDateList=StrJoinSplitUtils.join(taskNodeTriggerRO.getDateList());
            flowTaskNode.setTriggerDateList(triggerDateList);
            flowTaskNode.setTriggerDateListNo(triggerDateList);
            flowTaskNode.setTriggerDateStart(taskNodeTriggerRO.getStart());
            flowTaskNode.setTriggerDateEnd(taskNodeTriggerRO.getEnd());
        }

    }


    @Override
    public void executeByMQ(String nodeTaskId, String handleStatus, String handleUserId, XXRuleRoam data, boolean isLogMQ, String callWay, String callUserId) {

    }

    @Override
    public void executeByCall(String nodeTaskId, String handleStatus, String handleUserId, XXRuleRoam data, boolean isLogMQ, MethodExecuteByCallApi api, String callWay, String callUserId) {

    }


    /**
     * execute
     * */
    private List<Execute2WorkFlowByReturnVO> nextNodeExecute(XXRuleContext context,String ruleConfigId,String taskId ){

        List<WorkFlowNodeExecuteDTO> flowNodeExeList= context.getReturnWorkFlowNodeList();
        if(CollUtil.isEmpty(flowNodeExeList)){
            return null;
        }

        List<Execute2WorkFlowByReturnVO> returnList=new ArrayList<>();

        List<FlowTaskNodeInfo> taskNodeList= FlowTaskNodeUtils.getFlowTaskNodeInfoList(taskId,ruleConfigId,flowNodeExeList,returnList);
        List<RuleFlowExecuteLog>  logList = FlowTaskNodeUtils.getRuleFlowExecuteLogList(ruleConfigId,taskId,context.getNodeLogList());

        if(CollUtil.isNotEmpty(taskNodeList)){
            flowTaskNodeInfoService.saveBatch(taskNodeList);
        }

        if(CollUtil.isNotEmpty(logList)){
            ruleFlowExecuteLogService.saveBatch(logList);
        }
        return returnList;
    }






}
