package com.xxh.cloud.rule.execute.framework.modules.workflow.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.framework.common.utils.XXDateUtils;
import com.xxh.cloud.rule.basic.api.bean.dto.WorkFlowNodeExecuteDTO;
import com.xxh.cloud.rule.basic.constant.WorkFlowNodeStatusEnum;
import com.xxh.cloud.rule.basic.constant.WorkFlowNodeTriggerStatus;
import com.xxh.cloud.rule.basic.constant.WorkFlowStatusEnum;
import com.xxh.cloud.rule.basic.context.XXRuleRoam;
import com.xxh.cloud.rule.core.context.XXRuleContext;
import com.xxh.cloud.rule.core.entity.flow.XXFlowRuleBO;
import com.xxh.cloud.rule.execute.framework.modules.base.entity.FlowTaskInfo;
import com.xxh.cloud.rule.execute.framework.modules.base.entity.FlowTaskNodeInfo;
import com.xxh.cloud.rule.execute.framework.modules.base.entity.RuleFlowExecuteLog;
import com.xxh.cloud.rule.execute.framework.modules.base.service.FlowTaskInfoService;
import com.xxh.cloud.rule.execute.framework.modules.base.service.FlowTaskNodeInfoService;
import com.xxh.cloud.rule.execute.framework.modules.base.service.RuleFlowExecuteLogService;
import com.xxh.cloud.rule.execute.framework.modules.cache.RuleWorkFlowCacheService;
import com.xxh.cloud.rule.execute.framework.modules.workflow.bean.vos.Execute2WorkFlowByReturnVO;
import com.xxh.cloud.rule.execute.framework.modules.workflow.service.FlowNodeTriggerWorkFlowService;
import com.xxh.cloud.rule.execute.framework.modules.workflow.utils.FlowTaskNodeUtils;
import com.xxh.cloud.rule.flow.workflow.XXWorkFlowExecutor;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Log4j2
@Service
public class FlowNodeTriggerWorkFlowServiceImpl implements FlowNodeTriggerWorkFlowService {



    @Autowired
    private FlowTaskInfoService flowTaskInfoService;
    @Autowired
    private FlowTaskNodeInfoService flowTaskNodeInfoService;

    @Autowired
    private RuleFlowExecuteLogService ruleFlowExecuteLogService;

//    @Autowired
//    private FlowTaskInfoService flowTaskInfoService;

    @Autowired
    private RuleWorkFlowCacheService ruleWorkFlowCacheService;

    @Override
    public List<Execute2WorkFlowByReturnVO> executeByReturn(String nodeTaskId ,String triggerExeDate, XXRuleRoam data, boolean isLogMQ) {
        FlowTaskNodeInfo flowTaskNode= flowTaskNodeInfoService.getById(nodeTaskId);
        if(flowTaskNode==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(!WorkFlowNodeStatusEnum.Complete.getCode().equals(flowTaskNode.getStatus())){
            return null;
        }


        if(WorkFlowNodeTriggerStatus.TS_NOT.getCode().equals(flowTaskNode.getTriggerStatus())
            || WorkFlowNodeTriggerStatus.TS_ING.getCode().equals(flowTaskNode.getTriggerStatus())){

            Date nowDate=new Date();
            if (nowDate.compareTo(flowTaskNode.getTriggerDateEnd())>=0){
                log.debug("+++++当前时间大于截止日期,流程终止");
                flowTaskNode.setTriggerStatus(WorkFlowNodeTriggerStatus.TS_Success.getCode());
                flowTaskNode.setUpdateDate(new Date());
                flowTaskNodeInfoService.updateById(flowTaskNode);
                //终止流程
                FlowTaskInfo flowTask= flowTaskInfoService.getById(flowTaskNode.getTaskId());
                flowTask.setStatus(WorkFlowStatusEnum.Fail.getCode());
                flowTask.setEndTime(new Date());
                flowTask.setCompletion(flowTaskNode.getNodeAdvanceRatio());
                flowTaskInfoService.updateById(flowTask);
            }

            boolean flag= XXDateUtils.between(nowDate,flowTaskNode.getTriggerDateStart(),flowTaskNode.getTriggerDateEnd());
            if(flag==false){
                log.debug("+++++执行:不在开始时间和截止时间区间中");
                return null;
            }


            List<String> notDateList= StrJoinSplitUtils.split2List(flowTaskNode.getTriggerDateListNo());
            List<String> yesDateList=StrJoinSplitUtils.split2List(flowTaskNode.getTriggerDateListYes());

            if(CollUtil.isEmpty(notDateList)){
                log.debug("+++++ 执行:没有未执行时间");
                return null;
            }

            XXFlowRuleBO ruleConfig = ruleWorkFlowCacheService.getWorkFlowRule(flowTaskNode.getRuleFlowId());

            if(ruleConfig==null){
                log.debug("+++++ 执行:没有规则配置，配置ID : "+flowTaskNode.getRuleFlowId());
                return null;
            }

            XXRuleContext context= XXWorkFlowExecutor.next(flowTaskNode.getTaskId(),ruleConfig,flowTaskNode.getNodeCode(),data);
            List<WorkFlowNodeExecuteDTO> flowNodeExeList= context.getReturnWorkFlowNodeList();


            if(CollUtil.isEmpty(flowNodeExeList)){
                List<Execute2WorkFlowByReturnVO> returnList=new ArrayList<>();

                if(StrUtil.isNotEmpty(triggerExeDate)){
                    notDateList.remove(triggerExeDate);
                    yesDateList.add(triggerExeDate);
                }


                if(CollUtil.isEmpty(notDateList)){ //无法执行下面，且未执行未空
                    flowTaskNode.setTriggerDateListNo("");
                    flowTaskNode.setTriggerDateListYes(StrJoinSplitUtils.join(yesDateList));
                    flowTaskNode.setTriggerStatus(WorkFlowNodeTriggerStatus.TS_Success.getCode());
                    flowTaskNode.setUpdateDate(new Date());
                    flowTaskNodeInfoService.updateById(flowTaskNode);
                    //终止流程
                    FlowTaskInfo flowTask= flowTaskInfoService.getById(flowTaskNode.getTaskId());
                    flowTask.setStatus(WorkFlowStatusEnum.Fail.getCode());
                    flowTask.setEndTime(new Date());
                    flowTask.setCompletion(flowTaskNode.getNodeAdvanceRatio());
                    flowTaskInfoService.updateById(flowTask);
                }else {
                    flowTaskNode.setTriggerDateListNo(StrJoinSplitUtils.join(notDateList));
                    flowTaskNode.setTriggerDateListYes(StrJoinSplitUtils.join(yesDateList));
                    flowTaskNode.setTriggerStatus(WorkFlowNodeTriggerStatus.TS_ING.getCode());
                    flowTaskNode.setUpdateDate(new Date());
                    flowTaskNodeInfoService.updateById(flowTaskNode);
                }
                return null;
            }

            //下一步
            List<Execute2WorkFlowByReturnVO> returnList=new ArrayList<>();
            if(StrUtil.isNotEmpty(triggerExeDate)){
                notDateList.remove(triggerExeDate);
                yesDateList.add(triggerExeDate);
            }

            if(CollUtil.isEmpty(notDateList)){
                flowTaskNode.setTriggerDateListNo("");
            }else {
                flowTaskNode.setTriggerDateListNo(StrJoinSplitUtils.join(notDateList));
            }
            flowTaskNode.setTriggerDateListYes(StrJoinSplitUtils.join(yesDateList));
            flowTaskNode.setTriggerStatus(WorkFlowNodeTriggerStatus.TS_Success.getCode());
            flowTaskNode.setUpdateDate(new Date());
            flowTaskNodeInfoService.updateById(flowTaskNode);


            List<FlowTaskNodeInfo> taskNodeList= FlowTaskNodeUtils.getFlowTaskNodeInfoList(flowTaskNode.getTaskId(),flowTaskNode.getRuleFlowId(),flowNodeExeList,returnList);
            List<RuleFlowExecuteLog>  logList=FlowTaskNodeUtils.getRuleFlowExecuteLogList(flowTaskNode.getRuleFlowId(),flowTaskNode.getTaskId(),context.getNodeLogList());

            if(CollUtil.isNotEmpty(taskNodeList)){
                flowTaskNodeInfoService.saveBatch(taskNodeList);
            }
            if(CollUtil.isNotEmpty(logList)){
                ruleFlowExecuteLogService.saveBatch(logList);
            }

            if(context.isRuleEnd()){
                FlowTaskInfo flowTask= flowTaskInfoService.getById(flowTaskNode.getTaskId());
                flowTask.setStatus(WorkFlowStatusEnum.Success.getCode());
                flowTask.setEndTime(new Date());
                flowTask.setCompletion(flowTaskNode.getNodeAdvanceRatio());
                flowTaskInfoService.updateById(flowTask);
            }

            return returnList;
        }

        return null;
    }


}
