package com.xxh.cloud.rule.execute.framework.modules.workflow.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.rule.basic.api.MethodExecuteByCallApi;
import com.xxh.cloud.rule.basic.api.bean.dto.WorkFlowNodeExecuteDTO;
import com.xxh.cloud.rule.basic.constant.*;
import com.xxh.cloud.rule.basic.context.XXRuleRoam;
import com.xxh.cloud.rule.core.constant.FlowResultEnum;
import com.xxh.cloud.rule.core.context.XXRuleContext;
import com.xxh.cloud.rule.core.context.XXRuleNodeLogBean;
import com.xxh.cloud.rule.core.entity.flow.XXFlowRuleBO;
import com.xxh.cloud.rule.execute.framework.modules.base.entity.FlowTaskInfo;
import com.xxh.cloud.rule.execute.framework.modules.base.entity.FlowTaskNodeInfo;
import com.xxh.cloud.rule.execute.framework.modules.base.entity.RuleFlowExecuteLog;
import com.xxh.cloud.rule.execute.framework.modules.base.service.FlowTaskInfoService;
import com.xxh.cloud.rule.execute.framework.modules.base.service.FlowTaskNodeInfoService;
import com.xxh.cloud.rule.execute.framework.modules.base.service.RuleFlowExecuteLogService;
import com.xxh.cloud.rule.execute.framework.modules.cache.RuleWorkFlowCacheService;
import com.xxh.cloud.rule.execute.framework.modules.rule.bean.vos.ExecuteActionMethodBO;
import com.xxh.cloud.rule.execute.framework.modules.workflow.bean.vos.Execute2WorkFlowByReturnVO;
import com.xxh.cloud.rule.execute.framework.modules.workflow.bean.vos.WorkFlowNodeTriggerConfig;
import com.xxh.cloud.rule.execute.framework.modules.workflow.service.FlowStartProcessWorkFlowService;
import com.xxh.cloud.rule.execute.framework.modules.workflow.utils.FlowTaskNodeUtils;
import com.xxh.cloud.rule.flow.workflow.XXWorkFlowExecutor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service
public class FlowStartProcessWorkFlowServiceImpl implements FlowStartProcessWorkFlowService {


    @Autowired
    private RuleWorkFlowCacheService ruleWorkFlowCacheService;
    @Autowired
    private FlowTaskInfoService flowTaskInfoService;
    @Autowired
    private FlowTaskNodeInfoService flowTaskNodeInfoService;
    @Autowired
    private RuleFlowExecuteLogService ruleFlowExecuteLogService;


    private XXFlowRuleBO getXXFlowRule(String ruleConfigId){
        XXFlowRuleBO ruleConfig = ruleWorkFlowCacheService.getWorkFlowRule(ruleConfigId);
        if(ruleConfig==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS,"规则配置不存在");
        }
        //todo:判断规则分类
        return ruleConfig;
    }


    @Override
    public List<Execute2WorkFlowByReturnVO> executeByReturn(String ruleConfigId, XXRuleRoam data, boolean isLogMQ, String callWay, String callUserId) {
        XXFlowRuleBO ruleConfig=this.getXXFlowRule(ruleConfigId);
//
        String taskId= StrUtil.format("FT{}",IdUtil.getSnowflake(1,1).nextIdStr());
        XXRuleContext context= XXWorkFlowExecutor.start(taskId,ruleConfig,data);

        List<WorkFlowNodeExecuteDTO> flowNodeExeList= context.getReturnWorkFlowNodeList();

        if(CollUtil.isEmpty(flowNodeExeList)){
            return null;
        }

        List<Execute2WorkFlowByReturnVO> returnList=new ArrayList<>();

        FlowTaskInfo flowTask= FlowTaskNodeUtils.getFlowTaskInfo(taskId,data.getPatientId(),ruleConfig);

        List<FlowTaskNodeInfo> taskNodeList= FlowTaskNodeUtils.getFlowTaskNodeInfoList(taskId, flowTask.getRuleFlowId(),flowNodeExeList,returnList);

        List<RuleFlowExecuteLog>  logList = FlowTaskNodeUtils.getRuleFlowExecuteLogList(flowTask.getRuleFlowId(),flowTask.getId(),context.getNodeLogList());

        if(context.isRuleEnd()){
            flowTask.setStatus(WorkFlowStatusEnum.Success.getCode());
        }

        flowTaskInfoService.save(flowTask);

        if(CollUtil.isNotEmpty(taskNodeList)){
            flowTaskNodeInfoService.saveBatch(taskNodeList);
        }

        if(CollUtil.isNotEmpty(logList)){
            ruleFlowExecuteLogService.saveBatch(logList);
        }

        return returnList;
    }

    @Override
    public void executeByMQ(String ruleConfigId, XXRuleRoam data, boolean isLogMQ, String callWay, String callUserId) {
        //todo:未实现
    }

    @Override
    public void executeByCall(String ruleConfigId, XXRuleRoam data, boolean isLogMQ, MethodExecuteByCallApi api, String callWay, String callUserId) {
        //todo:未实现
    }











}
