package com.xxh.cloud.rule.execute.framework.modules.workflow.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.rule.basic.constant.WorkFlowNodeStatusEnum;
import com.xxh.cloud.rule.basic.constant.WorkFlowNodeTriggerStatus;
import com.xxh.cloud.rule.basic.constant.WorkFlowStatusEnum;
import com.xxh.cloud.rule.core.constant.RuleConfigBizTypeEnum;
import com.xxh.cloud.rule.execute.framework.modules.base.entity.FlowTaskInfo;
import com.xxh.cloud.rule.execute.framework.modules.base.entity.FlowTaskNodeInfo;
import com.xxh.cloud.rule.execute.framework.modules.base.service.FlowTaskInfoService;
import com.xxh.cloud.rule.execute.framework.modules.base.service.FlowTaskNodeInfoService;
import com.xxh.cloud.rule.execute.framework.modules.cache.RuleWorkFlowCacheService;
import com.xxh.cloud.rule.execute.framework.modules.workflow.bean.ros.PatientExeRuleIdRO;
import com.xxh.cloud.rule.execute.framework.modules.workflow.mapper.FlowEventTaskQueryMapper;
import com.xxh.cloud.rule.execute.framework.modules.workflow.service.FlowTaskExecuteQueryService;
import com.xxh.cloud.rule.execute.inference.client.bean.bos.FlowEventTriggerNodeTaskResultBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

@Service
public class FlowTaskExecuteQueryServiceImpl implements FlowTaskExecuteQueryService {



    @Autowired
    private RuleWorkFlowCacheService ruleWorkFlowCacheService;
    @Autowired
    private FlowTaskInfoService flowTaskInfoService;
    @Autowired
    private FlowTaskNodeInfoService flowTaskNodeInfoService;

    @Autowired
    private FlowEventTaskQueryMapper flowEventTaskQueryMapper;


    @Override
    public PatientExeRuleIdRO getPatientExe2WorkFlowList(String patientUserId){

        PatientExeRuleIdRO result=new PatientExeRuleIdRO();

        //执行中
        List<FlowTaskInfo> flowTaskList= flowTaskInfoService.list(
                new QueryWrapper<FlowTaskInfo>()
                        .eq("patient_user_id",patientUserId)
                        .eq("rows_status", YN.YES.getCode())
                        .eq("rule_flow_biz_type", RuleConfigBizTypeEnum.CBT_WorkFlow.getCode())
                        .in("status",WorkFlowStatusEnum.None.getCode(),WorkFlowStatusEnum.Ing.getCode())
        );

        //执行中任务ID
        Set<String> flowTaskIds=new HashSet<>();
        if(CollUtil.isNotEmpty(flowTaskList)){
            flowTaskList.forEach(item->{
                flowTaskIds.add(item.getId());
                result.addExeIngRuleId(item.getRuleFlowId());
            });
        }

        //可执行规则ID
        List<String> exeNotRuleConfigIds= ruleWorkFlowCacheService.getWorkFlowRuleId2WorkFlowNot(result.getExeIngRuleConfigIds());
        result.setExeNotRuleConfigIds(exeNotRuleConfigIds);

        //执行中任务节点
        if(CollUtil.isNotEmpty(flowTaskIds)){
            List<FlowTaskNodeInfo> flowTaskNodeList= flowTaskNodeInfoService.list(
                    new QueryWrapper<FlowTaskNodeInfo>()
                            .in("task_id",flowTaskIds)
                            .eq("status",WorkFlowNodeStatusEnum.Complete.getCode())
                            .eq("is_cron_trigger",YN.YES.getCode())
                            .in("trigger_status", WorkFlowNodeTriggerStatus.TS_NOT.getCode(),WorkFlowNodeTriggerStatus.TS_ING.getCode())
            );
            //所有可执行（自动执行）的任务节点
            flowTaskNodeList.forEach(item->{
                result.addTriggerNodeTaskId(item.getId());
            });

        }

        return result;
    }

    @Override
    public List<String> getEventFlowListByEventId(String eventId) {
        return ruleWorkFlowCacheService.getWorkFlowRuleId2EventWorkFlow2DB(eventId);
    }




    @Override
    public List<FlowEventTriggerNodeTaskResultBO> getTriggerNodeTaskListByDay(String date) {
        List<FlowEventTriggerNodeTaskResultBO> list= flowEventTaskQueryMapper.getFlowEventTriggerNodeTaskResultList(
                new QueryWrapper<FlowEventTriggerNodeTaskResultBO>()
                        .eq("b.status",WorkFlowNodeStatusEnum.Complete.getCode())
                        .in("b.trigger_status",WorkFlowNodeTriggerStatus.TS_NOT.getCode(),WorkFlowNodeTriggerStatus.TS_ING.getCode())
                        .like("b.trigger_date_list_no",date)
        );
        return  list;

    }

    @Override
    public FlowTaskNodeInfo getFlowTaskNodeById(String nodeTaskId) {
        FlowTaskNodeInfo data=flowTaskNodeInfoService.getById(nodeTaskId);
        return data;
    }


}
