package com.xxh.cloud.rule.execute.framework.modules.workflow.utils;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.framework.common.constant.DateTimeUnitTypeEnum;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.rule.basic.api.bean.dto.WorkFlowNodeExecuteDTO;
import com.xxh.cloud.rule.basic.constant.*;
import com.xxh.cloud.rule.core.constant.HealthDateLimitUnitEnum;
import com.xxh.cloud.rule.core.constant.NodeTriggerTypeEnum;
import com.xxh.cloud.rule.core.context.XXRuleNodeLogBean;
import com.xxh.cloud.rule.core.entity.flow.XXFlowRuleBO;
import com.xxh.cloud.rule.core.utils.DateBetweenUtils;
import com.xxh.cloud.rule.execute.framework.modules.base.entity.FlowTaskInfo;
import com.xxh.cloud.rule.execute.framework.modules.base.entity.FlowTaskNodeInfo;
import com.xxh.cloud.rule.execute.framework.modules.base.entity.RuleFlowExecuteLog;
import com.xxh.cloud.rule.execute.framework.modules.rule.bean.vos.ExecuteActionMethodBO;
import com.xxh.cloud.rule.execute.framework.modules.workflow.bean.vos.Execute2WorkFlowByReturnVO;
import com.xxh.cloud.rule.execute.framework.modules.workflow.bean.vos.WorkFlowNodeTriggerConfig;

import javax.xml.crypto.Data;
import java.util.*;

public class FlowTaskNodeUtils {


    public static FlowTaskInfo getFlowTaskInfo(String id, String patientId, XXFlowRuleBO ruleConfig){
        FlowTaskInfo entity=new FlowTaskInfo();
        entity.setId(id);

        entity.setFlowUrgent(WorkFlowUrgentEnum.Low.getCode()); //
        entity.setGrade(WorkFlowGradeEnum.One.getCode());
        entity.setProjectCode(ruleConfig.getProjectCode());
        entity.setRuleFlowId(ruleConfig.getId());
        entity.setRuleFlowName(ruleConfig.getName());
        entity.setRuleFlowCategoryId(ruleConfig.getRuleCategoryId());
        entity.setRuleFlowVersion(ruleConfig.getVersion());
//        entity.set
        entity.setRuleFlowBizType(ruleConfig.getBizType());
        entity.setStartTime(new Date());
//        entity.setEndTime();
        entity.setStatus(WorkFlowStatusEnum.Ing.getCode());
        entity.setCompletion(0);
        entity.setDescription("业务调用");
//        entity.setThisStepNo();
//        entity.setThisStepName();
//        entity.setThisStepId();
        entity.setPatientUserId(patientId);
        entity.setRowsStatus(YN.YES.getCode());
        entity.setCreateDate(new Date());
        entity.setUpdateDate(new Date());
        return entity;
    }



    public static List<FlowTaskNodeInfo> getFlowTaskNodeInfoList(String taskId ,String ruleFlowId, List<WorkFlowNodeExecuteDTO> list , List<Execute2WorkFlowByReturnVO> returnList){
        List<FlowTaskNodeInfo> data=new ArrayList<>();
        list.forEach(item->{
            ExecuteActionMethodBO methodBO=new ExecuteActionMethodBO(item.getActionId(),item.getActionName(),item.getMethodCode(),item.getMethodName(),item.getParams());
            FlowTaskNodeInfo node= getFlowTaskNodeInfo(taskId,ruleFlowId,item,methodBO);
            data.add(node);
            returnList.add(new Execute2WorkFlowByReturnVO(node.getTaskId(),node.getId(),node.getRuleFlowId(),node.getStartUserId(),
                    node.getIsExecuteUser(),node.getExecuteUserIds() ,methodBO,node.getNodeCode()));

        });
        return data;
    }



    public static FlowTaskNodeInfo getFlowTaskNodeInfo(String taskId,String ruleFlowId, WorkFlowNodeExecuteDTO item , ExecuteActionMethodBO method){
        FlowTaskNodeInfo entity=new FlowTaskNodeInfo();
        entity.setId(IdUtil.getSnowflake(1,2).nextIdStr());
        entity.setRuleFlowId(ruleFlowId);
        entity.setTaskId(taskId);
        entity.setNodeCode(item.getNodeId());
        entity.setNodeName(item.getNodeName());
        entity.setNodeType(item.getNodeType());
        entity.setNodeRuleType(item.getNodeRuleType());
        entity.setNodeAdvanceRatio(item.getAdvanceRatio());
        entity.setNodeUp(item.getPreNodeId());
        entity.setNodeNext(null);


        entity.setIsCronTrigger(item.getIsTrigger());

        WorkFlowNodeTriggerConfig workFlowNodeTriggerConfig=null;
        if(YN.YES.getCode().equals(item.getIsTrigger())){
             workFlowNodeTriggerConfig=new WorkFlowNodeTriggerConfig(item.getTriggerStartNum(),
                    item.getTriggerEndNum(),item.getTriggerEndUnit(),item.getTriggerType(),item.getTriggerLoopNum(),item.getTriggerLoopUnit()
            );
            entity.setTriggerConfig(JSON.toJSONString(workFlowNodeTriggerConfig));
            entity.setTriggerStatus(WorkFlowNodeTriggerStatus.TS_NOT.getCode());
        }else {
            entity.setTriggerStatus(WorkFlowNodeTriggerStatus.TS_NONE.getCode());
        }

        entity.setActionConfig(JSON.toJSONString(method));

        entity.setStartTime(new Date());
        entity.setIsExecuteUser(item.getIsExecutor());

//        if(YN.YES.getCode().equals(item.getIsExecutor())){
//            entity.setExecuteUserIds(StrJoinSplitUtils.join(item.getExecutorIds()));
//            entity.setExecuteYesUserIds(null);
//            entity.setExecuteNoUserIds(entity.getExecuteUserIds());
//            entity.setNodeCheckWay(item.getCheckWay());
//            entity.setNodeCheckRatio(item.getCheckRatio());
//            entity.setStatus(WorkFlowNodeStatusEnum.Complete.getCode());
//            entity.setDescription("有执行人");
//        }else {
//            entity.setStatus(WorkFlowNodeStatusEnum.Complete.getCode());
//            entity.setEndTime(new Date());
//            entity.setDescription("无执行人，自动执行");
//        }

        if(YN.YES.getCode().equals(item.getIsExecutor())){
            entity.setExecuteUserIds(StrJoinSplitUtils.join(item.getExecutorIds()));
            entity.setExecuteYesUserIds(null);
            entity.setExecuteNoUserIds(entity.getExecuteUserIds());
            entity.setNodeCheckWay(item.getCheckWay());
            entity.setNodeCheckRatio(item.getCheckRatio());
        }

        entity.setStatus(WorkFlowNodeStatusEnum.Complete.getCode());
        entity.setEndTime(new Date());
        entity.setDescription("自动执行");

        if(workFlowNodeTriggerConfig!=null){
            FlowTaskNodeTriggerRO taskNodeTriggerRO= getTriggerDateList2(workFlowNodeTriggerConfig);
            String triggerDateList=StrJoinSplitUtils.join(taskNodeTriggerRO.getDateList());
            entity.setTriggerDateList(triggerDateList);
            entity.setTriggerDateListNo(triggerDateList);
            entity.setTriggerDateStart(taskNodeTriggerRO.getStart());
            entity.setTriggerDateEnd(taskNodeTriggerRO.getEnd());
        }


        return entity;
    }






    public static List<RuleFlowExecuteLog> getRuleFlowExecuteLogList(String ruleId , String taskId, List<XXRuleNodeLogBean> list){
        if(CollUtil.isEmpty(list)){
            return null;
        }
        List<RuleFlowExecuteLog> data=new ArrayList<>();
        list.forEach(item->{
            data.add(getRuleFlowExecuteLog(ruleId,taskId, item));
        });
        return data;
    }


    public static RuleFlowExecuteLog getRuleFlowExecuteLog(String ruleId ,String taskId, XXRuleNodeLogBean item){
        RuleFlowExecuteLog entity=new RuleFlowExecuteLog();
        entity.setId(item.getId());
        entity.setType(item.getType());
        entity.setRuleId(ruleId);
        entity.setTaskId(taskId);
        entity.setNodeId(item.getNodeId());
        entity.setNodeCode(item.getNodeId());
        entity.setNodeName(item.getNodeName());
        entity.setNodeType(item.getNodeType());
        entity.setNodeRuleType(item.getNodeRuleType());
        entity.setStartTime(item.getStart());
        entity.setStartDate(item.getEnd());
//        entity.setTimeNum();
        if(item.isFlag()){
            entity.setNodeExecuteStatus(YN.YES.getCode());
        }else {
            entity.setNodeExecuteStatus(YN.NO.getCode());
        }

        if("条件".equals(item.getType())){
            if(CollUtil.isNotEmpty(item.getConditionList())){
                entity.setExecuteResult(JSON.toJSONString(item.getConditionList()));
            }

        }else if("工作流执行动作".equals(item.getType())){
            if(item.getWorkFlowResult()!=null){
                entity.setExecuteResult(JSON.toJSONString(item.getWorkFlowResult()));
            }
        }
        entity.setOpDate(new Date());
        return entity;
    }


    public static FlowTaskNodeTriggerRO getTriggerDateList2(WorkFlowNodeTriggerConfig config){
        FlowTaskNodeTriggerRO flowTaskNodeTrigger=FlowTaskNodeTriggerUtils.getDataList(config.getS(), config.getE(),config.getU(),config.getLt(), config.getLn(),config.getLu());
        return flowTaskNodeTrigger;
    }








}
