package com.xxh.cloud.rule.flow.executor;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.rule.core.constant.RuleNodeTypeEnum;
import com.xxh.cloud.rule.core.constant.XXActionNodeInterExeTypeEnum;
import com.xxh.cloud.rule.core.constant.XXConditionNodeExeRelationTypeEnum;
import com.xxh.cloud.rule.core.constant.FlowResultEnum;
import com.xxh.cloud.rule.core.context.XXRuleContext;
import com.xxh.cloud.rule.core.entity.flow.XXFlowNodeBO;
import com.xxh.cloud.rule.core.executor.condition.ConditionExecutor;
import com.xxh.cloud.rule.core.executor.method.MethodExecutor;
import com.xxh.cloud.rule.flow.executor.bean.ExeConditionParamBO;
import com.xxh.cloud.rule.flow.executor.wr.XXWeightedRoundRobin;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class XXRuleFlowHandler {



    private static final Logger logger = LoggerFactory.getLogger(XXRuleFlowHandler.class);


    /**
     * 执行
     * */
    public FlowResultEnum handle(XXRuleContext context, XXFlowNodeBO node){
           //执行本节点
           boolean nodeFlag= this.nodeProcess(context,node);

           if(nodeFlag==false){
               return FlowResultEnum.Kill;
           }

           //执行合流和条件节点
           if(CollUtil.isNotEmpty(node.getConditionNodes())){
               ExeConditionParamBO exeConditionParamBO=isConditionNode(node.getConditionNodes());
               boolean branchFlag=false;
               if(exeConditionParamBO.isCondition()){
                    //条件节点(1.任何一个节点通过就通过 2.执行所有节点后,有一个通过则通过 3.执行所有节点后,所有节点通过则通过)

                   //1.任何一个节点通过就通过
                   if(XXConditionNodeExeRelationTypeEnum.ANY.getCode().equals(exeConditionParamBO.getConditionExe())){
                       for(XXFlowNodeBO branchNode : node.getConditionNodes()){
                           FlowResultEnum branchResult =  this.handle(context,branchNode);
                           System.out.println(StrUtil.format("===节点{}下,条件节点执行结果 : {} -{}",node.getProperties().getTitle(),branchResult.getCode(),branchResult.getName()));
                           if(FlowResultEnum.End==branchResult || FlowResultEnum.Pass==branchResult){
                               branchFlag=true;
                               break;
                           }
                       }
                   }

                   //2.执行所有节点后,有一个通过则通过
                   if(XXConditionNodeExeRelationTypeEnum.ALL.getCode().equals(exeConditionParamBO.getConditionExe())){
                       Map<String,FlowResultEnum> branchResultMap=new HashMap<>();
                       for(XXFlowNodeBO branchNode : node.getConditionNodes()){
                           FlowResultEnum branchResult =  this.handle(context,branchNode);
                           branchResultMap.put(branchNode.getNodeId(),branchResult);
                       }

                       for(String branchNodeId:branchResultMap.keySet()){
                           FlowResultEnum resultEnum=branchResultMap.get(branchNodeId);
                           if(FlowResultEnum.End==resultEnum || FlowResultEnum.Pass==resultEnum){
                               branchFlag=true;
                               break;
                           }
                       }

                   }

                   //3.执行所有节点后,所有节点通过则通过
                   if(XXConditionNodeExeRelationTypeEnum.ALL_Success.getCode().equals(exeConditionParamBO.getConditionExe()) ){
                       Map<String,FlowResultEnum> branchResultMap=new HashMap<>();
                       for(XXFlowNodeBO branchNode : node.getConditionNodes()){
                           FlowResultEnum branchResult =  this.handle(context,branchNode);
                           branchResultMap.put(branchNode.getNodeId(),branchResult);
                       }
                       branchFlag=true;
                       for(String branchNodeId:branchResultMap.keySet()){
                           FlowResultEnum resultEnum=branchResultMap.get(branchNodeId);
                           if(FlowResultEnum.Kill==resultEnum){
                               branchFlag=false;
                               break;
                           }
                       }
                   }

               } else {
                   //分流节点
                   Map<String,XXFlowNodeBO> randomNodeMap=new HashMap<>();
                   Map<String,FlowResultEnum> branchResultMap=new HashMap<>();

                   //每次都执行
                   for(XXFlowNodeBO branchNode : node.getConditionNodes()){
                       if(StrUtil.isEmpty(branchNode.getProperties().getAction().getExeType())
                               || XXActionNodeInterExeTypeEnum.ANE_All.getCode().equals(branchNode.getProperties().getAction().getExeType())){
                           FlowResultEnum branchResult =  this.handle(context,branchNode);
                           branchResultMap.put(branchNode.getNodeId(),branchResult);
                       }else {
                           randomNodeMap.put(branchNode.getNodeId(),branchNode);
                       }
                   }

                   //随机分配执行
                   XXFlowNodeBO  randomNode= getRandomNode(context,node.getNodeId(),randomNodeMap);
                   if(randomNode!=null){
                       FlowResultEnum branchResult =  this.handle(context,randomNode);
                       branchResultMap.put(randomNode.getNodeId(),branchResult);
                   }

                   //分流节点全部执行后(任何一个节点通过就通过)
                   for(String branchNodeId:branchResultMap.keySet()){
                       FlowResultEnum resultEnum=branchResultMap.get(branchNodeId);
                       if(FlowResultEnum.End==resultEnum || FlowResultEnum.Pass==resultEnum){
                           branchFlag=true;
                           break;
                       }
                   }
               }

               //执行分支判断关系决定是否执行下一个
               if(branchFlag==false){
                   return FlowResultEnum.Kill;
               }
           }

           if(node.getChildNode()==null){
               return FlowResultEnum.End;
           }

           return this.handle(context,node.getChildNode());
    }


    /**
     * 获取分流节点随机执行的节点
     * */
    public static XXFlowNodeBO getRandomNode(XXRuleContext context ,String mainNodeId, Map<String,XXFlowNodeBO> randomNodeMap){
        if(MapUtil.isEmpty(randomNodeMap)){
            return null;
        }
        Map<String,Integer> nodeNumMap=new HashMap<>();
        randomNodeMap.forEach((id,node)->{
            nodeNumMap.put(node.getNodeId(),node.getProperties().getAction().getExeRatio());
        });
        String nodeId;
        if(context.getCallNum()!=null){
             nodeId= XXWeightedRoundRobin.selectNodeId(context.getCallNum().longValue(),context.getRuleId(),mainNodeId,context.getRuleTimestamp(),nodeNumMap);
        }else {
             nodeId= XXWeightedRoundRobin.selectNodeId(context.getRuleId(),mainNodeId,context.getRuleTimestamp(),nodeNumMap);
        }
        return randomNodeMap.get(nodeId);
    }


    /**
     * 判断是否条件节点
     * */
    public static ExeConditionParamBO isConditionNode(List<XXFlowNodeBO> nodeList){
        XXFlowNodeBO node = nodeList.get(0);
        if(node.getIsInterflow()!=null && node.getIsInterflow()==true){
            return new ExeConditionParamBO(false,"");
        }else {
            return new ExeConditionParamBO(true,node.getProperties().getConditionExeType()) ;
        }
    }


    /**
     *  节点执行
     * */
    private boolean nodeProcess(XXRuleContext context ,XXFlowNodeBO node){

        if(RuleNodeTypeEnum.RNT_Condition.getCode().equals(node.getProperties().getRuleType())){
            boolean flag = ConditionExecutor.processExecute(context,
                    node.getNodeId(),
                    node.getProperties().getTitle(),
                    node.getType(),
                    node.getProperties().getRuleType(),
                    node.getProperties().getCondition());
            return flag;
        }
//        else if(RuleNodeTypeEnum.RNT_ConditionFn.getCode().equals(node.getProperties().getRuleType())){
//            boolean  flag = ConditionExecutor.processExecute(context,
//                    node.getNodeId(),
//                    node.getProperties().getTitle(),
//                    node.getType(),
//                    node.getProperties().getRuleType(),
//                    node.getProperties().getCondition()
//            );
//            return flag;
//
//            throw new BusinessException(BusinessBaseErrorEnum.Run_Rule_ERROR,
//                    StrUtil.format("不支持的节点执行，节点编号:{} ,节点名称:{}",node.getNodeId(),node.getProperties().getTitle())
//            );
//        }
        else if(RuleNodeTypeEnum.RNT_Action.getCode().equals(node.getProperties().getRuleType())){
            boolean  flag = MethodExecutor.processExecute(context,
                    node.getNodeId(),
                    node.getProperties().getTitle(),
                    node.getType(),
                    node.getProperties().getRuleType(),
                    node.getProperties().getAction());
            //todo:
            return flag;
        }else {
            throw new BusinessException(BusinessBaseErrorEnum.Run_Rule_ERROR,
                    StrUtil.format("不支持的节点执行，节点编号:{} ,节点名称:{}",node.getNodeId(),node.getProperties().getTitle())
            );
        }
    }



}
