package com.xxh.cloud.rule.flow.variable;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.rule.core.constant.RuleNodeTypeEnum;
import com.xxh.cloud.rule.core.context.XXRuleContext;
import com.xxh.cloud.rule.core.entity.flow.XXFlowNodeBO;
import com.xxh.cloud.rule.core.executor.condition.ConditionExecutor;
import com.xxh.cloud.rule.core.executor.condition.ConditionVariable;
import com.xxh.cloud.rule.core.executor.method.MethodExecutor;
import com.xxh.cloud.rule.core.executor.method.MethodVariable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 规则流:获取引用参数-处理
 * */
public class XXRuleFlowVariableHandler {


    private static final Logger logger = LoggerFactory.getLogger(XXRuleFlowVariableHandler.class);

    public void handle(XXRuleContext context, XXFlowNodeBO node){
        logger.debug("++++++解析规则使用的变量");
        //执行本节点
        this.nodeProcess(context,node);
        //执行合流和条件节点
        if(CollUtil.isNotEmpty(node.getConditionNodes())){
            for(XXFlowNodeBO branchNode : node.getConditionNodes()){
                handle(context,branchNode);
            }
        }
        //无子节点,返回
        if(node.getChildNode()==null){
            return;
        }
        //执行子节点
        handle(context,node.getChildNode());
    }


    private void nodeProcess(XXRuleContext context ,XXFlowNodeBO node){
        if(RuleNodeTypeEnum.RNT_Condition.getCode().equals(node.getProperties().getRuleType())){
            logger.debug("++++++++++获取参数，处理属性条件配置");
            ConditionVariable.processVariable(context,node.getNodeId(),
                    node.getProperties().getTitle(),
                    node.getType(),
                    node.getProperties().getRuleType(),
                    node.getProperties().getCondition());
            System.out.println("+++++++++++++条件属性++++++++++++++++"+node.getProperties().getTitle());
        }else if(RuleNodeTypeEnum.RNT_Action.getCode().equals(node.getProperties().getRuleType())){
            logger.debug("++++++++++获取参数，处理动作方法配置");
            MethodVariable.processVariable(context,node.getNodeId(),
                    node.getProperties().getTitle(),
                    node.getType(),
                    node.getProperties().getRuleType(),
                    node.getProperties().getAction());
            System.out.println("+++++++++++++动作方法++++++++++++++++"+node.getProperties().getTitle());
        }
        else {
            throw new BusinessException(BusinessBaseErrorEnum.Run_Rule_ERROR,
                    StrUtil.format("不支持的节点执行，节点编号:{} ,节点名称:{}",node.getNodeId(),node.getProperties().getTitle())
            );
        }

    }

}
