package com.xxh.cloud.rule.flow.workflow;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.utils.ConsoleUtils;
import com.xxh.cloud.rule.core.constant.RuleNodeTypeEnum;
import com.xxh.cloud.rule.core.constant.XXActionNodeInterExeTypeEnum;
import com.xxh.cloud.rule.core.constant.XXConditionNodeExeRelationTypeEnum;
import com.xxh.cloud.rule.core.context.XXRuleContext;
import com.xxh.cloud.rule.core.entity.flow.XXFlowNodeBO;
import com.xxh.cloud.rule.core.executor.condition.ConditionExecutor;
import com.xxh.cloud.rule.core.executor.workflow.WorkFlowExecutor;
import com.xxh.cloud.rule.core.constant.FlowResultEnum;
import com.xxh.cloud.rule.flow.executor.XXRuleFlowHandler;
import com.xxh.cloud.rule.flow.executor.bean.ExeConditionParamBO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.Map;

public class XXWorkFlowHandler {



    private static final Logger logger = LoggerFactory.getLogger(XXRuleFlowHandler.class);



    /**
     * 获取当前节点的配置
     * */
    public XXFlowNodeBO getWorkFlowNode(XXFlowNodeBO node,String currentNodeId){
        //ConsoleUtils.log("++++++解析规则使用的变量,"+node.getNodeId());

        //执行本节点
        if(currentNodeId.equals(node.getNodeId())){
            return node;
        }

        //执行合流和条件节点
        if(CollUtil.isNotEmpty(node.getConditionNodes())){
            for(XXFlowNodeBO branchNode : node.getConditionNodes()){
                XXFlowNodeBO nodeBO = getWorkFlowNode(branchNode,currentNodeId);
                if(nodeBO!=null){
                    return nodeBO;
                }
            }
        }

        //无子节点,返回
        if(node.getChildNode()==null){
            return null;
        }

        //执行子节点
        return getWorkFlowNode(node.getChildNode(),currentNodeId);
    }



    /**
     * 验证是否执行下一个节点
     * */
    public XXFlowNodeBO getNextMainNode(XXFlowNodeBO nodeConfig,String currentNodeId){
        if(nodeConfig.getNodeId().equals(currentNodeId)){
            return nodeConfig;
        }
        if(nodeConfig.getChildNode()==null){
            return null;
        }
        return getNextMainNode(nodeConfig.getChildNode(),currentNodeId);
    }


    /**
     * 流程开始
     * */
    public FlowResultEnum workflowStart(XXRuleContext context, XXFlowNodeBO node){
       return this.workFlowExecute(context,node,"-9999",true);
    }

    /**
     * 流程执行
     * */
    public FlowResultEnum workflowNext(XXRuleContext context, XXFlowNodeBO node,String currentNodeId){
        return this.workFlowExecute(context,node,currentNodeId,false);
    }

    /**
     * 执行
     * */
    public FlowResultEnum workFlowExecute(XXRuleContext context, XXFlowNodeBO node,String currentNodeId,boolean isStart){
        //获取开始标记
        if(isStart==false){
            if(currentNodeId.equals(node.getNodeId())){
                isStart=true;
                //System.out.println("===================== 开始执行节点 : "+ node.getNodeId() +" , isStart = "+isStart );
            }else {
                if(CollUtil.isNotEmpty(node.getConditionNodes())){
                    for(XXFlowNodeBO branchNode : node.getConditionNodes()){
                        workFlowExecute(context,branchNode,currentNodeId,isStart);
                    }
                }
                //无子节点,返回
                if(node.getChildNode()==null){
                    return FlowResultEnum.End;
                }
                return workFlowExecute(context,node.getChildNode(),currentNodeId,isStart);
            }
        }

        //开始标记
        if(isStart==true){
            //System.out.println("************** 开始标记 ******************"+currentNodeId +" : "+ node.getNodeId());
            //开始节点不执行
            if(!currentNodeId.equals(node.getNodeId())){
                boolean nodeFlag= this.nodeProcess(context,node);
                if(nodeFlag==false){
                    return FlowResultEnum.Kill;
                }
            }

            //执行合流和条件节点
            if(CollUtil.isNotEmpty(node.getConditionNodes())){
                ExeConditionParamBO exeConditionParamBO=XXRuleFlowHandler.isConditionNode(node.getConditionNodes());
                boolean branchFlag=false;
                if(exeConditionParamBO.isCondition()){
                    //条件节点(1.任何一个节点通过就通过 2.执行所有节点后,有一个通过则通过 3.执行所有节点后,所有节点通过则通过)
                    //1.任何一个节点通过就通过
                    if(XXConditionNodeExeRelationTypeEnum.ANY.getCode().equals(exeConditionParamBO.getConditionExe())){
                        for(XXFlowNodeBO branchNode : node.getConditionNodes()){
                            FlowResultEnum branchResult =  this.workFlowExecute(context,branchNode,currentNodeId,isStart);
                            ConsoleUtils.log("===节点{}下,条件节点执行结果 : {} -{}",node.getProperties().getTitle(),branchResult.getCode(),branchResult.getName());
                            if(FlowResultEnum.End==branchResult || FlowResultEnum.Pass==branchResult){
                                branchFlag=true;
                                break;
                            }
                        }
                    }

                    //2.执行所有节点后,有一个通过则通过
                    if(XXConditionNodeExeRelationTypeEnum.ALL.getCode().equals(exeConditionParamBO.getConditionExe())){
                        Map<String,FlowResultEnum> branchResultMap=new HashMap<>();
                        for(XXFlowNodeBO branchNode : node.getConditionNodes()){
                            FlowResultEnum branchResult =  this.workFlowExecute(context,branchNode,currentNodeId,isStart);
                            branchResultMap.put(branchNode.getNodeId(),branchResult);
                        }

                        for(String branchNodeId:branchResultMap.keySet()){
                            FlowResultEnum resultEnum=branchResultMap.get(branchNodeId);
                            if(FlowResultEnum.End==resultEnum || FlowResultEnum.Pass==resultEnum){
                                branchFlag=true;
                                break;
                            }
                        }
                    }

                    //3.执行所有节点后,所有节点通过则通过
                    if(XXConditionNodeExeRelationTypeEnum.ALL_Success.getCode().equals(exeConditionParamBO.getConditionExe()) ){
                        Map<String,FlowResultEnum> branchResultMap=new HashMap<>();
                        for(XXFlowNodeBO branchNode : node.getConditionNodes()){
                            FlowResultEnum branchResult =  this.workFlowExecute(context,branchNode,currentNodeId,isStart);
                            branchResultMap.put(branchNode.getNodeId(),branchResult);
                        }
                        branchFlag=true;
                        for(String branchNodeId:branchResultMap.keySet()){
                            FlowResultEnum resultEnum=branchResultMap.get(branchNodeId);
                            if(FlowResultEnum.Kill==resultEnum){
                                branchFlag=false;
                                break;
                            }
                        }
                    }

                } else {
                    //分流节点
                    Map<String,FlowResultEnum> branchResultMap=new HashMap<>();
                    //每次都执行
                    for(XXFlowNodeBO branchNode : node.getConditionNodes()){
                        if(StrUtil.isEmpty(branchNode.getProperties().getAction().getExeType())
                                || XXActionNodeInterExeTypeEnum.ANE_All.getCode().equals(branchNode.getProperties().getAction().getExeType())){

                            FlowResultEnum branchResult =  this.workFlowExecute(context,branchNode,currentNodeId,isStart);
                            branchResultMap.put(branchNode.getNodeId(),branchResult);
                        }
                    }

                    //分流节点全部执行后(任何一个节点通过就通过)
                    for(String branchNodeId:branchResultMap.keySet()){
                        FlowResultEnum resultEnum=branchResultMap.get(branchNodeId);
                        if(FlowResultEnum.End==resultEnum || FlowResultEnum.Pass==resultEnum){
                            branchFlag=true;
                            break;
                        }
                    }

                }

                //执行分支判断关系决定是否执行下一个
                if(branchFlag==false){
                    return FlowResultEnum.Kill;
                }
            }

            //
            if(node.getChildNode()==null){
                return FlowResultEnum.End;
            }

            return this.workFlowExecute(context,node.getChildNode(),currentNodeId,isStart);
        }else {
            ConsoleUtils.log("++++++++++++++++++++++++++ 不能执行");
            //todo:
            return FlowResultEnum.End2;
        }
    }


    /**
     *  节点执行
     * */
    private boolean nodeProcess(XXRuleContext context ,XXFlowNodeBO node){
        if(RuleNodeTypeEnum.RNT_Condition.getCode().equals(node.getProperties().getRuleType())){
            boolean flag = ConditionExecutor.processExecute(context,
                    node.getNodeId(),
                    node.getProperties().getTitle(),
                    node.getType(),
                    node.getProperties().getRuleType(),
                    node.getProperties().getCondition());
            ConsoleUtils.log("===== 流程引擎条件节点执行：执行条件节点:"+node.getContent() + " , 结果 ="  + flag );
            return flag;
        }else if(RuleNodeTypeEnum.RNT_Action.getCode().equals(node.getProperties().getRuleType())){
            //不自动执行:false 自动执行:true
            boolean  flag = WorkFlowExecutor.processWorkFlow(context, node.getNodeId(), node.getType(),
                    node.getProperties().getRuleType(), node.getProperties().getTitle(),
                    node.getContent(),node.getPrevId(), node.getProperties().getAction());
//            ConsoleUtils.log(" ++++++ 执行动作节点:"+node.getContent() + " , 结果 ="  + flag );
            return flag;
        }else {
            throw new BusinessException(BusinessBaseErrorEnum.Run_Rule_ERROR,
                    StrUtil.format("不支持的节点执行，节点编号:{} ,节点名称:{}",node.getNodeId(),node.getProperties().getTitle())
            );
        }
    }


}
